@@ -17,22 +17,18 @@
 #include "note.h"
 #include "accidental.h"
 
-class QPainter;
-
 namespace Ms {
 
 //---------------------------------------------------------
 //   @@ Ambitus
 //---------------------------------------------------------
 
-class Ambitus : public Element {
-      Q_OBJECT
-
+class Ambitus final : public Element {
       NoteHead::Group     _noteHeadGroup;
       NoteHead::Type      _noteHeadType;
       MScore::DirectionH  _dir;
       bool  _hasLine;
-      qreal _lineWidth;                     // in spatium
+      Spatium _lineWidth;
       Accidental  _topAccid, _bottomAccid;
       int   _topPitch, _bottomPitch;
       int   _topTpc, _bottomTpc;
@@ -45,17 +41,20 @@ class Ambitus : public Element {
       void  normalize();
 
    public:
-
       Ambitus(Score* s);
       virtual Ambitus* clone() const override         { return new Ambitus(*this); }
 
+      virtual qreal mag() const override;
+
+      void initFrom(Ambitus* a);
+
       // getters and setters
-      virtual Element::Type type() const override     { return Element::Type::AMBITUS; }
+      virtual ElementType type() const override       { return ElementType::AMBITUS; }
       NoteHead::Group noteHeadGroup() const           { return _noteHeadGroup;}
       NoteHead::Type noteHeadType() const             { return _noteHeadType; }
       MScore::DirectionH direction() const            { return _dir;          }
       bool hasLine() const                            { return _hasLine;      }
-      qreal lineWidth() const                         { return _lineWidth;    }
+      Spatium lineWidth() const                       { return _lineWidth;    }
       int topOctave() const                           { return (_topPitch / 12) - 1; }
       int bottomOctave() const                        { return (_bottomPitch / 12) - 1; }
       int topPitch() const                            { return _topPitch;     }
@@ -67,7 +66,7 @@ class Ambitus : public Element {
       void setNoteHeadType (NoteHead::Type val)       { _noteHeadType  = val; }
       void setDirection    (MScore::DirectionH val)   { _dir = val;           }
       void setHasLine      (bool val)                 { _hasLine = val;       }
-      void setLineWidth    (qreal val)                { _lineWidth = val;     }
+      void setLineWidth    (Spatium val)              { _lineWidth = val;     }
       void setTopPitch     (int val);
       void setBottomPitch  (int val);
       void setTopTpc       (int val);
@@ -86,16 +85,17 @@ class Ambitus : public Element {
       virtual void      read(XmlReader&) override;
       virtual void      scanElements(void* data, void (*func)(void*, Element*), bool all=true) override;
       virtual void      setTrack(int val) override;
-      virtual Space     space() const override;
-      virtual void      write(Xml&) const override;
-      virtual QString   accessibleInfo() override;
+      virtual void      write(XmlWriter&) const override;
+      virtual bool      readProperties(XmlReader&) override;
+      virtual QString   accessibleInfo() const override;
 
       // properties
-      QVariant getProperty(P_ID ) const;
-      bool setProperty(P_ID propertyId, const QVariant&);
-      QVariant propertyDefault(P_ID id) const;
-      virtual Element* nextElement() override;
-      virtual Element* prevElement() override;
+      QVariant getProperty(Pid ) const;
+      bool setProperty(Pid propertyId, const QVariant&);
+      QVariant propertyDefault(Pid id) const;
+
+      virtual Element* nextSegmentElement() override;
+      virtual Element* prevSegmentElement() override;
       };
 
 
