@@ -15,7 +15,7 @@
 
 namespace Ms {
 
-class Xml;
+class XmlWriter;
 class XmlReader;
 
 //---------------------------------------------------------
@@ -35,7 +35,7 @@ class Audio {
       void setData(const QByteArray& ba) { _data = ba;   }
 
       void read(XmlReader&);
-      void write(Xml&) const;
+      void write(XmlWriter&) const;
       };
 
 
