@@ -281,7 +281,7 @@ void HChord::add(const QList<HDegree>& degreeList)
                         }
                   }
             else
-                  qDebug("degree type %d not supported", int(d.type()));
+                  qDebug("degree type %d not supported", static_cast<int>(d.type()));
 
 // qDebug("  HCHord::added  "); print();
             }
@@ -326,7 +326,7 @@ static void readRenderList(QString val, QList<RenderAction>& renderList)
 //   writeRenderList
 //---------------------------------------------------------
 
-static void writeRenderList(Xml& xml, const QList<RenderAction>* al, const QString& name)
+static void writeRenderList(XmlWriter& xml, const QList<RenderAction>* al, const QString& name)
       {
       QString s;
 
@@ -388,7 +388,7 @@ void ChordToken::read(XmlReader& e)
 //  write
 //---------------------------------------------------------
 
-void ChordToken::write(Xml& xml) const
+void ChordToken::write(XmlWriter& xml) const
       {
       QString t = "token";
       switch (tokenClass) {
@@ -1108,14 +1108,14 @@ bool ParsedChord::parse(const QString& s, const ChordList* cl, bool syntaxOnly,
                   if (_xmlDegrees.removeAll(unalt) > 0) {
                         QString alt(d);
                         alt.replace("alt","add");
-                        int i = _xmlDegrees.indexOf(d);
-                        _xmlDegrees.replace(i,alt);
+                        int i1 = _xmlDegrees.indexOf(d);
+                        _xmlDegrees.replace(i1, alt);
                         }
                   }
             }
 
       // construct handle
-      if (!_modifierList.isEmpty()) {
+      if (!_modifierList.empty()) {
             _modifierList.sort();
             _modifiers = "<" + _modifierList.join("><") + ">";
             }
@@ -1147,7 +1147,7 @@ QString ParsedChord::fromXml(const QString& rawKind, const QString& rawKindText,
       {
       QString kind = rawKind;
       QString kindText = rawKindText;
-      bool symbols = (useSymbols == "yes");
+      bool syms = (useSymbols == "yes");
       bool parens = (useParens == "yes");
       bool implied = false;
       bool extend = false;
@@ -1182,7 +1182,7 @@ QString ParsedChord::fromXml(const QString& rawKind, const QString& rawKindText,
             _quality = "augmented";
       else if (kind == "half-diminished") {
             _quality = "half-diminished";
-            if (symbols) {
+            if (syms) {
                   extension = 7;
                   extend = true;
                   }
@@ -1295,7 +1295,7 @@ QString ParsedChord::fromXml(const QString& rawKind, const QString& rawKindText,
       if (kind == "minor-seventh" && _modifierList.size() == 1 && _modifierList.front() == "b5")
             kind = "half-diminished";
       // force parens where necessary)
-      if (!parens && extension == 0 && !_modifierList.isEmpty()) {
+      if (!parens && extension == 0 && !_modifierList.empty()) {
             QString firstMod = _modifierList.front();
             if (firstMod != "" && (firstMod.startsWith('#') || firstMod.startsWith('b')))
                   parens = true;
@@ -1310,7 +1310,7 @@ QString ParsedChord::fromXml(const QString& rawKind, const QString& rawKindText,
             ParsedChord validate;
             validate.parse(kindText, cl, false);
             // kindText should parse to produce same kind, no degrees
-            if (validate._xmlKind != kind || !validate._xmlDegrees.isEmpty())
+            if (validate._xmlKind != kind || !validate._xmlDegrees.empty())
                   kindText = "";
             }
 
@@ -1327,15 +1327,15 @@ QString ParsedChord::fromXml(const QString& rawKind, const QString& rawKindText,
             _name = _extension;
       else {
             if (_quality == "major")
-                  _name = symbols ? "^" : "maj";
+                  _name = syms ? "^" : "maj";
             else if (_quality == "minor")
-                  _name = symbols ? "-" : "m";
+                  _name = syms ? "-" : "m";
             else if (_quality == "augmented")
-                  _name = symbols ? "+" : "aug";
+                  _name = syms ? "+" : "aug";
             else if (_quality == "diminished")
-                  _name = symbols ? "o" : "dim";
+                  _name = syms ? "o" : "dim";
             else if (_quality == "half-diminished")
-                  _name = symbols ? "0" : "m7b5";
+                  _name = syms ? "0" : "m7b5";
             else
                   _name = _quality;
             _name += _extension;
@@ -1378,7 +1378,7 @@ const QList<RenderAction>& ParsedChord::renderList(const ChordList* cl)
       {
       // generate anew on each call,
       // in case chord list has changed since last time
-      if (!_renderList.isEmpty())
+      if (!_renderList.empty())
             _renderList.clear();
       foreach (ChordToken tok, _tokenList) {
             QString n = tok.names.first();
@@ -1387,15 +1387,15 @@ const QList<RenderAction>& ParsedChord::renderList(const ChordList* cl)
             bool found = false;
             // potential definitions for token
             if (cl) {
-                  foreach (ChordToken ct, cl->chordTokenList) {
-                        foreach (QString ctn, ct.names) {
+                  for (ChordToken ct : cl->chordTokenList) {
+                        for (QString ctn : ct.names) {
                               if (ctn == n)
                                     definedTokens += ct;
                               }
                         }
                   }
             // find matching class, fallback on ChordTokenClass::ALL
-            foreach (ChordToken matchingTok, definedTokens) {
+            for (ChordToken matchingTok : definedTokens) {
                   if (tok.tokenClass == matchingTok.tokenClass) {
                         rl = matchingTok.renderList;
                         found = true;
@@ -1476,12 +1476,12 @@ void ChordDescription::complete(ParsedChord* pc, const ChordList* cl)
             // generate parsed chord for its rendering & semantic (xml) info
             pc = &tempPc;
             QString n;
-            if (!names.isEmpty())
+            if (!names.empty())
                   n = names.front();
             pc->parse(n, cl);
             }
       parsedChords.append(*pc);
-      if (renderList.isEmpty() || renderListGenerated) {
+      if (renderList.empty() || renderListGenerated) {
             renderList = pc->renderList(cl);
             renderListGenerated = true;
             }
@@ -1534,7 +1534,7 @@ void ChordDescription::read(XmlReader& e)
 //   write
 //---------------------------------------------------------
 
-void ChordDescription::write(Xml& xml) const
+void ChordDescription::write(XmlWriter& xml) const
       {
       if (generated && !exportOk)
             return;
@@ -1571,6 +1571,8 @@ void ChordList::read(XmlReader& e)
             if (tag == "font") {
                   ChordFont f;
                   f.family = e.attribute("family", "default");
+                  if (f.family == "MuseJazz")
+                        f.family = "MuseJazz Text";
                   f.mag    = 1.0;
                   while (e.readNextStartElement()) {
                         if (e.name() == "sym") {
@@ -1654,13 +1656,13 @@ void ChordList::read(XmlReader& e)
 //   write
 //---------------------------------------------------------
 
-void ChordList::write(Xml& xml) const
+void ChordList::write(XmlWriter& xml) const
       {
       int fontIdx = 0;
-      foreach (ChordFont f, fonts) {
+      for (ChordFont f : fonts) {
             xml.stag(QString("font id=\"%1\" family=\"%2\"").arg(fontIdx).arg(f.family));
             xml.tag("mag", f.mag);
-            foreach(ChordSymbol s, symbols) {
+            for (ChordSymbol s : symbols) {
                   if (s.fontIdx == fontIdx) {
                         if (s.code.isNull())
                               xml.tagE(QString("sym name=\"%1\" value=\"%2\"").arg(s.name).arg(s.value));
@@ -1673,12 +1675,12 @@ void ChordList::write(Xml& xml) const
             }
       foreach (ChordToken t, chordTokenList)
             t.write(xml);
-      if (!renderListRoot.isEmpty())
+      if (!renderListRoot.empty())
             writeRenderList(xml, &renderListRoot, "renderRoot");
-      if (!renderListBase.isEmpty())
+      if (!renderListBase.empty())
             writeRenderList(xml, &renderListBase, "renderBase");
-      foreach(const ChordDescription& d, *this)
-            d.write(xml);
+      for (const ChordDescription& cd : *this)
+            cd.write(xml);
       }
 
 //---------------------------------------------------------
@@ -1722,7 +1724,6 @@ bool ChordList::read(const QString& name)
             return false;
             }
       XmlReader e(&f);
-      docName = f.fileName();
 
       while (e.readNextStartElement()) {
             if (e.name() == "museScore") {
@@ -1753,18 +1754,18 @@ bool ChordList::write(const QString& name) const
       QFile f(info.filePath());
 
       if (!f.open(QIODevice::WriteOnly)) {
-            MScore::lastError = QObject::tr("Open Chord Description\n%1\nfailed: %2").arg(f.fileName()).arg(f.errorString());
+            MScore::lastError = QObject::tr("Open chord description\n%1\nfailed: %2").arg(f.fileName()).arg(f.errorString());
             return false;
             }
 
-      Xml xml(&f);
+      XmlWriter xml(0, &f);
       xml.header();
       xml.stag("museScore version=\"" MSC_VERSION "\"");
 
       write(xml);
       xml.etag();
       if (f.error() != QFile::NoError) {
-            MScore::lastError = QObject::tr("Write Chord Description failed: %1").arg(f.errorString());
+            MScore::lastError = QObject::tr("Write chord description failed: %1").arg(f.errorString());
             }
       return true;
       }
@@ -1779,7 +1780,7 @@ bool ChordList::loaded() const
       // since chords.xml really doesn't load enough to stand alone,
       // we need a way to track when a "real" chord list has been loaded
       // for lack of anything better, key off renderListRoot
-      return !renderListRoot.isEmpty();
+      return !renderListRoot.empty();
       }
 
 //---------------------------------------------------------
@@ -1796,5 +1797,19 @@ void ChordList::unload()
       chordTokenList.clear();
       }
 
+//---------------------------------------------------------
+//   print
+//    only for debugging
+//---------------------------------------------------------
+
+void RenderAction::print() const
+      {
+      static const char* names[] = {
+            "SET", "MOVE", "PUSH", "POP",
+            "NOTE", "ACCIDENTAL"
+            };
+      qDebug("%10s <%s> %f %f", names[int(type)], qPrintable(text), movex, movey);
+      }
+
 
 }
