@@ -21,11 +21,9 @@
 #include "element.h"
 #include "mscore.h"
 
-class QPainter;
-
 namespace Ms {
 
-class Xml;
+class XmlWriter;
 class MuseScoreView;
 class Segment;
 
@@ -39,27 +37,38 @@ static const int NO_CLEF = -1000;
 enum class ClefType : signed char {
       INVALID = -1,
       G = 0,
-      G1,
-      G2,
-      G3,
-      F,
-      F8,
-      F15,
-      F_B,
-      F_C,
+      G15_MB,
+      G8_VB,
+      G8_VA,
+      G15_MA,
+      G8_VB_O,
+      G8_VB_P,
+      G_1,
       C1,
       C2,
       C3,
       C4,
-      TAB,
-      PERC,
       C5,
-      G4,
+      C_19C,
+      C3_F18C,
+      C4_F18C,
+      C3_F20C,
+      C4_F20C,
+      F,
+      F15_MB,
+      F8_VB,
       F_8VA,
       F_15MA,
-      PERC2,            // no longer supported, but kept for compat. with old scores; rendered as PERC
-      TAB2,
-      G5,
+      F_B,
+      F_C,
+      F_F18C,
+      F_19C,
+      PERC,
+      PERC2,
+      TAB,
+      TAB4,
+      TAB_SERIF,
+      TAB4_SERIF,
       MAX
       };
 
@@ -89,10 +98,11 @@ class ClefInfo {
 
       const char* _tag;        ///< comprehensive name for instruments.xml
       const char* _sign;       ///< Name for musicXml.
-      int _line;               ///< Line for musicXml.
+      int _line;               ///< Line for musicXml and for positioning on the staff
       int _octChng;            ///< Octave change for musicXml.
       int _pitchOffset;        ///< Pitch offset for line 0.
       signed char _lines[14];
+      SymId _symId;
       const char* _name;
       StaffGroup _staffGroup;
 
@@ -102,6 +112,7 @@ class ClefInfo {
       static int line(ClefType t)              { return clefTable[int(t)]._line;        }
       static int octChng(ClefType t)           { return clefTable[int(t)]._octChng;     }
       static int pitchOffset(ClefType t)       { return clefTable[int(t)]._pitchOffset; }
+      static SymId symId(ClefType t)           { return clefTable[int(t)]._symId;       }
       static const signed char* lines(ClefType t)     { return clefTable[int(t)]._lines;       }
       static const char* name(ClefType t)      { return clefTable[int(t)]._name;        }
       static StaffGroup staffGroup(ClefType t) { return clefTable[int(t)]._staffGroup;  }
@@ -116,54 +127,34 @@ class ClefInfo {
 //   @P small         bool    small, mid-staff clef (read only, set by layout)
 //---------------------------------------------------------
 
-class Clef : public Element {
-      Q_OBJECT
-      Q_PROPERTY(bool showCourtesy READ showCourtesy WRITE undoSetShowCourtesy)
-      Q_PROPERTY(bool small READ small)
+class Clef final : public Element {
+      SymId symId;
+      bool _showCourtesy = true;
+      bool _small = false;
 
-      QList<Element*> elements;
-      bool _showCourtesy;
-      bool _showPreviousClef;       // show clef type at position tick-1
-                                    // used for first clef on staff immediatly followed
-                                    // by a different clef at same tick position
-      bool _small;
-
-      ClefTypeList _clefTypes;
-
-      ClefType    curClefType;      // cached value of clef type (for re-laying out)
-      int         curLines;         // cached value of staff nm. of lines  ( " )
-      qreal       curLineDist;      // cached value of staff line distance ( " )
-      void        layout1();        // lays the element out, using cached values
+      ClefTypeList _clefTypes { ClefType::INVALID };
 
    public:
       Clef(Score*);
-      Clef(const Clef&);
-      ~Clef();
       virtual Clef* clone() const        { return new Clef(*this); }
-      virtual Element::Type type() const { return Element::Type::CLEF; }
-      virtual void setSelected(bool f);
+      virtual ElementType type() const { return ElementType::CLEF; }
       virtual qreal mag() const;
 
       Segment* segment() const           { return (Segment*)parent(); }
       Measure* measure() const           { return (Measure*)parent()->parent(); }
 
-      virtual bool acceptDrop(const DropData&) const override;
-      virtual Element* drop(const DropData&);
+      virtual bool acceptDrop(EditData&) const override;
+      virtual Element* drop(EditData&);
       virtual void layout();
       virtual void draw(QPainter*) const;
       virtual void read(XmlReader&);
-      virtual void write(Xml&) const;
+      virtual void write(XmlWriter&) const;
 
-      virtual bool isEditable() const                    { return false; }
-
-      virtual void addElement(Element* e, qreal x, qreal y);
-      virtual Space space() const      { return Space(0.0, bbox().x() * 2.0 + width()); }
+      virtual bool isEditable() const  { return false; }
 
       bool small() const               { return _small; }
       void setSmall(bool val);
 
-      int tick() const;
-
       bool showCourtesy() const        { return _showCourtesy; }
       void setShowCourtesy(bool v)     { _showCourtesy = v; }
       void undoSetShowCourtesy(bool v);
@@ -182,15 +173,16 @@ class Clef : public Element {
       void setConcertClef(ClefType val);
       void setTransposingClef(ClefType val);
       void setClefType(const ClefTypeList& ctl) { _clefTypes = ctl; }
-      virtual void spatiumChanged(qreal oldValue, qreal newValue);
+      virtual void spatiumChanged(qreal oldValue, qreal newValue) override;
 
-      QVariant getProperty(P_ID propertyId) const;
-      bool setProperty(P_ID propertyId, const QVariant&);
-      QVariant propertyDefault(P_ID id) const;
+      QVariant getProperty(Pid propertyId) const;
+      bool setProperty(Pid propertyId, const QVariant&);
+      QVariant propertyDefault(Pid id) const;
 
-      virtual Element* nextElement() override;
-      virtual Element* prevElement() override;
-      QString accessibleInfo() override;
+      virtual Element* nextSegmentElement() override;
+      virtual Element* prevSegmentElement() override;
+      QString accessibleInfo() const override;
+      void clear();
       };
 
 }     // namespace Ms
