@@ -41,18 +41,19 @@ class TDuration {
 
    public:
       TDuration() : _val(DurationType::V_INVALID), _dots(0) {}
-      TDuration(const Fraction& l, bool truncate = false, int maxDots = MAX_DOTS, DurationType maxType = DurationType::V_LONG);
+      TDuration(const Fraction& l, bool truncate = false, int maxDots = 4, DurationType maxType = DurationType::V_LONG);
       TDuration(const QString&);
       TDuration(DurationType t) : _val(t), _dots(0) {}
 
       DurationType type() const             { return _val; }
       bool isValid() const                  { return _val != DurationType::V_INVALID; }
       bool isZero() const                   { return _val == DurationType::V_ZERO; }
+      bool isMeasure() const                { return _val == DurationType::V_MEASURE; }
       void setVal(int tick);
       void setType(DurationType t);
       void setType(const QString&);
 
-      int ticks() const;
+      Fraction ticks() const;
       bool operator==(const TDuration& t) const    { return t._val == _val && t._dots == _dots; }
       bool operator==(const DurationType& t) const { return t == _val; }
       bool operator!=(const TDuration& t) const    { return t._val != _val || t._dots != _dots; }
@@ -70,7 +71,12 @@ class TDuration {
       int hooks() const;
       bool hasStem() const;
       TDuration shift(int nSteps) const                              { TDuration d(type()); d.shiftType(nSteps); return d; } // dots are not retained
-      TDuration shiftRetainDots(int nSteps, bool stepDotted = false) { TDuration d(type()); d.setDots(_dots); d.shiftType(nSteps, stepDotted); return d; }
+      TDuration shiftRetainDots(int nSteps, bool stepDotted = false) {
+            TDuration d(type());
+            d.setDots(_dots);
+            d.shiftType(nSteps, stepDotted);
+            return d;
+            }
       int dots() const    { return _dots; }
       void setDots(int v);
       Fraction fraction() const;
@@ -78,18 +84,18 @@ class TDuration {
       static bool isValid(Fraction f);
       };
 
-QList<TDuration> toDurationList(Fraction l, bool useDots, int maxDots = MAX_DOTS, bool printRestRemains = true);
-QList<TDuration> toRhythmicDurationList(const Fraction& l, bool isRest, int rtickStart, const TimeSigFrac& nominal, Measure* msr, int maxDots);
+std::vector<TDuration> toDurationList(Fraction l, bool useDots, int maxDots = 4, bool printRestRemains = true);
+std::vector<TDuration> toRhythmicDurationList(const Fraction& l, bool isRest, Fraction rtickStart, const TimeSigFrac& nominal, Measure* msr, int maxDots);
 
 bool forceRhythmicSplit(bool isRest, BeatType startBeat, BeatType endBeat, int beatsCrossed, BeatType strongestBeatCrossed, const TimeSigFrac& nominal);
 bool forceRhythmicSplitSimple(bool isRest, BeatType startBeat, BeatType endBeat, int beatsCrossed, BeatType strongestBeatCrossed);
 bool forceRhythmicSplitCompound(bool isRest, BeatType startBeat, BeatType endBeat, int beatsCrossed, BeatType strongestBeatCrossed);
 
-void populateRhythmicList(QList<TDuration>* dList, const Fraction& l, bool isRest, int rtickStart, const TimeSigFrac& nominal, int maxDots);
-void splitCompoundBeatsForList(QList<TDuration> *dList, const Fraction& l, bool isRest, int rtickStart, const TimeSigFrac& nominal, int maxDots);
+void populateRhythmicList(std::vector<TDuration>* dList, const Fraction& l, bool isRest, const Fraction& rtickStart, const TimeSigFrac& nominal, int maxDots);
+void splitCompoundBeatsForList(std::vector<TDuration>* dList, const Fraction& l, bool isRest, const Fraction& rtickStart, const TimeSigFrac& nominal, int maxDots);
 }     // namespace Ms
 
-Q_DECLARE_METATYPE(Ms::TDuration)
+Q_DECLARE_METATYPE(Ms::TDuration);
 
 #endif
 
