@@ -17,6 +17,7 @@
 #include "measure.h"
 #include "segment.h"
 #include "score.h"
+#include "system.h"
 #include "undo.h"
 #include "xml.h"
 
@@ -24,18 +25,18 @@ namespace Ms {
 
 const char* keyNames[] = {
       QT_TRANSLATE_NOOP("MuseScore", "G major, E minor"),
-      QT_TRANSLATE_NOOP("MuseScore", "Cb major, Ab minor"),
+      QT_TRANSLATE_NOOP("MuseScore", "C♭ major, A♭ minor"),
       QT_TRANSLATE_NOOP("MuseScore", "D major, B minor"),
-      QT_TRANSLATE_NOOP("MuseScore", "Gb major, Eb minor"),
-      QT_TRANSLATE_NOOP("MuseScore", "A major, F# minor"),
-      QT_TRANSLATE_NOOP("MuseScore", "Db major, Bb minor"),
-      QT_TRANSLATE_NOOP("MuseScore", "E major, C# minor"),
-      QT_TRANSLATE_NOOP("MuseScore", "Ab major, F minor"),
-      QT_TRANSLATE_NOOP("MuseScore", "B major, G# minor"),
-      QT_TRANSLATE_NOOP("MuseScore", "Eb major, C minor"),
-      QT_TRANSLATE_NOOP("MuseScore", "F# major, D# minor"),
-      QT_TRANSLATE_NOOP("MuseScore", "Bb major, G minor"),
-      QT_TRANSLATE_NOOP("MuseScore", "C# major, A# minor"),
+      QT_TRANSLATE_NOOP("MuseScore", "G♭ major, E♭ minor"),
+      QT_TRANSLATE_NOOP("MuseScore", "A major, F♯ minor"),
+      QT_TRANSLATE_NOOP("MuseScore", "D♭ major, B♭ minor"),
+      QT_TRANSLATE_NOOP("MuseScore", "E major, C♯ minor"),
+      QT_TRANSLATE_NOOP("MuseScore", "A♭ major, F minor"),
+      QT_TRANSLATE_NOOP("MuseScore", "B major, G♯ minor"),
+      QT_TRANSLATE_NOOP("MuseScore", "E♭ major, C minor"),
+      QT_TRANSLATE_NOOP("MuseScore", "F♯ major, D♯ minor"),
+      QT_TRANSLATE_NOOP("MuseScore", "B♭ major, G minor"),
+      QT_TRANSLATE_NOOP("MuseScore", "C♯ major, A♯ minor"),
       QT_TRANSLATE_NOOP("MuseScore", "F major, D minor"),
       QT_TRANSLATE_NOOP("MuseScore", "C major, A minor"),
       QT_TRANSLATE_NOOP("MuseScore", "Open/Atonal")
@@ -46,9 +47,8 @@ const char* keyNames[] = {
 //---------------------------------------------------------
 
 KeySig::KeySig(Score* s)
-  : Element(s)
+  : Element(s, ElementFlag::ON_STAFF)
       {
-      setFlags(ElementFlag::SELECTABLE | ElementFlag::ON_STAFF);
       _showCourtesy = true;
       _hideNaturals = false;
       }
@@ -67,7 +67,7 @@ KeySig::KeySig(const KeySig& k)
 
 qreal KeySig::mag() const
       {
-      return staff() ? staff()->mag() : 1.0;
+      return staff() ? staff()->mag(tick()) : 1.0;
       }
 
 //---------------------------------------------------------
@@ -76,9 +76,10 @@ qreal KeySig::mag() const
 
 void KeySig::addLayout(SymId sym, qreal x, int line)
       {
+      qreal stepDistance = staff() ? staff()->lineDistance(tick()) * 0.5 : 0.5;
       KeySym ks;
       ks.sym    = sym;
-      ks.spos   = QPointF(x, qreal(line) * .5);
+      ks.spos   = QPointF(x, qreal(line) * stepDistance);
       _sig.keySymbols().append(ks);
       }
 
@@ -108,14 +109,16 @@ void KeySig::layout()
       if (staff()) {
             // Look for a clef before the key signature at the same tick
             Clef* c = nullptr;
-            for (Segment* seg = segment()->prev1(); !c && seg && seg->tick() == segment()->tick(); seg = seg->prev1())
-                  if (seg->segmentType() == Segment::Type::Clef)
-                        c = static_cast<Clef*>(seg->element(track()));
+            if (segment()) {
+                  for (Segment* seg = segment()->prev1(); !c && seg && seg->tick() == tick(); seg = seg->prev1())
+                        if (seg->isClefType() || seg->isHeaderClefType())
+                              c = toClef(seg->element(track()));
+                  }
             if (c)
                   clef = c->clefType();
             else
                   // no clef found, so get the clef type from the clefs list, using the previous tick
-                  clef = staff()->clef(segment()->tick() - 1);
+                  clef = staff()->clef(tick() - Fraction::fromTicks(1));
             }
 
       int accidentals = 0, naturals = 0;
@@ -139,32 +142,31 @@ void KeySig::layout()
       // AND style says they are not off
       // OR key sig is CMaj/Amin (in which case they are always shown)
 
-      Measure* m           = measure();
-      Measure* prevMeasure = m ? m->prevMeasureMM() : nullptr;
+      bool naturalsOn = false;
+      Measure* prevMeasure = measure() ? measure()->prevMeasure() : 0;
 
-      // display of naturals defaults according to style
-      bool naturalsOn = score()->styleI(StyleIdx::keySigNaturals) != int(KeySigNatural::NONE) || t1 == 0;
-      // suppress if we are hiding all naturals
-      if (_hideNaturals)
+      // If we're not force hiding naturals (Continuous panel), use score style settings
+      if (!_hideNaturals) {
+            const bool newSection = (!segment()
+               || (segment()->rtick().isZero() && (!prevMeasure || prevMeasure->sectionBreak()))
+               );
+            naturalsOn = !newSection && (score()->styleI(Sid::keySigNaturals) != int(KeySigNatural::NONE) || (t1 == 0));
+            }
+
+
+      // Don't repeat naturals if shown in courtesy
+      if (measure() && measure()->system() && measure() == measure()->system()->firstMeasure()
+          && prevMeasure && prevMeasure->findSegment(SegmentType::KeySigAnnounce, tick())
+          && !segment()->isKeySigAnnounceType())
             naturalsOn = false;
-      // suppress after section break
-      else if (prevMeasure && prevMeasure->isFinalMeasureOfSection())
-            naturalsOn = false;
-      // suppress after courtesy keysig
-      // this may or may not have actually been generated yet,
-      // and even if it has been, it might be removed later,
-      // so instead of checking for existence of courtesy keysig,
-      // check instead for the conditions that would normally trigger it
-      else if (score()->styleB(StyleIdx::genCourtesyKeysig) && showCourtesy() && m && prevMeasure && m->system() != prevMeasure->system())
-            naturalsOn = false;
-      // suppress if no staff
-      else if (track() == -1)
+      if (track() == -1)
             naturalsOn = false;
 
       int coffset = 0;
       Key t2      = Key::C;
       if (naturalsOn) {
-            t2 = staff()->key(segment()->tick() - 1);
+            if (staff())
+                  t2 = staff()->key(tick() - Fraction(1, 480*4));
             if (t2 == Key::C)
                   naturalsOn = false;
             else {
@@ -191,9 +193,10 @@ void KeySig::layout()
 
       // naturals should go BEFORE accidentals if style says so
       // OR going from sharps to flats or vice versa (i.e. t1 & t2 have opposite signs)
+
       bool prefixNaturals =
             naturalsOn
-            && (score()->styleI(StyleIdx::keySigNaturals) == int(KeySigNatural::BEFORE) || t1 * int(t2) < 0);
+            && (score()->styleI(Sid::keySigNaturals) == int(KeySigNatural::BEFORE) || t1 * int(t2) < 0);
 
       // naturals should go AFTER accidentals if they should not go before!
       bool suffixNaturals = naturalsOn && !prefixNaturals;
@@ -217,31 +220,31 @@ void KeySig::layout()
 
       switch(t1) {
             case 7:  addLayout(SymId::accidentalSharp, xo + 6.0 * sspread, lines[6]);
-                  // fall through
+                     // fall through
             case 6:  addLayout(SymId::accidentalSharp, xo + 5.0 * sspread, lines[5]);
-                  // fall through
+                     // fall through
             case 5:  addLayout(SymId::accidentalSharp, xo + 4.0 * sspread, lines[4]);
-                  // fall through
+                     // fall through
             case 4:  addLayout(SymId::accidentalSharp, xo + 3.0 * sspread, lines[3]);
-                  // fall through
+                     // fall through
             case 3:  addLayout(SymId::accidentalSharp, xo + 2.0 * sspread, lines[2]);
-                  // fall through
+                     // fall through
             case 2:  addLayout(SymId::accidentalSharp, xo + 1.0 * sspread, lines[1]);
-                  // fall through
+                     // fall through
             case 1:  addLayout(SymId::accidentalSharp, xo,                 lines[0]);
                      break;
             case -7: addLayout(SymId::accidentalFlat, xo + 6.0 * fspread, lines[13]);
-                  // fall through
+                     // fall through
             case -6: addLayout(SymId::accidentalFlat, xo + 5.0 * fspread, lines[12]);
-                  // fall through
+                     // fall through
             case -5: addLayout(SymId::accidentalFlat, xo + 4.0 * fspread, lines[11]);
-                  // fall through
+                     // fall through
             case -4: addLayout(SymId::accidentalFlat, xo + 3.0 * fspread, lines[10]);
-                  // fall through
+                     // fall through
             case -3: addLayout(SymId::accidentalFlat, xo + 2.0 * fspread, lines[9]);
-                  // fall through
+                     // fall through
             case -2: addLayout(SymId::accidentalFlat, xo + 1.0 * fspread, lines[8]);
-                  // fall through
+                     // fall through
             case -1: addLayout(SymId::accidentalFlat, xo,                 lines[7]);
             case 0:
                   break;
@@ -273,6 +276,26 @@ void KeySig::layout()
             }
       }
 
+//---------------------------------------------------------
+//   shape
+//---------------------------------------------------------
+
+Shape KeySig::shape() const
+      {
+      QRectF box(bbox());
+      const Staff* st = staff();
+      if (st && addToSkyline()) {
+            // Extend key signature shape up and down to
+            // the first ledger line height to ensure that
+            // no notes will be too close to the keysig.
+            const qreal sp = spatium();
+            const qreal y = pos().y();
+            box.setTop(std::min(-sp - y, box.top()));
+            box.setBottom(std::max(st->height() - y + sp, box.bottom()));
+            }
+      return Shape(box);
+      }
+
 //---------------------------------------------------------
 //   set
 //---------------------------------------------------------
@@ -282,7 +305,7 @@ void KeySig::draw(QPainter* p) const
       p->setPen(curColor());
       for (const KeySym& ks: _sig.keySymbols())
             drawSymbol(ks.sym, p, QPointF(ks.pos.x(), ks.pos.y()));
-      if (!parent() && (isAtonal() || isCustom()) && _sig.keySymbols().isEmpty()) {
+      if (!parent() && (isAtonal() || isCustom()) && _sig.keySymbols().empty()) {
             // empty custom or atonal key signature - draw something for palette
             p->setPen(Qt::gray);
             drawSymbol(SymId::timeSigX, p, QPointF(symWidth(SymId::timeSigX) * -0.5, 2.0 * spatium()));
@@ -293,19 +316,19 @@ void KeySig::draw(QPainter* p) const
 //   acceptDrop
 //---------------------------------------------------------
 
-bool KeySig::acceptDrop(const DropData& data) const
+bool KeySig::acceptDrop(EditData& data) const
       {
-      return data.element->type() == Element::Type::KEYSIG;
+      return data.dropElement->type() == ElementType::KEYSIG;
       }
 
 //---------------------------------------------------------
 //   drop
 //---------------------------------------------------------
 
-Element* KeySig::drop(const DropData& data)
+Element* KeySig::drop(EditData& data)
       {
-      KeySig* ks = static_cast<KeySig*>(data.element);
-      if (ks->type() != Element::Type::KEYSIG) {
+      KeySig* ks = toKeySig(data.dropElement);
+      if (ks->type() != ElementType::KEYSIG) {
             delete ks;
             return 0;
             }
@@ -318,7 +341,7 @@ Element* KeySig::drop(const DropData& data)
             }
       else {
             // apply to all staves:
-            foreach(Staff* s, score()->rootScore()->staves())
+            foreach(Staff* s, score()->masterScore()->staves())
                   score()->undoChangeKeySig(s, tick(), k);
             }
       return this;
@@ -335,22 +358,13 @@ void KeySig::setKey(Key key)
       setKeySigEvent(e);
       }
 
-//---------------------------------------------------------
-//   space
-//---------------------------------------------------------
-
-Space KeySig::space() const
-      {
-      return Space(point(score()->styleS(StyleIdx::keysigLeftMargin)), width());
-      }
-
 //---------------------------------------------------------
 //   write
 //---------------------------------------------------------
 
-void KeySig::write(Xml& xml) const
+void KeySig::write(XmlWriter& xml) const
       {
-      xml.stag(name());
+      xml.stag(this);
       Element::writeProperties(xml);
       if (_sig.isAtonal()) {
             xml.tag("custom", 1);
@@ -394,8 +408,8 @@ void KeySig::read(XmlReader& e)
             if (tag == "KeySym") {
                   KeySym ks;
                   while (e.readNextStartElement()) {
-                        const QStringRef& tag(e.name());
-                        if (tag == "sym") {
+                        const QStringRef& t(e.name());
+                        if (t == "sym") {
                               QString val(e.readElementText());
                               bool valid;
                               SymId id = SymId(val.toInt(&valid));
@@ -409,7 +423,7 @@ void KeySig::read(XmlReader& e)
                                     }
                               ks.sym = id;
                               }
-                        else if (tag == "pos")
+                        else if (t == "pos")
                               ks.spos = e.readPoint();
                         else
                               e.unknown();
@@ -447,7 +461,7 @@ void KeySig::read(XmlReader& e)
       // for backward compatibility
       if (!_sig.isValid())
             _sig.initFromSubtype(subtype);
-      if (_sig.custom() && _sig.keySymbols().isEmpty())
+      if (_sig.custom() && _sig.keySymbols().empty())
             _sig.setMode(KeyMode::NONE);
       }
 
@@ -504,6 +518,18 @@ bool KeySig::operator==(const KeySig& k) const
       return _sig == k._sig;
       }
 
+//---------------------------------------------------------
+//   isChange
+//---------------------------------------------------------
+
+bool KeySig::isChange() const
+      {
+      if (!staff())
+            return false;
+      Fraction keyTick = tick();
+      return staff()->currentKeyTick(keyTick) == keyTick;
+      }
+
 //---------------------------------------------------------
 //   changeKeySigEvent
 //---------------------------------------------------------
@@ -515,32 +541,24 @@ void KeySig::changeKeySigEvent(const KeySigEvent& t)
       setKeySigEvent(t);
       }
 
-//---------------------------------------------------------
-//   tick
-//---------------------------------------------------------
-
-int KeySig::tick() const
-      {
-      return segment() ? segment()->tick() : 0;
-      }
-
 //---------------------------------------------------------
 //   undoSetShowCourtesy
 //---------------------------------------------------------
 
 void KeySig::undoSetShowCourtesy(bool v)
       {
-      score()->undoChangeProperty(this, P_ID::SHOW_COURTESY, v);
+      undoChangeProperty(Pid::SHOW_COURTESY, v);
       }
 
 //---------------------------------------------------------
 //   getProperty
 //---------------------------------------------------------
 
-QVariant KeySig::getProperty(P_ID propertyId) const
+QVariant KeySig::getProperty(Pid propertyId) const
       {
-      switch(propertyId) {
-            case P_ID::SHOW_COURTESY: return int(showCourtesy());
+      switch (propertyId) {
+            case Pid::KEY:           return int(key());
+            case Pid::SHOW_COURTESY: return int(showCourtesy());
             default:
                   return Element::getProperty(propertyId);
             }
@@ -550,10 +568,15 @@ QVariant KeySig::getProperty(P_ID propertyId) const
 //   setProperty
 //---------------------------------------------------------
 
-bool KeySig::setProperty(P_ID propertyId, const QVariant& v)
+bool KeySig::setProperty(Pid propertyId, const QVariant& v)
       {
-      switch(propertyId) {
-            case P_ID::SHOW_COURTESY:
+      switch (propertyId) {
+            case Pid::KEY:
+                  if (generated())
+                        return false;
+                  setKey(Key(v.toInt()));
+                  break;
+            case Pid::SHOW_COURTESY:
                   if (generated())
                         return false;
                   setShowCourtesy(v.toBool());
@@ -563,7 +586,7 @@ bool KeySig::setProperty(P_ID propertyId, const QVariant& v)
                         return false;
                   break;
             }
-      score()->setLayoutAll(true);
+      score()->setLayoutAll();
       setGenerated(false);
       return true;
       }
@@ -572,29 +595,30 @@ bool KeySig::setProperty(P_ID propertyId, const QVariant& v)
 //   propertyDefault
 //---------------------------------------------------------
 
-QVariant KeySig::propertyDefault(P_ID id) const
+QVariant KeySig::propertyDefault(Pid id) const
       {
-      switch(id) {
-            case P_ID::SHOW_COURTESY:     return true;
+      switch (id) {
+            case Pid::KEY:               return int(Key::INVALID);
+            case Pid::SHOW_COURTESY:     return true;
             default:
                   return Element::propertyDefault(id);
             }
       }
 
 //---------------------------------------------------------
-//   nextElement
+//   nextSegmentElement
 //---------------------------------------------------------
 
-Element* KeySig::nextElement()
+Element* KeySig::nextSegmentElement()
       {
       return segment()->firstInNextSegments(staffIdx());
       }
 
 //---------------------------------------------------------
-//   prevElement
+//   prevSegmentElement
 //---------------------------------------------------------
 
-Element* KeySig::prevElement()
+Element* KeySig::prevSegmentElement()
       {
       return segment()->lastInPrevSegments(staffIdx());
       }
@@ -603,13 +627,13 @@ Element* KeySig::prevElement()
 //   accessibleInfo
 //---------------------------------------------------------
 
-QString KeySig::accessibleInfo()
+QString KeySig::accessibleInfo() const
       {
       QString keySigType;
       if (isAtonal())
             return QString("%1: %2").arg(Element::accessibleInfo()).arg(qApp->translate("MuseScore", keyNames[15]));
       else if (isCustom())
-            return tr("%1: Custom").arg(Element::accessibleInfo());
+            return QObject::tr("%1: Custom").arg(Element::accessibleInfo());
 
       if (key() == Key::C)
             return QString("%1: %2").arg(Element::accessibleInfo()).arg(qApp->translate("MuseScore", keyNames[14]));
