@@ -9,6 +9,7 @@
 //  as published by the Free Software Foundation and appearing in
 //  the file LICENCE.GPL
 //=============================================================================
+
 #include "navigate.h"
 #include "element.h"
 #include "clef.h"
@@ -18,7 +19,6 @@
 #include "chord.h"
 #include "system.h"
 #include "segment.h"
-#include "lyrics.h"
 #include "harmony.h"
 #include "utils.h"
 #include "input.h"
@@ -45,14 +45,14 @@ ChordRest* nextChordRest(ChordRest* cr, bool skipGrace)
             //
             // cr is a grace note
 
-            Chord* c  = static_cast<Chord*>(cr);
-            Chord* pc = static_cast<Chord*>(cr->parent());
+            Chord* c  = toChord(cr);
+            Chord* pc = toChord(cr->parent());
 
             if (skipGrace) {
-                  cr = static_cast<ChordRest*>(cr->parent());
+                  cr = toChordRest(cr->parent());
                   }
             else if (cr->isGraceBefore()) {
-                  QList<Chord*> cl = pc->graceNotesBefore();
+                  QVector<Chord*> cl = pc->graceNotesBefore();
                   auto i = std::find(cl.begin(), cl.end(), c);
                   if (i == cl.end())
                         return 0;   // unable to find self?
@@ -63,7 +63,7 @@ ChordRest* nextChordRest(ChordRest* cr, bool skipGrace)
                   return pc;
                   }
             else {
-                  QList<Chord*> cl = pc->graceNotesAfter();
+                  QVector<Chord*> cl = pc->graceNotesAfter();
                   auto i = std::find(cl.begin(), cl.end(), c);
                   if (i == cl.end())
                         return 0;   // unable to find self?
@@ -77,27 +77,27 @@ ChordRest* nextChordRest(ChordRest* cr, bool skipGrace)
       else {
             //
             // cr is not a grace note
-            if (cr->type() == Element::Type::CHORD && !skipGrace) {
-                  Chord* c = static_cast<Chord*>(cr);
+            if (cr->isChord() && !skipGrace) {
+                  Chord* c = toChord(cr);
                   if (!c->graceNotes().empty()) {
-                        QList<Chord*> cl = c->graceNotesAfter();
-                        if (!cl.isEmpty())
+                        QVector<Chord*> cl = c->graceNotesAfter();
+                        if (!cl.empty())
                               return cl.first();
                         }
                   }
             }
 
-      int track = cr->track();
-      Segment::Type st = Segment::Type::ChordRest;
+      int track      = cr->track();
+      SegmentType st = SegmentType::ChordRest;
 
       for (Segment* seg = cr->segment()->next1MM(st); seg; seg = seg->next1MM(st)) {
-            ChordRest* e = static_cast<ChordRest*>(seg->element(track));
+            ChordRest* e = toChordRest(seg->element(track));
             if (e) {
-                  if (e->type() == Element::Type::CHORD && !skipGrace) {
-                        Chord* c = static_cast<Chord*>(e);
+                  if (e->isChord() && !skipGrace) {
+                        Chord* c = toChord(e);
                         if (!c->graceNotes().empty()) {
-                              QList<Chord*> cl = c->graceNotesBefore();
-                              if (!cl.isEmpty())
+                              QVector<Chord*> cl = c->graceNotesBefore();
+                              if (!cl.empty())
                                     return cl.first();
                               }
                         }
@@ -123,14 +123,14 @@ ChordRest* prevChordRest(ChordRest* cr, bool skipGrace)
             //
             // cr is a grace note
 
-            Chord* c  = static_cast<Chord*>(cr);
-            Chord* pc = static_cast<Chord*>(cr->parent());
+            Chord* c  = toChord(cr);
+            Chord* pc = toChord(cr->parent());
 
             if (skipGrace) {
-                  cr = static_cast<ChordRest*>(cr->parent());
+                  cr = toChordRest(cr->parent());
                   }
             else if (cr->isGraceBefore()) {
-                  QList<Chord*> cl = pc->graceNotesBefore();
+                  QVector<Chord*> cl = pc->graceNotesBefore();
                   auto i = std::find(cl.begin(), cl.end(), c);
                   if (i == cl.end())
                         return 0;   // unable to find self?
@@ -140,7 +140,7 @@ ChordRest* prevChordRest(ChordRest* cr, bool skipGrace)
                   cr = pc;
                   }
             else {
-                  QList<Chord*> cl = pc->graceNotesAfter();
+                  QVector<Chord*> cl = pc->graceNotesAfter();
                   auto i = std::find(cl.begin(), cl.end(), c);
                   if (i == cl.end())
                         return 0;   // unable to find self?
@@ -153,22 +153,22 @@ ChordRest* prevChordRest(ChordRest* cr, bool skipGrace)
       else {
             //
             // cr is not a grace note
-            if (cr->type() == Element::Type::CHORD && !skipGrace) {
-                  Chord* c = static_cast<Chord*>(cr);
-                  QList<Chord*> cl = c->graceNotesBefore();
-                  if (!cl.isEmpty())
+            if (cr->isChord() && !skipGrace) {
+                  Chord* c = toChord(cr);
+                  QVector<Chord*> cl = c->graceNotesBefore();
+                  if (!cl.empty())
                         return cl.last();
                   }
             }
 
       int track = cr->track();
-      Segment::Type st = Segment::Type::ChordRest;
+      SegmentType st = SegmentType::ChordRest;
       for (Segment* seg = cr->segment()->prev1MM(st); seg; seg = seg->prev1MM(st)) {
-            ChordRest* e = static_cast<ChordRest*>(seg->element(track));
+            ChordRest* e = toChordRest(seg->element(track));
             if (e) {
-                  if (e->type() == Element::Type::CHORD && !skipGrace) {
-                        QList<Chord*> cl = static_cast<Chord*>(e)->graceNotesAfter();
-                        if (!cl.isEmpty())
+                  if (e->type() == ElementType::CHORD && !skipGrace) {
+                        QVector<Chord*> cl = toChord(e)->graceNotesAfter();
+                        if (!cl.empty())
                               return cl.last();
                         }
                   return e;
@@ -190,16 +190,16 @@ ChordRest* prevChordRest(ChordRest* cr, bool skipGrace)
 Element* Score::upAlt(Element* element)
       {
       Element* re = 0;
-      if (element->type() == Element::Type::REST)
-            re = prevTrack(static_cast<Rest*>(element));
-      else if (element->type() == Element::Type::NOTE) {
-            Chord* chord = static_cast<Note*>(element)->chord();
-            const QList<Note*>& notes = chord->notes();
-            int idx = notes.indexOf(static_cast<Note*>(element));
-            if (idx < notes.size()-1) {
-                  ++idx;
-                  re = notes.value(idx);
-                  }
+      if (element->isRest())
+            re = prevTrack(toRest(element));
+      else if (element->isNote()) {
+            Note* note = toNote(element);
+            Chord* chord = note->chord();
+            const std::vector<Note*>& notes = chord->notes();
+            auto i = std::find(notes.begin(), notes.end(), note);
+            ++i;
+            if (i != notes.end())
+                  re = *i;
             else {
                   re = prevTrack(chord);
                   if (re->track() == chord->track())
@@ -208,8 +208,8 @@ Element* Score::upAlt(Element* element)
             }
       if (re == 0)
             return 0;
-      if (re->type() == Element::Type::CHORD)
-            re = static_cast<Chord*>(re)->notes().front();
+      if (re->isChord())
+            re = toChord(re)->notes().front();
       return re;
       }
 
@@ -232,15 +232,16 @@ Note* Score::upAltCtrl(Note* note) const
 Element* Score::downAlt(Element* element)
       {
       Element* re = 0;
-      if (element->type() == Element::Type::REST)
-            re = nextTrack(static_cast<Rest*>(element));
-      else if (element->type() == Element::Type::NOTE) {
-            Chord* chord = static_cast<Note*>(element)->chord();
-            const QList<Note*>& notes = chord->notes();
-            int idx = notes.indexOf(static_cast<Note*>(element));
-            if (idx > 0) {
-                  --idx;
-                  re = notes.value(idx);
+      if (element->isRest())
+            re = nextTrack(toRest(element));
+      else if (element->isNote()) {
+            Note* note   = toNote(element);
+            Chord* chord = note->chord();
+            const std::vector<Note*>& notes = chord->notes();
+            auto i = std::find(notes.begin(), notes.end(), note);
+            if (i != notes.begin()) {
+                  --i;
+                  re = *i;
                   }
             else {
                   re = nextTrack(chord);
@@ -250,8 +251,8 @@ Element* Score::downAlt(Element* element)
             }
       if (re == 0)
             return 0;
-      if (re->type() == Element::Type::CHORD)
-            re = static_cast<Chord*>(re)->notes().back();
+      if (re->isChord())
+            re = toChord(re)->notes().back();
       return re;
       }
 
@@ -271,7 +272,7 @@ Note* Score::downAltCtrl(Note* note) const
 
 Element* Score::firstElement()
       {
-      Segment *s = firstSegment();
+      Segment *s = firstSegment(SegmentType::All);
       return s ? s->element(0) : nullptr;
       }
 
@@ -286,16 +287,17 @@ Element* Score::lastElement()
       if (!seg)
             return nullptr;
       while (true) {
-            for (int i = (staves().size() - 1) * VOICES; i < staves().size() * VOICES; i++) {
-                  if (seg->element(i) != 0)
+            for (int i = (staves().size() -1) * VOICES; i < staves().size() * VOICES; i++) {
+                  if (seg->element(i))
                         re = seg->element(i);
                   }
             if (re) {
-                  if (re->type() == Element::Type::CHORD)
-                        return static_cast<Chord*>(re)->notes().first();
+                  if (re->isChord()) {
+                        return toChord(re)->notes().front();
+                        }
                   return re;
                   }
-            seg = seg->prev1MM(Segment::Type::All);
+            seg = seg->prev1MM(SegmentType::All);
             }
       }
 
@@ -314,10 +316,10 @@ ChordRest* Score::upStaff(ChordRest* cr)
             Element* el = segment->element(track);
             if (!el)
                   continue;
-            if (el->type() == Element::Type::NOTE)
-                  el = static_cast<Note*>(el)->chord();
+            if (el->isNote())
+                  el = toNote(el)->chord();
             if (el->isChordRest())
-                  return static_cast<ChordRest*>(el);
+                  return toChordRest(el);
             }
       return 0;
       }
@@ -338,10 +340,10 @@ ChordRest* Score::downStaff(ChordRest* cr)
             Element* el = segment->element(track);
             if (!el)
                   continue;
-            if (el->type() == Element::Type::NOTE)
-                  el = static_cast<Note*>(el)->chord();
+            if (el->isNote())
+                  el = toNote(el)->chord();
             if (el->isChordRest())
-                  return static_cast<ChordRest*>(el);
+                  return toChordRest(el);
             }
       return 0;
       }
@@ -365,7 +367,7 @@ ChordRest* Score::nextTrack(ChordRest* cr)
 
       while (!el) {
             // find next non-empty track
-            while (++track < tracks){
+            while (++track < tracks) {
                   if (measure->hasVoice(track))
                         break;
                   }
@@ -373,8 +375,8 @@ ChordRest* Score::nextTrack(ChordRest* cr)
             if (track == tracks)
                   return cr;
             // find element at same or previous segment within this track
-            for (Segment* segment = cr->segment(); segment; segment = segment->prev(Segment::Type::ChordRest)) {
-                  el = static_cast<ChordRest*>(segment->element(track));
+            for (Segment* segment = cr->segment(); segment; segment = segment->prev(SegmentType::ChordRest)) {
+                  el = toChordRest(segment->element(track));
                   if (el)
                         break;
                   }
@@ -408,8 +410,8 @@ ChordRest* Score::prevTrack(ChordRest* cr)
             if (track < 0)
                   return cr;
             // find element at same or previous segment within this track
-            for (Segment* segment = cr->segment(); segment != 0; segment = segment->prev(Segment::Type::ChordRest)) {
-                  el = static_cast<ChordRest*>(segment->element(track));
+            for (Segment* segment = cr->segment(); segment; segment = segment->prev(SegmentType::ChordRest)) {
+                  el = toChordRest(segment->element(track));
                   if (el)
                         break;
                   }
@@ -432,10 +434,10 @@ ChordRest* Score::nextMeasure(ChordRest* element, bool selectBehavior, bool mmRe
       else
             measure = element->measure()->nextMeasure();
 
-      if (measure == 0)
+      if (!measure)
             return 0;
 
-      int endTick = element->measure()->last()->nextChordRest(element->track(), true)->tick();
+      Fraction endTick = element->measure()->last()->nextChordRest(element->track(), true)->tick();
       bool last   = false;
 
       if (selection().isRange()) {
@@ -463,7 +465,7 @@ ChordRest* Score::nextMeasure(ChordRest* element, bool selectBehavior, bool mmRe
                   Element* pel = seg->element(track);
 
                   if (pel && pel->isChordRest())
-                        return static_cast<ChordRest*>(pel);
+                        return toChordRest(pel);
                   }
             }
       return 0;
@@ -484,11 +486,10 @@ ChordRest* Score::prevMeasure(ChordRest* element, bool mmRest)
       else
             measure = element->measure()->prevMeasure();
 
-      int startTick = element->measure()->first()->nextChordRest(element->track())->tick();
+      Fraction startTick = element->measure()->first()->nextChordRest(element->track())->tick();
       bool last = false;
 
-      if ((selection().isRange())
-          && selection().isEndActive() && selection().startSegment()->tick() <= startTick)
+      if (selection().isRange() && selection().isEndActive() && selection().startSegment()->tick() <= startTick)
             last = true;
       else if (element->tick() != startTick) {
             measure = element->measure();
@@ -507,11 +508,216 @@ ChordRest* Score::prevMeasure(ChordRest* element, bool mmRest)
                   Element* pel = seg->element(track);
 
                   if (pel && pel->isChordRest())
-                        return static_cast<ChordRest*>(pel);
+                        return toChordRest(pel);
                   }
             }
       return 0;
       }
 
+//---------------------------------------------------------
+//   nextElement
+//---------------------------------------------------------
+
+Element* Score::nextElement()
+      {
+      Element* e = getSelectedElement();
+      if (!e)
+            return nullptr;
+      int staffId = e->staffIdx();
+      while (e) {
+            switch (e->type()) {
+                  case ElementType::NOTE:
+                  case ElementType::CHORD: {
+                        Element* next = e->nextElement();
+                        if (next)
+                              return next;
+                        else
+                              break;
+                       }
+                  case ElementType::SEGMENT: {
+                        Segment* s = toSegment(e);
+                        Element* next = s->nextElement(staffId);
+                        if (next)
+                              return next;
+                        else
+                              break;
+                        }
+                  case ElementType::MEASURE: {
+                        Measure* m = toMeasure(e);
+                        Element* next = m->nextElementStaff(staffId);
+                        if (next)
+                              return next;
+                        else
+                              break;
+                        }
+                  case ElementType::CLEF:
+                  case ElementType::KEYSIG:
+                  case ElementType::TIMESIG:
+                  case ElementType::BAR_LINE: {
+                       for (; e && e->type() != ElementType::SEGMENT; e = e->parent()) {
+                             ;
+                             }
+                       Segment* s = toSegment(e);
+                       Element* next = s->nextElement(staffId);
+                       if (next)
+                             return next;
+                       else
+                             return score()->firstElement();
+                       }
+                  case ElementType::SLUR_SEGMENT:
+                  case ElementType::TEXTLINE_SEGMENT:
+                  case ElementType::HAIRPIN_SEGMENT:
+                  case ElementType::OTTAVA_SEGMENT:
+                  case ElementType::TRILL_SEGMENT:
+                  case ElementType::VIBRATO_SEGMENT:
+                  case ElementType::VOLTA_SEGMENT:
+                  case ElementType::LET_RING_SEGMENT:
+                  case ElementType::PALM_MUTE_SEGMENT:
+                  case ElementType::PEDAL_SEGMENT: {
+                        SpannerSegment* s = toSpannerSegment(e);
+                        Spanner* sp = s->spanner();
+                        Spanner* nextSp = sp->nextSpanner(sp, staffId);
+                        if (nextSp)
+                              return nextSp->spannerSegments().front();
+
+                        Segment* seg = tick2segment(sp->tick());
+                        if (seg) {
+                              Segment* nextSegment = seg->next1();
+                              while (nextSegment) {
+                                    Element* nextEl = nextSegment->firstElementOfSegment(nextSegment, staffId);
+                                    if (nextEl)
+                                          return nextEl;
+                                    nextSegment = nextSegment->next1();
+                                    }
+                              }
+                        break;
+                        }
+                  case ElementType::GLISSANDO_SEGMENT:
+                  case ElementType::TIE_SEGMENT: {
+                        SpannerSegment* s = toSpannerSegment(e);
+                        Spanner* sp = s->spanner();
+                        Element* elSt = sp->startElement();
+                        Note* n = toNote(elSt);
+                        Element* next =  n->nextElement();
+                        if (next)
+                              return next;
+                        else
+                              break;
+                        }
+                  default:
+                        break;
+                  }
+            e = e->parent();
+            }
+      return score()->firstElement();
+      }
+
+//---------------------------------------------------------
+//   prevElement
+//---------------------------------------------------------
+
+Element* Score::prevElement()
+      {
+      Element* e = getSelectedElement();
+      if (!e)
+            return nullptr;
+      int staffId = e->staffIdx();
+      while (e) {
+            switch (e->type()) {
+                  case ElementType::NOTE:
+                  case ElementType::REST:
+                  case ElementType::CHORD: {
+                        Element* prev = e->prevElement();
+                        if (prev)
+                              return prev;
+                        else
+                              break;
+                        }
+                  case ElementType::SEGMENT: {
+                        Segment* s = toSegment(e);
+                        Element* prev = s->prevElement(staffId);
+                        if (prev)
+                              return prev;
+                        else
+                              break;
+                        }
+                  case ElementType::MEASURE: {
+                        Measure* m = toMeasure(e);
+                        return m->prevElementStaff(staffId);
+                        }
+                  case ElementType::CLEF:
+                  case ElementType::KEYSIG:
+                  case ElementType::TIMESIG:
+                  case ElementType::BAR_LINE: {
+                        for (; e && e->type() != ElementType::SEGMENT; e = e->parent()) {
+                              ;
+                              }
+                        Segment* s = toSegment(e);
+                        return s->prevElement(staffId);
+                        }
+                  case ElementType::SLUR_SEGMENT:
+                  case ElementType::TEXTLINE_SEGMENT:
+                  case ElementType::HAIRPIN_SEGMENT:
+                  case ElementType::OTTAVA_SEGMENT:
+                  case ElementType::TRILL_SEGMENT:
+                  case ElementType::VIBRATO_SEGMENT:
+                  case ElementType::VOLTA_SEGMENT:
+                  case ElementType::PEDAL_SEGMENT: {
+                        SpannerSegment* s = toSpannerSegment(e);
+                        Spanner* sp = s->spanner();
+                        Element* stEl = sp->startElement();
+                        Spanner* prevSp = sp->prevSpanner(sp, staffId);
+                        if (prevSp)
+                              return prevSp->spannerSegments().front();
+                        else {
+                              Segment* startSeg = sp->startSegment();
+                              if (!startSeg->annotations().empty()) {
+                                    Element* last = startSeg->lastAnnotation(startSeg, staffId);
+                                    if (last)
+                                          return last;
+                                    }
+                              Element* el = startSeg->lastElementOfSegment(startSeg, staffId);
+                              if (stEl->type() == ElementType::CHORD || stEl->type() == ElementType::REST
+                                       || stEl->type() == ElementType::REPEAT_MEASURE || stEl->type() == ElementType::NOTE) {
+                                    ChordRest* cr = startSeg->cr(stEl->track());
+                                    if (cr) {
+                                          Element* elCr = cr->lastElementBeforeSegment();
+                                          if (elCr) {
+                                                return elCr;
+                                                }
+                                          }
+                                    }
+                              if (el->isChord())
+                                    return toChord(el)->lastElementBeforeSegment();
+                              else if (el->isNote()) {
+                                    Chord* c = toNote(el)->chord();
+                                    return c->lastElementBeforeSegment();
+                                    }
+                              else {
+                                    return el;
+                                    }
+                             }
+                        }
+                  case ElementType::GLISSANDO_SEGMENT:
+                  case ElementType::TIE_SEGMENT: {
+                        SpannerSegment* s = toSpannerSegment(e);
+                        Spanner* sp = s->spanner();
+                        Element* elSt = sp->startElement();
+                        Q_ASSERT(elSt->type() == ElementType::NOTE);
+                        Note* n = toNote(elSt);
+                        Element* prev =  n->prevElement();
+                        if(prev)
+                              return prev;
+                        else
+                              break;
+                        }
+                  default:
+                        break;
+                  }
+            e = e->parent();
+            }
+      return score()->lastElement();
+      }
+
 }
 
