@@ -22,8 +22,8 @@
 #include "symbol.h"
 #include "noteevent.h"
 #include "pitchspelling.h"
-
-class QPainter;
+#include "shape.h"
+#include "key.h"
 
 namespace Ms {
 
@@ -43,63 +43,121 @@ class StaffType;
 enum class SymId;
 enum class AccidentalType : char;
 
-static const int MAX_DOTS = 3;
+static const int MAX_DOTS = 4;
 
 //---------------------------------------------------------
 //   @@ NoteHead
 //---------------------------------------------------------
 
-class NoteHead : public Symbol {
-      Q_OBJECT
-
-      Q_ENUMS(Group)
-      Q_ENUMS(Type)
+class NoteHead final : public Symbol {
+      Q_GADGET
    public:
       enum class Group : signed char {
+            ///.\{
             HEAD_NORMAL = 0,
             HEAD_CROSS,
-            HEAD_DIAMOND,
-            HEAD_TRIANGLE,
-            HEAD_MI,
-            HEAD_SLASH,
+            HEAD_PLUS,
             HEAD_XCIRCLE,
+            HEAD_WITHX,
+            HEAD_TRIANGLE_UP,
+            HEAD_TRIANGLE_DOWN,
+            HEAD_SLASHED1,
+            HEAD_SLASHED2,
+            HEAD_DIAMOND,
+            HEAD_DIAMOND_OLD,
+            HEAD_CIRCLED,
+            HEAD_CIRCLED_LARGE,
+            HEAD_LARGE_ARROW,
+            HEAD_BREVIS_ALT,
+
+            HEAD_SLASH,
+
+            HEAD_SOL,
+            HEAD_LA,
+            HEAD_FA,
+            HEAD_MI,
             HEAD_DO,
             HEAD_RE,
-            HEAD_FA,
-            HEAD_LA,
             HEAD_TI,
-            HEAD_SOL,
-            HEAD_BREVIS_ALT,
+            // not exposed from here
+            HEAD_DO_WALKER,
+            HEAD_RE_WALKER,
+            HEAD_TI_WALKER,
+            HEAD_DO_FUNK,
+            HEAD_RE_FUNK,
+            HEAD_TI_FUNK,
+
+            HEAD_DO_NAME,
+            HEAD_RE_NAME,
+            HEAD_MI_NAME,
+            HEAD_FA_NAME,
+            HEAD_SOL_NAME,
+            HEAD_LA_NAME,
+            HEAD_TI_NAME,
+            HEAD_SI_NAME,
+
+            HEAD_A_SHARP,
+            HEAD_A,
+            HEAD_A_FLAT,
+            HEAD_B_SHARP,
+            HEAD_B,
+            HEAD_B_FLAT,
+            HEAD_C_SHARP,
+            HEAD_C,
+            HEAD_C_FLAT,
+            HEAD_D_SHARP,
+            HEAD_D,
+            HEAD_D_FLAT,
+            HEAD_E_SHARP,
+            HEAD_E,
+            HEAD_E_FLAT,
+            HEAD_F_SHARP,
+            HEAD_F,
+            HEAD_F_FLAT,
+            HEAD_G_SHARP,
+            HEAD_G,
+            HEAD_G_FLAT,
+            HEAD_H,
+            HEAD_H_SHARP,
+
             HEAD_CUSTOM,
             HEAD_GROUPS,
             HEAD_INVALID = -1
+            ///\}
             };
       enum class Type : signed char {
+            ///.\{
             HEAD_AUTO    = -1,
             HEAD_WHOLE   = 0,
             HEAD_HALF    = 1,
             HEAD_QUARTER = 2,
             HEAD_BREVIS  = 3,
             HEAD_TYPES
+            ///\}
             };
 
+      Q_ENUM(Group)
+      Q_ENUM(Type)
+
       NoteHead(Score* s = 0) : Symbol(s) {}
       NoteHead &operator=(const NoteHead&) = delete;
       virtual NoteHead* clone() const override    { return new NoteHead(*this); }
-      virtual Element::Type type() const override { return Element::Type::NOTEHEAD; }
-
-      virtual void write(Xml& xml) const override;
+      virtual ElementType type() const override { return ElementType::NOTEHEAD; }
 
       Group headGroup() const;
 
-      static const char* groupToGroupName(Group group);
-      static const char* type2name(Type type);
-      static Type name2type(const QString&);
+      static QString group2userName(Group group);
+      static QString type2userName(Type type);
+      static QString group2name(Group group);
+      static QString type2name(Type type);
+      static Group name2group(QString s);
+      static Type name2type(QString s);
       };
 
 //---------------------------------------------------------
 //   NoteVal
-//    helper structure
+///    helper structure
+///   \cond PLUGIN_API \private \endcond
 //---------------------------------------------------------
 
 struct NoteVal {
@@ -114,6 +172,8 @@ struct NoteVal {
       NoteVal(int p) : pitch(p) {}
       };
 
+static const int INVALID_LINE = -10000;
+
 //---------------------------------------------------------------------------------------
 //   @@ Note
 ///    Graphic representation of a note.
@@ -148,56 +208,21 @@ struct NoteVal {
 //   @P veloType         enum (Note.OFFSET_VAL, Note.USER_VAL)
 //---------------------------------------------------------------------------------------
 
-class Note : public Element {
-      Q_OBJECT
-      Q_PROPERTY(Ms::Accidental*                accidental        READ accidental)
-      Q_PROPERTY(int                            accidentalType    READ qmlAccidentalType  WRITE qmlSetAccidentalType)
-      Q_PROPERTY(QQmlListProperty<Ms::NoteDot>  dots              READ qmlDots)
-      Q_PROPERTY(int                            dotsCount         READ qmlDotsCount)
-      Q_PROPERTY(QQmlListProperty<Ms::Element>  elements          READ qmlElements)
-      Q_PROPERTY(int                            fret              READ fret               WRITE undoSetFret)
-      Q_PROPERTY(bool                           ghost             READ ghost              WRITE undoSetGhost)
-      Q_PROPERTY(Ms::NoteHead::Group            headGroup         READ headGroup          WRITE undoSetHeadGroup)
-      Q_PROPERTY(Ms::NoteHead::Type             headType          READ headType           WRITE undoSetHeadType)
-      Q_PROPERTY(bool                           hidden            READ hidden)
-      Q_PROPERTY(int                            line              READ line)
-      Q_PROPERTY(bool                           mirror            READ mirror)
-      Q_PROPERTY(int                            pitch             READ pitch              WRITE undoSetPitch)
-      Q_PROPERTY(bool                           play              READ play               WRITE undoSetPlay)
-      Q_PROPERTY(int                            ppitch            READ ppitch)
-      Q_PROPERTY(bool                           small             READ small              WRITE undoSetSmall)
-      Q_PROPERTY(int                            string            READ string             WRITE undoSetString)
-      Q_PROPERTY(int                            subchannel        READ subchannel)
-      Q_PROPERTY(Ms::Tie*                       tieBack           READ tieBack)
-      Q_PROPERTY(Ms::Tie*                       tieFor            READ tieFor)
-      Q_PROPERTY(int                            tpc               READ tpc)
-      Q_PROPERTY(int                            tpc1              READ tpc1               WRITE undoSetTpc1)
-      Q_PROPERTY(int                            tpc2              READ tpc2               WRITE undoSetTpc2)
-      Q_PROPERTY(qreal                          tuning            READ tuning             WRITE undoSetTuning)
-      Q_PROPERTY(Ms::MScore::Direction          userDotPosition   READ userDotPosition    WRITE undoSetUserDotPosition)
-      Q_PROPERTY(Ms::MScore::DirectionH         userMirror        READ userMirror         WRITE undoSetUserMirror)
-      Q_PROPERTY(int                            veloOffset        READ veloOffset         WRITE undoSetVeloOffset)
-      Q_PROPERTY(Ms::Note::ValueType            veloType          READ veloType           WRITE undoSetVeloType)
-
-      Q_ENUMS(ValueType)
-      Q_ENUMS(Ms::MScore::Direction)
-      Q_ENUMS(Ms::MScore::DirectionH)
-
+class Note final : public Element {
+      Q_GADGET
    public:
       enum class ValueType : char { OFFSET_VAL, USER_VAL };
-      int qmlAccidentalType() const { return int(accidentalType()); }
-      void qmlSetAccidentalType(int t) { setAccidentalType(static_cast<AccidentalType>(t)); }
+      Q_ENUM(ValueType)
 
    private:
       bool _ghost         { false };      ///< ghost note (guitar: death note)
-      bool _hidden        { false };      ///< markes this note as the hidden one if there are
+      bool _hidden        { false };      ///< marks this note as the hidden one if there are
                                           ///< overlapping notes; hidden notes are not played
                                           ///< and heads + accidentals are not shown
       bool _dotsHidden    { false };      ///< dots of hidden notes are hidden too
                                           ///< except if only one note is dotted
       bool _fretConflict  { false };      ///< used by TAB staves to mark a fretting conflict:
-                                          ///< two or mor enotes on the same string
-
+                                          ///< two or more notes on the same string
       bool dragMode       { false };
       bool _mirror        { false };      ///< True if note is mirrored at stem.
       bool _small         { false };
@@ -205,8 +230,8 @@ class Note : public Element {
       mutable bool _mark  { false };      // for use in sequencer
       bool _fixed         { false };      // for slash notation
 
-      MScore::DirectionH _userMirror { MScore::DirectionH::AUTO };    ///< user override of mirror
-      MScore::Direction _userDotPosition { MScore::Direction::AUTO }; ///< user override of dot position
+      MScore::DirectionH _userMirror { MScore::DirectionH::AUTO };      ///< user override of mirror
+      Direction _userDotPosition     { Direction::AUTO };               ///< user override of dot position
 
       NoteHead::Group _headGroup { NoteHead::Group::HEAD_NORMAL };
       NoteHead::Type  _headType  { NoteHead::Type::HEAD_AUTO    };
@@ -217,7 +242,7 @@ class Note : public Element {
       char _onTimeType     { 0 };    // compatibility only 1 - user, 2 - offset
 
       int _subchannel     { 0  };   ///< articulation
-      int _line           { 0  };   ///< y-Position; 0 - top line.
+      int _line           { INVALID_LINE  };   ///< y-Position; 0 - top line.
       int _fret           { -1 };   ///< for tablature view
       int _string         { -1 };
       mutable int _tpc[2] { Tpc::TPC_INVALID, Tpc::TPC_INVALID }; ///< tonal pitch class  (concert/transposing)
@@ -225,30 +250,34 @@ class Note : public Element {
 
       int _veloOffset     { 0 };    ///< velocity user offset in percent, or absolute velocity for this note
       int _fixedLine      { 0 };    // fixed line number if _fixed == true
-      int _lineOffset     { 0 };    ///< Used during mouse dragging.
       qreal _tuning       { 0.0 };  ///< pitch offset in cent, playable only by internal synthesizer
 
       Accidental* _accidental { 0 };
 
-      ElementList _el;        ///< fingering, other text, symbols or images
       Tie* _tieFor        { 0 };
       Tie* _tieBack       { 0 };
 
-      QList<NoteDot*> _dots { 0, 0, 0 };
-
+      ElementList _el;        ///< fingering, other text, symbols or images
+      QVector<NoteDot*> _dots;
       NoteEventList _playEvents;
       QVector<Spanner*> _spannerFor;
       QVector<Spanner*> _spannerBack;
-      
+
       SymId _cachedNoteheadSym; // use in draw to avoid recomputing at every update
-      
-      virtual QRectF drag(EditData*) override;
-      void endDrag();
-      void endEdit();
+      SymId _cachedSymNull; // additional symbol for some transparent notehead
+
+      QString _fretString;
+
+      virtual void startDrag(EditData&) override;
+      virtual QRectF drag(EditData&) override;
+      virtual void endDrag(EditData&) override;
+      virtual void editDrag(EditData&) override;
       void addSpanner(Spanner*);
       void removeSpanner(Spanner*);
       int concertPitchIdx() const;
       void updateRelLine(int relLine, bool undoable);
+      bool isNoteName() const;
+      SymId noteHead() const;
 
    public:
       Note(Score* s = 0);
@@ -257,12 +286,12 @@ class Note : public Element {
 
       Note& operator=(const Note&) = delete;
       virtual Note* clone() const override  { return new Note(*this, false); }
-      Element::Type type() const override   { return Element::Type::NOTE; }
+      ElementType type() const override   { return ElementType::NOTE; }
+
+      virtual void undoUnlink() override;
 
       virtual qreal mag() const override;
 
-      QPointF pagePos() const;      ///< position in page coordinates
-      QPointF canvasPos() const;    ///< position in page coordinates
       void layout();
       void layout2();
       //setter is used only in drumset tools to setup the notehead preview in the drumset editor and the palette
@@ -274,8 +303,8 @@ class Note : public Element {
 
       qreal headWidth() const;
       qreal headHeight() const;
-      qreal tabHeadWidth(StaffType* tab = 0) const;
-      qreal tabHeadHeight(StaffType* tab = 0) const;
+      qreal tabHeadWidth(const StaffType* tab = 0) const;
+      qreal tabHeadHeight(const StaffType* tab = 0) const;
       QPointF stemDownNW() const;
       QPointF stemUpSE() const;
       qreal bboxXShift() const;
@@ -283,7 +312,6 @@ class Note : public Element {
       qreal bboxRightPos() const;
       qreal headBodyWidth() const;
 
-      SymId noteHead() const;
       NoteHead::Group headGroup() const   { return _headGroup; }
       NoteHead::Type headType() const     { return _headType;  }
       void setHeadGroup(NoteHead::Group val);
@@ -293,7 +321,6 @@ class Note : public Element {
       virtual QString subtypeName() const override;
 
       void setPitch(int val);
-      void undoSetPitch(int val);
       void setPitch(int pitch, int tpc1, int tpc2);
       int pitch() const                   { return _pitch;    }
       int ppitch() const;           ///< playback pitch
@@ -318,8 +345,6 @@ class Note : public Element {
       void setTpcFromPitch();
       int tpc1default(int pitch) const;
       int tpc2default(int pitch) const;
-      void undoSetTpc1(int tpc)      { undoChangeProperty(P_ID::TPC1, tpc); }
-      void undoSetTpc2(int tpc)      { undoChangeProperty(P_ID::TPC2, tpc); }
       int transposeTpc(int tpc);
 
       Accidental* accidental() const      { return _accidental; }
@@ -329,7 +354,8 @@ class Note : public Element {
       void setAccidentalType(AccidentalType type);
 
       int line() const;
-      void setLine(int n);
+      void setLine(int n)             { _line = n;      }
+      int physicalLine() const;
 
       int fret() const                { return _fret;   }
       void setFret(int val)           { _fret = val;    }
@@ -357,20 +383,21 @@ class Note : public Element {
       void setTieFor(Tie* t)          { _tieFor = t;     }
       void setTieBack(Tie* t)         { _tieBack = t;    }
       Note* firstTiedNote() const;
-      Note* lastTiedNote() const;
-      QList<Note*> tiedNotes() const;
+      const Note* lastTiedNote() const;
       void disconnectTiedNotes();
       void connectTiedNotes();
 
       Chord* chord() const            { return (Chord*)parent(); }
       void setChord(Chord* a)         { setParent((Element*)a);  }
-      void draw(QPainter*) const;
+      virtual void draw(QPainter*) const override;
 
       virtual void read(XmlReader&) override;
-      virtual void write(Xml& xml) const override;
+      virtual bool readProperties(XmlReader&) override;
+      virtual void readAddConnector(ConnectorInfoReader* info, bool pasteMode) override;
+      virtual void write(XmlWriter&) const override;
 
-      bool acceptDrop(const DropData&) const override;
-      Element* drop(const DropData&);
+      bool acceptDrop(EditData&) const override;
+      Element* drop(EditData&);
 
       bool hidden() const                       { return _hidden; }
       void setHidden(bool val)                  { _hidden = val;  }
@@ -380,10 +407,8 @@ class Note : public Element {
       NoteType noteType() const;
       QString  noteTypeUserName() const;
 
-      ElementList el()                            { return _el; }
-      const ElementList el() const                { return _el; }
-
-      QQmlListProperty<Ms::Element> qmlElements() { return QmlListAccess<Ms::Element>(this, _el); }
+      ElementList& el()                           { return _el; }
+      const ElementList& el() const               { return _el; }
 
       int subchannel() const                    { return _subchannel; }
       void setSubchannel(int val)               { _subchannel = val;  }
@@ -391,8 +416,8 @@ class Note : public Element {
       MScore::DirectionH userMirror() const             { return _userMirror; }
       void setUserMirror(MScore::DirectionH d)          { _userMirror = d; }
 
-      MScore::Direction userDotPosition() const         { return _userDotPosition; }
-      void setUserDotPosition(MScore::Direction d)      { _userDotPosition = d;    }
+      Direction userDotPosition() const         { return _userDotPosition; }
+      void setUserDotPosition(Direction d)      { _userDotPosition = d;    }
       bool dotIsUp() const;               // actual dot position
 
       void reset();
@@ -406,74 +431,67 @@ class Note : public Element {
       void setOffTimeOffset(int v);
 
       int customizeVelocity(int velo) const;
+      NoteDot* dot(int n)                         { return _dots[n];          }
+      const QVector<NoteDot*>& dots() const       { return _dots;             }
+      QVector<NoteDot*>& dots()                   { return _dots;             }
 
-      NoteDot* dot(int n)                       { return _dots[n];           }
-      QQmlListProperty<Ms::NoteDot> qmlDots() { return QmlListAccess<Ms::NoteDot>(this, _dots);  }
       int qmlDotsCount();
       void updateAccidental(AccidentalState*);
       void updateLine();
-      void setNval(const NoteVal&, int tick = -1);
+      void setNval(const NoteVal&, Fraction = { -1, 1} );
       NoteEventList& playEvents()                { return _playEvents; }
       const NoteEventList& playEvents() const    { return _playEvents; }
       NoteEvent* noteEvent(int idx)              { return &_playEvents[idx]; }
       void setPlayEvents(const NoteEventList& l) { _playEvents = l;    }
 
-      QVector<Spanner*> spannerFor() const         { return _spannerFor;         }
-      QVector<Spanner*> spannerBack() const        { return _spannerBack;        }
+      const QVector<Spanner*>& spannerFor() const   { return _spannerFor;         }
+      const QVector<Spanner*>& spannerBack() const  { return _spannerBack;        }
 
-      void addSpannerBack(Spanner* e)            { if (!_spannerBack.contains(e)) _spannerBack.push_back(e); }
+      void addSpannerBack(Spanner* e)            { if (!_spannerBack.contains(e)) _spannerBack.push_back(e);  }
       bool removeSpannerBack(Spanner* e)         { return _spannerBack.removeOne(e); }
-      void addSpannerFor(Spanner* e)             { if (!_spannerFor.contains(e)) _spannerFor.push_back(e);   }
+      void addSpannerFor(Spanner* e)             { if (!_spannerFor.contains(e)) _spannerFor.push_back(e);    }
       bool removeSpannerFor(Spanner* e)          { return _spannerFor.removeOne(e);  }
 
       void transposeDiatonic(int interval, bool keepAlterations, bool useDoubleAccidentals);
 
-      void undoSetFret(int);
-      void undoSetString(int);
-      void undoSetGhost(bool);
-      void undoSetMirror(bool);
-      void undoSetSmall(bool);
-      void undoSetPlay(bool);
-      void undoSetTuning(qreal);
-      void undoSetVeloType(ValueType);
-      void undoSetVeloOffset(int);
-      void undoSetOnTimeUserOffset(int);
-      void undoSetOffTimeUserOffset(int);
-      void undoSetUserMirror(MScore::DirectionH);
-      void undoSetUserDotPosition(MScore::Direction);
-      void undoSetHeadGroup(NoteHead::Group);
-      void undoSetHeadType(NoteHead::Type);
-
-      virtual QVariant getProperty(P_ID propertyId) const override;
-      virtual bool setProperty(P_ID propertyId, const QVariant&) override;
-      virtual QVariant propertyDefault(P_ID) const override;
+      virtual void localSpatiumChanged(qreal oldValue, qreal newValue) override;
+      virtual QVariant getProperty(Pid propertyId) const override;
+      virtual bool setProperty(Pid propertyId, const QVariant&) override;
+      void undoChangeDotsVisible(bool v);
+      virtual QVariant propertyDefault(Pid) const override;
+      virtual QString propertyUserValue(Pid) const override;
 
       bool mark() const               { return _mark;   }
       void setMark(bool v) const      { _mark = v;   }
       virtual void setScore(Score* s) override;
-      void setDotY(MScore::Direction);
+      void setDotY(Direction);
 
-      void addBracket();
-      bool isBefore(Note*);
+      void addParentheses();
 
+      static SymId noteHead(int direction, NoteHead::Group, NoteHead::Type, int tpc, Key key, NoteHeadScheme scheme);
       static SymId noteHead(int direction, NoteHead::Group, NoteHead::Type);
       NoteVal noteVal() const;
 
+      Element* nextInEl(Element* e);
+      Element* prevInEl(Element* e);
       virtual Element* nextElement() override;
       virtual Element* prevElement() override;
-      virtual QString accessibleInfo() override;
-      virtual QString screenReaderInfo() override;
-      virtual QString accessibleExtraInfo() override;
+      virtual Element* lastElementBeforeSegment();
+      virtual Element* nextSegmentElement() override;
+      virtual Element* prevSegmentElement() override;
+
+      virtual QString accessibleInfo() const override;
+      virtual QString screenReaderInfo() const override;
+      virtual QString accessibleExtraInfo() const override;
+
+      virtual Shape shape() const override;
+      std::vector<Note*> tiedNotes() const;
+
+      void setOffTimeType(int v) { _offTimeType = v; }
+      void setOnTimeType(int v)  { _onTimeType = v; }
+      int offTimeType() const    { return _offTimeType; }
+      int onTimeType() const     { return _onTimeType; }
       };
 
-// extern const SymId noteHeads[2][int(NoteHead::Group::HEAD_GROUPS)][int(NoteHead::Type::HEAD_TYPES)];
-
-
 }     // namespace Ms
-
-Q_DECLARE_METATYPE(Ms::NoteHead::Group);
-Q_DECLARE_METATYPE(Ms::NoteHead::Type);
-Q_DECLARE_METATYPE(Ms::Note::ValueType);
-
 #endif
-
