@@ -11,19 +11,32 @@
 //=============================================================================
 
 #include "noteline.h"
+#include "textline.h"
 
 namespace Ms {
 
 NoteLine::NoteLine(Score* s)
-   : TextLine(s)
+   : TextLineBase(s)
       {
+//TODO-ws      init();
       }
 
 NoteLine::NoteLine(const NoteLine& nl)
-   : TextLine(nl)
+   : TextLineBase(nl)
       {
       }
 
+//---------------------------------------------------------
+//   createLineSegment
+//---------------------------------------------------------
+
+LineSegment* NoteLine::createLineSegment()
+      {
+      TextLineSegment* seg = new TextLineSegment(this, score());
+      seg->setTrack(track());
+      return seg;
+      }
+
 
 }     // namespace Ms
 
