@@ -42,6 +42,7 @@ struct ElementPattern {
       int voice;
       const System* system;
       bool subtypeValid;
+      Fraction durationTicks;
       };
 
 //---------------------------------------------------------
@@ -73,6 +74,10 @@ enum class SelState : char {
                   // is selected
       };
 
+//---------------------------------------------------------
+//   SelectionFilterType
+//---------------------------------------------------------
+
 enum class SelectionFilterType {
       NONE                    = 0,
       FIRST_VOICE             = 1 << 0,
@@ -99,12 +104,18 @@ enum class SelectionFilterType {
       ALL                     = -1
       };
 
+
+//---------------------------------------------------------
+//   SelectionFilter
+//---------------------------------------------------------
+
 class SelectionFilter {
       Score* _score;
       int _filtered;
 
 public:
       SelectionFilter()                      { _score = 0; _filtered = (int)SelectionFilterType::ALL;}
+      SelectionFilter(SelectionFilterType f) : _score(nullptr), _filtered(int(f)) {}
       SelectionFilter(Score* score)          { _score = score; _filtered = (int)SelectionFilterType::ALL;}
       int& filtered()                        { return _filtered; }
       void setFiltered(SelectionFilterType type, bool set);
@@ -129,6 +140,11 @@ class Selection {
       Segment* _startSegment;
       Segment* _endSegment;         // next segment after selection
 
+      Fraction _plannedTick1 { -1, 1 }; // Will be actually selected on updateSelectedElements() call.
+      Fraction _plannedTick2 { -1, 1 }; // Used by setRangeTicks() to restore proper selection after
+                              // command end in case some changes are expected to segments'
+                              // structure (e.g. MMRests reconstruction).
+
       Segment* _activeSegment;
       int _activeTrack;
 
@@ -151,7 +167,7 @@ class Selection {
       void setState(SelState s);
 
       const QList<Element*>& elements() const { return _el; }
-      QList<Note*> noteList(int track = -1) const;
+      std::vector<Note*> noteList(int track = -1) const;
 
       const QList<Element*> uniqueElements() const;
       QList<Note*> uniqueNotes(int track = -1) const;
@@ -179,13 +195,14 @@ class Selection {
       void setStartSegment(Segment* s)  { _startSegment = s; }
       void setEndSegment(Segment* s)    { _endSegment = s; }
       void setRange(Segment* startSegment, Segment* endSegment, int staffStart, int staffEnd);
+      void setRangeTicks(const Fraction& tick1, const Fraction& tick2, int staffStart, int staffEnd);
       Segment* activeSegment() const    { return _activeSegment; }
       void setActiveSegment(Segment* s) { _activeSegment = s; }
       ChordRest* activeCR() const;
       bool isStartActive() const;
       bool isEndActive() const;
-      int tickStart() const;
-      int tickEnd() const;
+      Fraction tickStart() const;
+      Fraction tickEnd() const;
       int staffStart() const            { return _staffStart;  }
       int staffEnd() const              { return _staffEnd;    }
       int activeTrack() const           { return _activeTrack; }
@@ -196,7 +213,7 @@ class Selection {
       void updateSelectedElements();
       bool measureRange(Measure** m1, Measure** m2) const;
       void extendRangeSelection(ChordRest* cr);
-      void extendRangeSelection(Segment* seg, Segment* segAfter, int staffIdx, int tick, int etick);
+      void extendRangeSelection(Segment* seg, Segment* segAfter, int staffIdx, const Fraction& tick, const Fraction& etick);
       };
 
 
