@@ -19,7 +19,7 @@ namespace Ms {
 //   ticks_beat
 //---------------------------------------------------------
 
-static int ticks_beat(int n)
+int ticks_beat(int n)
       {
       int m = (MScore::division * 4) / n;
       if ((MScore::division * 4) % n) {
@@ -230,6 +230,22 @@ void TimeSigMap::del(int tick)
       normalize();
       }
 
+//---------------------------------------------------------
+//   clearRange
+//    Clears the given range, start tick included, end tick
+//    excluded.
+//---------------------------------------------------------
+
+void TimeSigMap::clearRange(int tick1, int tick2)
+      {
+      iterator first = lower_bound(tick1);
+      iterator last = lower_bound(tick2);
+      if (first == last)
+            return;
+      erase(first, last);
+      normalize();
+      }
+
 //---------------------------------------------------------
 //   TimeSigMap::normalize
 //---------------------------------------------------------
@@ -350,7 +366,7 @@ int TimeSigMap::bar2tick(int bar, int beat) const
 //   TimeSigMap::write
 //---------------------------------------------------------
 
-void TimeSigMap::write(Xml& xml) const
+void TimeSigMap::write(XmlWriter& xml) const
       {
       xml.stag("siglist");
       for (auto i = begin(); i != end(); ++i)
@@ -392,7 +408,7 @@ SigEvent::SigEvent(const SigEvent& e)
 //   SigEvent::write
 //---------------------------------------------------------
 
-void SigEvent::write(Xml& xml, int tick) const
+void SigEvent::write(XmlWriter& xml, int tick) const
       {
       xml.stag(QString("sig tick=\"%1\"").arg(tick));
       xml.tag("nom",   _timesig.numerator());
@@ -534,6 +550,14 @@ void TimeSigMap::dump() const
                i->first, qPrintable(i->second.timesig().print()), i->second.bar());
       }
 
+//---------------------------------------------------------
+//   dUnitTicks
+//---------------------------------------------------------
+
+int TimeSigFrac::dUnitTicks() const
+      {
+      return (4 * MScore::division) / denominator();
+      }
 
 }
 
