@@ -49,11 +49,11 @@ public:
       StringData() {}
       StringData(int numFrets, int numStrings, int strings[]);
       StringData(int numFrets, QList<instrString>& strings);
-      bool        convertPitch(int pitch, Staff* staff, int tick, int* string, int* fret) const;
-      int         fret(int pitch, int string, Staff* staff, int tick) const;
+      bool        convertPitch(int pitch, Staff* staff, const Fraction& tick, int* string, int* fret) const;
+      int         fret(int pitch, int string, Staff* staff, const Fraction& tick) const;
       void        fretChords(Chord * chord) const;
-      int         getPitch(int string, int fret, Staff* staff, int tick) const;
-      static int  pitchOffsetAt(Staff* staff, int tick);
+      int         getPitch(int string, int fret, Staff* staff, const Fraction& tick) const;
+      static int  pitchOffsetAt(Staff* staff, const Fraction& tick);
       int         strings() const                   { return stringTable.size(); }
       int         frettedStrings() const;
       const QList<instrString>&  stringList() const { return stringTable; }
@@ -61,9 +61,8 @@ public:
       int         frets() const                 { return _frets; }
       void        setFrets(int val)             { _frets = val; }
       void        read(XmlReader&);
-      void        write(Xml&) const;
-//      void        readMusicXML(XmlReader& de);
-      void        writeMusicXML(Xml& xml) const;
+      void        write(XmlWriter&) const;
+      void        writeMusicXML(XmlWriter& xml) const;
       bool operator==(const StringData& d) const { return d._frets == _frets && d.stringTable == stringTable; }
       };
 
