@@ -187,10 +187,26 @@ void CapExplicitBarline::readCapx(XmlReader& e)
 void CapClef::readCapx(XmlReader& e)
       {
       QString clef = e.attribute("clef");
-      if (clef == "G2-") { form = Form::G; line = ClefLine::L2; oct = Oct::OCT_BASSA; }
-      else if (clef == "treble") { form = Form::G; line = ClefLine::L2; oct = Oct::OCT_NULL; }
-      else if (clef == "bass") { form = Form::F; line = ClefLine::L4; oct = Oct::OCT_NULL; }
-      else { /* default */ form = Form::G; line = ClefLine::L2; oct = Oct::OCT_NULL; }
+      if (clef == "G2-") {
+            form = Form::G;
+            line = ClefLine::L2;
+            oct = Oct::OCT_BASSA;
+            }
+      else if (clef == "treble") {
+            form = Form::G;
+            line = ClefLine::L2;
+            oct = Oct::OCT_NULL;
+            }
+      else if (clef == "bass") {
+            form = Form::F;
+            line = ClefLine::L4;
+            oct = Oct::OCT_NULL;
+            }
+      else {
+            /* default */ form = Form::G;
+            line = ClefLine::L2;
+            oct = Oct::OCT_NULL;
+            }
       qDebug("Clef::read '%s' -> form %d line %d oct %d", qPrintable(clef), int(form), int(line), int(oct));
       e.readNext();
       }
@@ -918,10 +934,10 @@ void Capella::readCapxStaveLayout(XmlReader& e, CapStaffLayout* sl, int /*idx*/)
                   sl->abbrev = e.attribute("abbrev");
                   // elements name and abbrev overrule attributes name and abbrev
                   while (e.readNextStartElement()) {
-                        const QStringRef& tag(e.name());
-                        if (tag == "name")
+                        const QStringRef& t(e.name());
+                        if (t == "name")
                               sl->name = e.readElementText();
-                        else if (tag == "abbrev")
+                        else if (t == "abbrev")
                               sl->abbrev = e.readElementText();
                         else
                               e.unknown();
@@ -1166,7 +1182,7 @@ void Capella::readCapx(XmlReader& e)
 
 void convertCapella(Score* score, Capella* cap, bool capxMode);
 
-Score::FileError importCapXml(Score* score, const QString& name)
+Score::FileError importCapXml(MasterScore* score, const QString& name)
       {
       qDebug("importCapXml(score %p, name %s)", score, qPrintable(name));
       MQZipReader uz(name);
