@@ -1,9 +1,8 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: editpitch.cpp 3775 2010-12-17 23:55:35Z miwarre $
 //
-//  Copyright (C) 2002-2009 Werner Schweer and others
+//  Copyright (C) 2002-2016 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -58,18 +57,6 @@ void EditPitch::hideEvent(QHideEvent* ev)
       QWidget::hideEvent(ev);
       }
 
-void EditPitch::changeEvent(QEvent *e)
-      {
-      QDialog::changeEvent(e);
-      switch (e->type()) {
-            case QEvent::LanguageChange:
-                  retranslateUi(this);
-                  break;
-            default:
-            break;
-            }
-      }
-
 void EditPitch::accept()
       {
       on_tableWidget_cellDoubleClicked(tableWidget->currentRow(), tableWidget->currentColumn());
