@@ -1,9 +1,8 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: editpitch.h 3775 2010-12-17 23:55:35Z miwarre $
 //
-//  Copyright (C) 2002-2009 Werner Schweer and others
+//  Copyright (C) 2002-2016 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -34,9 +33,6 @@ class EditPitch : public QDialog, private Ui::EditPitchBase {
 
       virtual void hideEvent(QHideEvent*);
 
-   protected:
-      void changeEvent(QEvent *e);
-
    private slots:
       void on_tableWidget_cellDoubleClicked(int row, int column);
       void accept();
