@@ -1,7 +1,6 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: editstyle.h 5403 2012-03-03 00:01:53Z miwarre $
 //
 //  Copyright (C) 2002-2010 Werner Schweer and others
 //
@@ -23,6 +22,7 @@
 
 #include "ui_editstyle.h"
 #include "globals.h"
+#include "libmscore/mscore.h"
 #include "libmscore/style.h"
 
 namespace Ms {
@@ -34,8 +34,10 @@ class Score;
 //---------------------------------------------------------
 
 struct StyleWidget {
-      StyleIdx idx;
-      QWidget* widget;
+      Sid idx;
+      bool showPercent;
+      QObject* widget;
+      QToolButton* reset;
       };
 
 //---------------------------------------------------------
@@ -47,21 +49,20 @@ class EditStyle : public QDialog, private Ui::EditStyleBase {
 
       Score* cs;
       QPushButton* buttonApplyToAllParts;
-      MStyle lstyle;    // local copy of style
-
       QButtonGroup* stemGroups[VOICES];
-
       QVector<StyleWidget> styleWidgets;
+      QButtonGroup* keySigNatGroup;
+      QButtonGroup* clefTypeGroup;
+      bool isTooBig;
+      bool hasShown;
 
+      virtual void showEvent(QShowEvent*);
       virtual void hideEvent(QHideEvent*);
-
-      void getValues();
+      QVariant getValue(Sid idx);
       void setValues();
-      void setHeaderText(StyleIdx idx, QTextEdit* te);
-      void setFooterText(StyleIdx idx, QTextEdit* te);
 
-      void apply();
       void applyToAllParts();
+      const StyleWidget& styleWidget(Sid) const;
 
    private slots:
       void selectChordDescriptionFile();
@@ -70,14 +71,24 @@ class EditStyle : public QDialog, private Ui::EditStyleBase {
       void toggleFooterOddEven(bool);
       void buttonClicked(QAbstractButton*);
       void setSwingParams(bool);
+      void concertPitchToggled(bool);
+      void lyricsDashMinLengthValueChanged(double);
+      void lyricsDashMaxLengthValueChanged(double);
       void systemMinDistanceValueChanged(double);
       void systemMaxDistanceValueChanged(double);
-      void resetStyleValue(int i);
-
+      void resetStyleValue(int);
+      void valueChanged(int);
+      void textStyleChanged(int);
+      void resetTextStyle(Pid);
+      void textStyleValueChanged(Pid, QVariant);
       void on_comboFBFont_currentIndexChanged(int index);
+      void on_buttonTogglePagelist_clicked();
+      void editUserStyleName();
+      void endEditUserStyleName();
+      void resetUserStyleName();
 
 public:
-      static const int PAGE_NOTE = 6;
+      static const int PAGE_NOTE = 10;
       EditStyle(Score*, QWidget*);
       void setPage(int no);
       };
@@ -85,5 +96,3 @@ public:
 
 } // namespace Ms
 #endif
-
-
