@@ -1,7 +1,6 @@
 //=============================================================================
 //  MuseScore
 //  Music Composition & Notation
-//  $Id:$
 //
 //  Copyright (C) 2013 Werner Schweer
 //
@@ -23,21 +22,19 @@ namespace Ms {
 //---------------------------------------------------------
 
 InspectorArpeggio::InspectorArpeggio(QWidget* parent)
-   : InspectorBase(parent)
+   : InspectorElementBase(parent)
       {
-      e.setupUi(addWidget());
       g.setupUi(addWidget());
 
-      iList = {
-            { P_ID::COLOR,           0, false, e.color,    e.resetColor    },
-            { P_ID::VISIBLE,         0, false, e.visible,  e.resetVisible  },
-            { P_ID::USER_OFF,        0, false, e.offsetX,  e.resetX        },
-            { P_ID::USER_OFF,        1, false, e.offsetY,  e.resetY        },
-
-            { P_ID::PLAY,            0, 0,     g.playArpeggio, g.resetPlayArpeggio}
+      const std::vector<InspectorItem> iiList = {
+            { Pid::PLAY,            0,    g.playArpeggio, g.resetPlayArpeggio},
+            { Pid::TIME_STRETCH,    0,    g.stretch,      g.resetStretch }
+            };
+      const std::vector<InspectorPanel> ppList = {
+            { g.title, g.panel }
             };
 
-      mapSignals();
+      mapSignals(iiList, ppList);
       }
 }
 
