@@ -1,7 +1,6 @@
 //=============================================================================
 //  MuseScore
 //  Music Composition & Notation
-//  $Id:$
 //
 //  Copyright (C) 2011 Werner Schweer and others
 //
@@ -24,10 +23,9 @@ namespace Ms {
 //   InspectorBeam
 //---------------------------------------------------------
 
-class InspectorBeam : public InspectorBase {
+class InspectorBeam : public InspectorElementBase {
       Q_OBJECT
 
-      UiInspectorElement e;
       Ui::InspectorBeam b;
 
    protected slots:
