@@ -1,7 +1,6 @@
 //=============================================================================
 //  MuseScore
 //  Music Composition & Notation
-//  $Id:$
 //
 //  Copyright (C) 2012 Werner Schweer and others
 //
@@ -15,7 +14,8 @@
 #define __INSPECTOR_HAIRPIN_H__
 
 #include "inspector.h"
-#include "inspectorBase.h"
+// #include "inspectorBase.h"
+#include "inspectorTextLineBase.h"
 #include "ui_inspector_hairpin.h"
 #include "ui_inspector_line.h"
 
@@ -25,13 +25,17 @@ namespace Ms {
 //   InspectorHairpin
 //---------------------------------------------------------
 
-class InspectorHairpin : public InspectorBase {
+// class InspectorHairpin : public InspectorElementBase {
+class InspectorHairpin : public InspectorTextLineBase {
       Q_OBJECT
 
-      UiInspectorElement e;
-      Ui::InspectorLine l;
+//      Ui::InspectorLine l;
       Ui::InspectorHairpin h;
 
+      void updateLineType();
+      virtual void setElement() override;
+      virtual void valueChanged(int idx) override;
+
    public:
       InspectorHairpin(QWidget* parent);
       virtual void postInit() override;
