@@ -1,7 +1,6 @@
 //=============================================================================
 //  MuseScore
 //  Music Composition & Notation
-//  $Id:$
 //
 //  Copyright (C) 2013 Werner Schweer and others
 //
@@ -14,10 +13,8 @@
 #ifndef __INSPECTOR_MARKER_H__
 #define __INSPECTOR_MARKER_H__
 
-#include "inspector.h"
-#include "inspectorBase.h"
+#include "inspectorTextBase.h"
 #include "ui_inspector_marker.h"
-#include "libmscore/property.h"
 
 namespace Ms {
 
@@ -25,16 +22,13 @@ namespace Ms {
 //   InspectorMarker
 //---------------------------------------------------------
 
-class InspectorMarker : public InspectorBase {
+class InspectorMarker : public InspectorTextBase {
       Q_OBJECT
 
-      UiInspectorElement b;
-      Ui::InspectorText t;
       Ui::InspectorMarker  m;
 
    public:
       InspectorMarker(QWidget* parent);
-      virtual void setElement() override;
       };
 
 
