@@ -14,8 +14,9 @@
 #define __INSPECTOR_TEXT_H__
 
 #include "inspector.h"
-#include "inspectorBase.h"
+#include "inspectorTextBase.h"
 #include "ui_inspector_text.h"
+#include "ui_inspector_frametext.h"
 
 namespace Ms {
 
@@ -23,15 +24,13 @@ namespace Ms {
 //   InspectorText
 //---------------------------------------------------------
 
-class InspectorText : public InspectorBase {
+class InspectorText : public InspectorTextBase {
       Q_OBJECT
 
-      UiInspectorElement e;
-      Ui::InspectorText t;
+      Ui::InspectorFrameText f;
 
    public:
       InspectorText(QWidget* parent);
-      virtual void setElement() override;
       };
 
 }
