@@ -1,7 +1,6 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: mididriver.cpp 5568 2012-04-22 10:08:43Z wschweer $
 //
 //  Copyright (C) 2008 Werner Schweer and others
 //
@@ -457,7 +456,7 @@ void AlsaMidiDriver::read()
 
 void AlsaMidiDriver::write(const Event& e)
       {
-      Score* cs = mscore->currentScore();
+      MasterScore* cs = mscore->currentScore()->masterScore();
       int port  = cs->midiPort(e.channel());
       int chn   = cs->midiChannel(e.channel());
       int a     = e.dataA();
