@@ -1,7 +1,7 @@
+
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: newwizard.cpp 5626 2012-05-13 18:33:52Z lasconic $
 //
 //  Copyright (C) 2008 Werner Schweer and others
 //
@@ -23,7 +23,7 @@
 #include "preferences.h"
 #include "palette.h"
 #include "instrdialog.h"
-#include "scoreBrowser.h"
+#include "templateBrowser.h"
 #include "extension.h"
 
 #include "libmscore/instrtemplate.h"
@@ -54,6 +54,7 @@ TimesigWizard::TimesigWizard(QWidget* parent)
       connect(tsCommonTime, SIGNAL(toggled(bool)), SLOT(commonTimeToggled(bool)));
       connect(tsCutTime,    SIGNAL(toggled(bool)), SLOT(cutTimeToggled(bool)));
       connect(tsFraction,   SIGNAL(toggled(bool)), SLOT(fractionToggled(bool)));
+      pickupMeasure->setChecked(false); // checked in the UI file to enable screen reader on pickup duration controls
       }
 
 //---------------------------------------------------------
@@ -154,10 +155,10 @@ TitleWizard::TitleWizard(QWidget* parent)
       }
 
 //---------------------------------------------------------
-//   NewWizardPage1
+//   NewWizardInfoPage
 //---------------------------------------------------------
 
-NewWizardPage1::NewWizardPage1(QWidget* parent)
+NewWizardInfoPage::NewWizardInfoPage(QWidget* parent)
    : QWizardPage(parent)
       {
       setTitle(tr("Create New Score"));
@@ -176,77 +177,79 @@ NewWizardPage1::NewWizardPage1(QWidget* parent)
 //   initializePage
 //---------------------------------------------------------
 
-void NewWizardPage1::initializePage()
+void NewWizardInfoPage::initializePage()
       {
       w->title->setText("");
       w->subtitle->setText("");
       }
 
 //---------------------------------------------------------
-//   NewWizardPage2
+//   NewWizardInstrumentsPage
 //---------------------------------------------------------
 
-NewWizardPage2::NewWizardPage2(QWidget* parent)
+NewWizardInstrumentsPage::NewWizardInstrumentsPage(QWidget* parent)
    : QWizardPage(parent)
       {
+      setFinalPage(true);
       setTitle(tr("Create New Score"));
       setSubTitle(tr("Choose instruments on the left to add to instrument list on the right:"));
       setAccessibleName(title());
       setAccessibleDescription(subTitle());
-      complete = false;
-      w = new InstrumentsWidget;
+      instrumentsWidget = new InstrumentsWidget;
       QGridLayout* grid = new QGridLayout;
       grid->setSpacing(0);
       grid->setContentsMargins(0, 0, 0, 0);
-      grid->addWidget(w, 0, 0);
+      grid->addWidget(instrumentsWidget, 0, 0);
       setLayout(grid);
-      connect(w, SIGNAL(completeChanged(bool)), this, SLOT(setComplete(bool)));
+      connect(instrumentsWidget, SIGNAL(completeChanged(bool)), SLOT(setComplete(bool)));
       }
 
 //---------------------------------------------------------
 //   initializePage
 //---------------------------------------------------------
 
-void NewWizardPage2::initializePage()
+void NewWizardInstrumentsPage::initializePage()
       {
-      w->init();
+      complete = false;
+      instrumentsWidget->init();
       }
 
 //---------------------------------------------------------
 //   setComplete
 //---------------------------------------------------------
 
-void NewWizardPage2::setComplete(bool val)
+void NewWizardInstrumentsPage::setComplete(bool val)
       {
       complete = val;
       emit completeChanged();
       }
 
+//---------------------------------------------------------
+//   isComplete
+//---------------------------------------------------------
+
+bool NewWizardInstrumentsPage::isComplete() const
+      {
+      return complete;
+      }
+
 //---------------------------------------------------------
 //   createInstruments
 //---------------------------------------------------------
 
-void NewWizardPage2::createInstruments(Score* s)
+void NewWizardInstrumentsPage::createInstruments(Score* s)
       {
-      w->createInstruments(s);
+      instrumentsWidget->createInstruments(s);
       }
 
 //---------------------------------------------------------
-//   buildInstrumentsList
+//   NewWizardTimesigPage
 //---------------------------------------------------------
 
-void NewWizardPage2::buildInstrumentsList()
-      {
-      w->buildTemplateList();
-      }
-
-//---------------------------------------------------------
-//   NewWizardPage3
-//---------------------------------------------------------
-
-NewWizardPage3::NewWizardPage3(QWidget* parent)
+NewWizardTimesigPage::NewWizardTimesigPage(QWidget* parent)
    : QWizardPage(parent)
       {
+      setFinalPage(true);
       setTitle(tr("Create New Score"));
       setSubTitle(tr("Choose time signature:"));
       setAccessibleName(title());
@@ -259,66 +262,53 @@ NewWizardPage3::NewWizardPage3(QWidget* parent)
       }
 
 //---------------------------------------------------------
-//   NewWizardPage4
+//   NewWizardTemplatePage
 //---------------------------------------------------------
 
-NewWizardPage4::NewWizardPage4(QWidget* parent)
+NewWizardTemplatePage::NewWizardTemplatePage(QWidget* parent)
    : QWizardPage(parent)
       {
+      setFinalPage(true);
       setTitle(tr("Create New Score"));
       setSubTitle(tr("Choose template file:"));
       setAccessibleName(title());
       setAccessibleDescription(subTitle());
 
-      templateFileBrowser = new ScoreBrowser;
+      templateFileBrowser = new TemplateBrowser(this);
       templateFileBrowser->setStripNumbers(true);
-      templateFileBrowser->setShowCustomCategory(true);
-      templateFileBrowser->setSizePolicy(QSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored));
-      buildTemplatesList();
 
       QVBoxLayout* layout = new QVBoxLayout;
-      QHBoxLayout* searchLayout = new QHBoxLayout;
-      QLineEdit* search = new QLineEdit;
-      search->setPlaceholderText(tr("Search"));
-      search->setClearButtonEnabled(true);
-      search->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
-      searchLayout->addSpacerItem(new QSpacerItem(0, 0, QSizePolicy::MinimumExpanding, QSizePolicy::Maximum));
-      searchLayout->addWidget(search);
-
-      layout->addLayout(searchLayout);
       layout->addWidget(templateFileBrowser);
       setLayout(layout);
 
       connect(templateFileBrowser, SIGNAL(scoreSelected(const QString&)), SLOT(templateChanged(const QString&)));
       connect(templateFileBrowser, SIGNAL(scoreActivated(const QString&)), SLOT(fileAccepted(const QString&)));
-      connect(search, &QLineEdit::textChanged, [this] (const QString& searchString) {
-            this->templateFileBrowser->filter(searchString);
-            });
+      buildTemplatesList();
       }
 
 //---------------------------------------------------------
 //   initializePage
 //---------------------------------------------------------
 
-void NewWizardPage4::initializePage()
+void NewWizardTemplatePage::initializePage()
       {
-      templateFileBrowser->show();
-      path.clear();
+
       }
 
 //---------------------------------------------------------
 //   buildTemplatesList
 //---------------------------------------------------------
 
-void NewWizardPage4::buildTemplatesList()
+void NewWizardTemplatePage::buildTemplatesList()
       {
+
       QDir dir(mscoreGlobalShare + "/templates");
       QFileInfoList fil = dir.entryInfoList(QDir::NoDotAndDotDot | QDir::Readable | QDir::Dirs | QDir::Files, QDir::Name);
       if(fil.isEmpty()){
           fil.append(QFileInfo(QFile(":data/Empty_Score.mscz")));
           }
 
-      QDir myTemplatesDir(preferences.myTemplatesPath);
+      QDir myTemplatesDir(preferences.getString(PREF_APP_PATHS_MYTEMPLATES));
       fil.append(myTemplatesDir.entryInfoList(QDir::NoDotAndDotDot | QDir::Readable | QDir::Dirs | QDir::Files, QDir::Name));
 
       // append templates directories from extensions
@@ -334,7 +324,7 @@ void NewWizardPage4::buildTemplatesList()
 //   isComplete
 //---------------------------------------------------------
 
-bool NewWizardPage4::isComplete() const
+bool NewWizardTemplatePage::isComplete() const
       {
       return !path.isEmpty();
       }
@@ -343,19 +333,21 @@ bool NewWizardPage4::isComplete() const
 //   fileAccepted
 //---------------------------------------------------------
 
-void NewWizardPage4::fileAccepted(const QString& s)
+void NewWizardTemplatePage::fileAccepted(const QString& s)
       {
       path = s;
       templateFileBrowser->show();
-      wizard()->next();
+      if (wizard()->currentPage() == this)
+            wizard()->next();
       }
 
 //---------------------------------------------------------
 //   templateChanged
 //---------------------------------------------------------
 
-void NewWizardPage4::templateChanged(const QString& s)
+void NewWizardTemplatePage::templateChanged(const QString& s)
       {
+      setFinalPage(QFileInfo(s).completeBaseName() != "00-Blank");
       path = s;
       emit completeChanged();
       }
@@ -364,18 +356,19 @@ void NewWizardPage4::templateChanged(const QString& s)
 //   templatePath
 //---------------------------------------------------------
 
-QString NewWizardPage4::templatePath() const
+QString NewWizardTemplatePage::templatePath() const
       {
       return path;
       }
 
 //---------------------------------------------------------
-//   NewWizardPage5
+//   NewWizardKeysigPage
 //---------------------------------------------------------
 
-NewWizardPage5::NewWizardPage5(QWidget* parent)
+NewWizardKeysigPage::NewWizardKeysigPage(QWidget* parent)
    : QWizardPage(parent)
       {
+      setFinalPage(true);
       setTitle(tr("Create New Score"));
       setSubTitle(tr("Choose key signature and tempo:"));
       setAccessibleName(title());
@@ -383,12 +376,21 @@ NewWizardPage5::NewWizardPage5(QWidget* parent)
 
       QGroupBox* b1 = new QGroupBox;
       b1->setTitle(tr("Key Signature"));
-      b1->setAccessibleName(title());
+      b1->setAccessibleName(b1->title());
+      b1->setAccessibleDescription(tr("Choose a key signature"));
       sp = MuseScore::newKeySigPalette();
+      sp->setMoreElements(false);
+      sp->setShowContextMenu(false);
       sp->setSelectable(true);
       sp->setDisableDoubleClick(true);
-      sp->setSelected(14);
+      int keysigCMajorIdx = 14;
+      sp->setSelected(keysigCMajorIdx);
       PaletteScrollArea* sa = new PaletteScrollArea(sp);
+      // set widget name to include name of selected key signature
+      // we could set the description, but some screen readers ignore it
+      sa->setAccessibleName(tr("Key Signature: %1").arg(qApp->translate("Palette", sp->cellAt(keysigCMajorIdx)->name.toUtf8())));
+      QAccessibleEvent event(sa, QAccessible::NameChanged);
+      QAccessible::updateAccessibility(&event);
       QVBoxLayout* l1 = new QVBoxLayout;
       l1->addWidget(sa);
       b1->setLayout(l1);
@@ -397,6 +399,8 @@ NewWizardPage5::NewWizardPage5(QWidget* parent)
       tempoGroup->setCheckable(true);
       tempoGroup->setChecked(false);
       tempoGroup->setTitle(tr("Tempo"));
+      tempoGroup->setAccessibleName(tempoGroup->title());
+      tempoGroup->setAccessibleDescription(tr("Add tempo marking to score"));
       QLabel* bpm = new QLabel;
       bpm->setText(tr("BPM:"));
       _tempo = new QDoubleSpinBox;
@@ -415,13 +419,14 @@ NewWizardPage5::NewWizardPage5(QWidget* parent)
       l3->addWidget(tempoGroup);
       l3->addStretch(100);
       setLayout(l3);
+      setFocusPolicy(Qt::StrongFocus);
       }
 
 //---------------------------------------------------------
 //   keysig
 //---------------------------------------------------------
 
-KeySigEvent NewWizardPage5::keysig() const
+KeySigEvent NewWizardKeysigPage::keysig() const
       {
       int idx    = sp->getSelectedIdx();
       Element* e = sp->element(idx);
@@ -437,35 +442,47 @@ NewWizard::NewWizard(QWidget* parent)
       {
       setObjectName("NewWizard");
       setWindowFlags(this->windowFlags() & ~Qt::WindowContextHelpButtonHint);
-      setWizardStyle(QWizard::ClassicStyle);
+      auto wizardStyleValue = QWizard::ModernStyle; //Modern Winsows look
+#ifdef Q_OS_MAC
+      wizardStyleValue = QWizard::MacStyle;
+      setOption(QWizard::CancelButtonOnLeft, true);
+#endif
+      setWizardStyle(wizardStyleValue);
+      
       setPixmap(QWizard::LogoPixmap, QPixmap(":/data/mscore.png"));
       setPixmap(QWizard::WatermarkPixmap, QPixmap());
       setWindowTitle(tr("New Score Wizard"));
 
       setOption(QWizard::NoCancelButton, false);
-      setOption(QWizard::CancelButtonOnLeft, true);
       setOption(QWizard::HaveFinishButtonOnEarlyPages, true);
       setOption(QWizard::HaveNextButtonOnLastPage, true);
 
+      infoPage = new NewWizardInfoPage;
+      instrumentsPage = new NewWizardInstrumentsPage;
+      timesigPage = new NewWizardTimesigPage;
+      templatePage = new NewWizardTemplatePage;
+      keysigPage = new NewWizardKeysigPage;
 
-      p1 = new NewWizardPage1;
-      p2 = new NewWizardPage2;
-      p3 = new NewWizardPage3;
-      p4 = new NewWizardPage4;
-      p5 = new NewWizardPage5;
+//  enum Page { Invalid = -1, Type, Instruments, Template, Keysig, Timesig};
 
-      setPage(int(Page::Type), p1);
-      setPage(int(Page::Template), p4);
-      setPage(int(Page::Instruments), p2);
-      setPage(int(Page::Timesig), p3);
-      setPage(int(Page::Keysig), p5);
-      p2->setFinalPage(true);
-      p3->setFinalPage(true);
-      p4->setFinalPage(true);
-      p5->setFinalPage(true);
+      setPage(Page::Type,        infoPage);
+      setPage(Page::Instruments, instrumentsPage);
+      setPage(Page::Template,    templatePage);
+      setPage(Page::Keysig,      keysigPage);
+      setPage(Page::Timesig,     timesigPage);
 
       resize(QSize(840, 560)); //ensure default size if no geometry in settings
       MuseScore::restoreGeometry(this);
+      connect(this, SIGNAL(currentIdChanged(int)), SLOT(idChanged(int)));
+      }
+
+//---------------------------------------------------------
+//   idChanged
+//---------------------------------------------------------
+
+void NewWizard::idChanged(int /*id*/)
+      {
+//printf("\n===\nWizard: id changed %d\n", id);
       }
 
 //---------------------------------------------------------
@@ -474,22 +491,26 @@ NewWizard::NewWizard(QWidget* parent)
 
 int NewWizard::nextId() const
       {
-      switch(Page(currentId())) {
+      int next;
+      switch (Page(currentId())) {
             case Page::Type:
-                  return int(Page::Template);
-            case Page::Template: {
-                  if (emptyScore())
-                        return int(Page::Instruments);
-                  return int(Page::Keysig);
-                  }
+                  next = Page::Template;
+                  break;
+            case Page::Template:
+                  next = emptyScore() ? Page::Instruments : Page::Keysig;
+                  break;
             case Page::Instruments:
-                  return int(Page::Keysig);
+                  next = Page::Keysig;
+                  break;
             case Page::Keysig:
-                  return int(Page::Timesig);
+                  next = Page::Timesig;
+                  break;
             case Page::Timesig:
             default:
-                  return int(Page::Invalid);
+                  next = Page::Invalid;
+                  break;
             }
+      return next;
       }
 
 //---------------------------------------------------------
@@ -498,9 +519,10 @@ int NewWizard::nextId() const
 
 bool NewWizard::emptyScore() const
       {
-      QString p = p4->templatePath();
+      QString p = templatePage->templatePath();
       QFileInfo fi(p);
-      return fi.completeBaseName() == "00-Blank";
+      bool val = fi.completeBaseName() == "00-Blank";
+      return val;
       }
 
 //---------------------------------------------------------
@@ -509,8 +531,8 @@ bool NewWizard::emptyScore() const
 
 void NewWizard::updateValues() const
       {
-      p2->buildInstrumentsList();
-      p4->buildTemplatesList();
+      //p2->buildInstrumentsList();
+      templatePage->buildTemplatesList();
       }
 
 //---------------------------------------------------------
