@@ -1,7 +1,6 @@
 //=============================================================================
 //  MuseScore
 //  Music Composition & Notation
-//  $Id:$
 //
 //  Copyright (C) 2011 Werner Schweer and others
 //
@@ -11,6 +10,8 @@
 //  the file LICENSE.GPL
 //=============================================================================
 
+#include "config.h"
+
 #ifdef OMR
 
 #ifndef __OMRPANEL_H__
@@ -27,7 +28,7 @@ class OmrView;
 //---------------------------------------------------------
 
 class OmrPanel : public QDockWidget {
-      //Q_OBJECT
+      Q_OBJECT
 
       Ui::OmrPanel op;
 
@@ -53,9 +54,9 @@ class OmrPanel : public QDockWidget {
    public:
       OmrPanel(QWidget* parent = 0);
       void setOmrView(OmrView*);
-      };
+   };
 
 
-} // namespace Ms
+}// namespace Ms
 #endif
 #endif
