@@ -1,7 +1,6 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: pa.cpp 5662 2012-05-23 07:35:47Z wschweer $
 //
 //  Copyright (C) 2002-2010 Werner Schweer and others
 //
@@ -77,6 +76,8 @@ Portaudio::~Portaudio()
             if (err != paNoError)
                   qDebug("Portaudio close stream failed: %s", Pa_GetErrorText(err));
             Pa_Terminate();
+            if (midiDriver)
+                  delete midiDriver;
             }
       }
 
@@ -96,7 +97,7 @@ bool Portaudio::init(bool)
       if (MScore::debugMode)
             qDebug("using PortAudio Version: %s", Pa_GetVersionText());
 
-      PaDeviceIndex idx = preferences.portaudioDevice;
+      PaDeviceIndex idx = preferences.getInt(PREF_IO_PORTAUDIO_DEVICE);
       if (idx < 0) {
             idx = Pa_GetDefaultOutputDevice();
             qDebug("No device selected.  PortAudio detected %d devices.  Will use the default device (index %d).", Pa_GetDeviceCount(), idx);
@@ -319,7 +320,7 @@ void Portaudio::putEvent(const NPlayEvent& e, unsigned framePos)
                               }
                         break;
                         }
-                  //Fallback
+                  // fall through
             case ME_PITCHBEND:
                   {
                   long msg = Pm_Message(e.type() | chan, less128(e.dataA()), less128(e.dataB()));
@@ -359,7 +360,7 @@ void Portaudio::putEvent(const NPlayEvent& e, unsigned framePos)
 
 int Portaudio::currentApi() const
       {
-      PaDeviceIndex idx = preferences.portaudioDevice;
+      PaDeviceIndex idx = preferences.getInt(PREF_IO_PORTAUDIO_DEVICE);
       if (idx < 0)
             idx = Pa_GetDefaultOutputDevice();
 
@@ -383,7 +384,7 @@ int Portaudio::currentApi() const
 
 int Portaudio::currentDevice() const
       {
-      PaDeviceIndex idx = preferences.portaudioDevice;
+      PaDeviceIndex idx = preferences.getInt(PREF_IO_PORTAUDIO_DEVICE);
       if (idx < 0)
             idx = Pa_GetDefaultOutputDevice();
 
