@@ -1,9 +1,8 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id:$
 //
-//  Copyright (C) 2011 Werner Schweer and others
+//  Copyright (C) 2011-2016 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -36,6 +35,7 @@ static const qreal BKEY_HEIGHT = 25.0;
 HPiano::HPiano(QWidget* parent)
    : QGraphicsView(parent)
       {
+      scaleVal = 1.0;
       setLineWidth(0);
       setMidLineWidth(0);
 
@@ -50,6 +50,10 @@ HPiano::HPiano(QWidget* parent)
       grabGesture(Qt::PinchGesture);      // laptop pad (Mac) and touchscreen
 
       scene()->setSceneRect(0.0, 0.0, KEY_WIDTH * 52, KEY_HEIGHT);
+      QSizePolicy policy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+      setSizePolicy(policy);
+      int margin = 16;
+      setMaximumSize(QSize((KEY_WIDTH * 52 + margin/2) * scaleVal, 1000));
 
       _firstKey   = 21;
       _lastKey    = 108;   // 88 key piano
@@ -121,7 +125,15 @@ void HPiano::setScale(qreal s)
       if (s != scaleVal) {
             scaleVal = s;
             int margin = 16;
-            setMaximumSize(QSize((KEY_WIDTH * 52 + margin/2) * scaleVal, (KEY_HEIGHT + margin) * scaleVal ));
+            QDockWidget* par = static_cast<QDockWidget*>(parent());
+            if (par) {
+                  if (!par->isFloating())
+                        setMaximumSize(QSize((KEY_WIDTH * 52 + margin/2) * scaleVal, 1000));
+                  else
+                        setMaximumSize(QSize((KEY_WIDTH * 52 + margin/2) * scaleVal, (KEY_HEIGHT + margin) * scaleVal));
+                  }
+            else
+                  setMaximumSize(QSize((KEY_WIDTH * 52 + margin/2) * scaleVal, (KEY_HEIGHT + margin) * scaleVal));
             setMinimumSize(QSize(100 * scaleVal, (KEY_HEIGHT + margin) * scaleVal));
             QTransform t;
             t.scale(scaleVal, scaleVal);
@@ -142,9 +154,9 @@ QSize HPiano::sizeHint() const
 //   pressKeys
 //---------------------------------------------------------
 
-void HPiano::setPressedPitches(QSet<int> pitches)
+void HPiano::setPressedPlaybackPitches(QSet<int> pitches)
       {
-      _pressedPitches = pitches;
+      _pressedPlaybackPitches = pitches;
       updateAllKeys();
       }
 
@@ -172,7 +184,7 @@ void HPiano::releasePitch(int pitch)
 //   changeSelection
 //---------------------------------------------------------
 
-void HPiano::changeSelection(Selection selection)
+void HPiano::changeSelection(const Selection& selection)
       {
       for (PianoKeyItem* key : keys) {
             key->setHighlighted(false);
@@ -206,11 +218,21 @@ void HPiano::clearSelection()
 void HPiano::updateAllKeys()
       {
       for (PianoKeyItem* key : keys) {
-            key->setPressed(_pressedPitches.contains(key->pitch()));
+            key->setPressed(_pressedPitches.contains(key->pitch())
+                            || _pressedPlaybackPitches.contains(key->pitch()));
             key->update();
             }
       }
 
+void HPiano::setMaximum(bool top_level) {
+      int margin = 16;
+      if (!top_level)
+            setMaximumSize(QSize((KEY_WIDTH * 52 + margin/2) * scaleVal, 1000));
+      else
+            setMaximumSize(QSize((KEY_WIDTH * 52 + margin/2) * scaleVal, (KEY_HEIGHT + margin) * scaleVal));
+      updateAllKeys();
+      }
+
 //---------------------------------------------------------
 //   PianoKeyItem
 //---------------------------------------------------------
@@ -224,6 +246,10 @@ PianoKeyItem::PianoKeyItem(HPiano* _piano, int p)
       _selected = false;
       _highlighted = false;
       type = -1;
+
+      QString pitchNames[] = {"C", "C#", "D", "D#", "E", "F", "F#", "G", "G#", "A", "A#", "B"};
+      QString text = pitchNames[_pitch % 12] + QString::number((_pitch / 12) - 1);
+      setToolTip(text);
       }
 
 //---------------------------------------------------------
@@ -359,12 +385,12 @@ void PianoKeyItem::paint(QPainter* p, const QStyleOptionGraphicsItem* /*o*/, QWi
       p->setRenderHint(QPainter::Antialiasing, true);
       p->setPen(QPen(Qt::black, .8));
       if (_pressed) {
-            QColor c(preferences.pianoHlColor);
+            QColor c(preferences.getColor(PREF_UI_PIANO_HIGHLIGHTCOLOR));
             c.setAlpha(180);
             p->setBrush(c);
             }
       else if (_selected) {
-            QColor c(preferences.pianoHlColor);
+            QColor c(preferences.getColor(PREF_UI_PIANO_HIGHLIGHTCOLOR));
             c.setAlpha(100);
             p->setBrush(c);
             }
@@ -373,11 +399,12 @@ void PianoKeyItem::paint(QPainter* p, const QStyleOptionGraphicsItem* /*o*/, QWi
       else
             p->setBrush(type >= 7 ? Qt::black : Qt::white);
       p->drawPath(path());
-      if (_pitch == 60) {
-            QFont f("FreeSerif", 8);
+      if (_pitch % 12 == 0) {
+            QFont f("FreeSerif", 6);
             p->setFont(f);
+            QString text = "C" + QString::number((_pitch / 12) - 1);
             p->drawText(QRectF(KEY_WIDTH / 2, KEY_HEIGHT - 8, 0, 0),
-               Qt::AlignCenter | Qt::TextDontClip, "c'");
+               Qt::AlignCenter | Qt::TextDontClip, text);
             }
       }
 
@@ -389,28 +416,52 @@ PianoTools::PianoTools(QWidget* parent)
    : QDockWidget(parent)
       {
       setObjectName("piano");
-      setWindowTitle(tr("Piano Keyboard"));
       setAllowedAreas(Qt::DockWidgetAreas(Qt::TopDockWidgetArea | Qt::BottomDockWidgetArea));
 
       _piano = new HPiano;
       _piano->setFocusPolicy(Qt::ClickFocus);
       setWidget(_piano);
 
+      QSizePolicy policy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+      setSizePolicy(policy);
+
       connect(_piano, SIGNAL(keyPressed(int, bool, int)), SIGNAL(keyPressed(int, bool, int)));
       connect(_piano, SIGNAL(keyReleased(int, bool, int)), SIGNAL(keyReleased(int, bool, int)));
+      connect(this, SIGNAL(topLevelChanged(bool)), _piano, SLOT(setMaximum(bool)));
+      retranslate();
+      }
+
+//---------------------------------------------------------
+//   retranslate
+//---------------------------------------------------------
+
+void PianoTools::retranslate()
+      {
+      setWindowTitle(tr("Piano Keyboard"));
       }
 
 //---------------------------------------------------------
 //   heartBeat
 //---------------------------------------------------------
 
-void PianoTools::heartBeat(QList<const Ms::Note *> notes)
+void PianoTools::setPlaybackNotes(QList<const Ms::Note *> notes)
       {
       QSet<int> pitches;
       for (const Note* note : notes) {
           pitches.insert(note->ppitch());
           }
-      _piano->setPressedPitches(pitches);
+      _piano->setPressedPlaybackPitches(pitches);
+      }
+
+//---------------------------------------------------------
+//   changeEvent
+//---------------------------------------------------------
+
+void PianoTools::changeEvent(QEvent *event)
+      {
+      QDockWidget::changeEvent(event);
+      if (event->type() == QEvent::LanguageChange)
+            retranslate();
       }
 
 //---------------------------------------------------------
@@ -483,7 +534,7 @@ bool HPiano::gestureEvent(QGestureEvent *event)
 //   changeSelection
 //---------------------------------------------------------
 
-void PianoTools::changeSelection(Selection selection)
+void PianoTools::changeSelection(const Selection& selection)
       {
       _piano->changeSelection(selection);
       }
