@@ -1,7 +1,6 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: editstaff.h 3629 2010-10-26 10:40:47Z wschweer $
 //
 //  Copyright (C) 2002-2009 Werner Schweer and others
 //
@@ -42,7 +41,7 @@ class SelectInstrument : public QDialog, private Ui::SelectInstrument {
       void on_instrumentList_itemSelectionChanged();
       void on_instrumentList_itemDoubleClicked(QTreeWidgetItem* item, int);
 
-      void on_search_textChanged(const QString &searchPhrase);
+      void on_search_textChanged(const QString&);
 
       void on_instrumentGenreFilter_currentIndexChanged(int);
       void filterInstrumentsByGenre(QTreeWidget *, QString);
