@@ -40,6 +40,10 @@
 ****************************************************************************/
 
 #include "svggenerator.h"
+#include "libmscore/element.h"
+#include "libmscore/image.h"
+#include "libmscore/imageStore.h"
+#include "libmscore/mscore.h"
 
 ///////////////////////////////////////////////////////////////////////////////
 // FOR GRADIENT FUNCTIONALITY THAT IS NOT IMPLEMENTED (YET):
@@ -136,7 +140,7 @@ static void translate_dashPattern(QVector<qreal> pattern, const qreal& width, QS
 // Gets the contents of the SVG class attribute, based on element type/name
 static QString getClass(const Ms::Element *e)
 {
-    Ms::Element::Type eType;
+    Ms::ElementType eType;
               QString eName;
 
     // Add element type as "class"
@@ -240,6 +244,8 @@ protected:
 // The Ms::Element being generated right now
     const Ms::Element* _element = NULL;
 
+    void writeImage(const QRectF& r, const QByteArray& imageData, const QString& mimeFormat);
+
 // SVG strings as constants
 #define SVG_SPACE    ' '
 #define SVG_QUOTE    "\""
@@ -979,6 +985,7 @@ int SvgGenerator::metric(QPaintDevice::PaintDeviceMetric metric) const
     case QPaintDevice::PdmPhysicalDpiY:
         return d->engine->resolution();
     case QPaintDevice::PdmDevicePixelRatio:
+    case QPaintDevice::PdmDevicePixelRatioScaled:
         return 1;
     default:
         qWarning("SvgGenerator::metric(), unhandled metric %d\n", metric);
@@ -1090,13 +1097,6 @@ void SvgPaintEngine::drawImage(const QRectF  &r, const QImage &image,
     Q_UNUSED(sr);
     Q_UNUSED(flags);
 
-    stream() << SVG_IMAGE           << stateString
-             << SVG_X << SVG_QUOTE  << r.x() + _dx << SVG_QUOTE
-             << SVG_Y << SVG_QUOTE  << r.y() + _dy << SVG_QUOTE
-             << SVG_WIDTH           << r.width()   << SVG_QUOTE
-             << SVG_HEIGHT          << r.height()  << SVG_QUOTE
-             << SVG_PRESERVE_ASPECT << SVG_NONE    << SVG_QUOTE;
-
     QByteArray      data;
     QBuffer buffer(&data);
 
@@ -1104,35 +1104,70 @@ void SvgPaintEngine::drawImage(const QRectF  &r, const QImage &image,
     image.save(&buffer, "PNG");
     buffer.close();
 
-    stream() << " xlink:href=\"data:image/png;base64,"
-             << data.toBase64() << SVG_QUOTE << SVG_ELEMENT_END << endl;
+
+    // check whether we can just use the original raster image
+    // to reduce the resulting file size
+    if (_element && _element->isImage()) {
+        static const QList<QString> allowedImageTypes {
+            "image/jpeg",
+            "image/png"
+        };
+
+        const Ms::Image* img = Ms::toImage(_element);
+        const Ms::ImageStoreItem* storeItem = img->storeItem(); // holds the original image file content
+        if (img->getImageType() == Ms::ImageType::RASTER && storeItem) {
+            const QByteArray& imgData = storeItem->buffer();
+            const QMimeType type = QMimeDatabase().mimeTypeForData(imgData);
+            if (type.isValid() && allowedImageTypes.contains(type.name())
+                && imgData.size() < data.size()) {
+                // this really can save space, save the original image
+                writeImage(r, imgData, type.name());
+                return;
+            }
+        }
+    }
+
+    writeImage(r, data, "image/png");
 }
 
-void SvgPaintEngine::updateState(const QPaintEngineState &state)
+void SvgPaintEngine::writeImage(const QRectF& r, const QByteArray& imageData, const QString& mimeFormat)
+{
+    stream() << SVG_IMAGE           << stateString
+             << SVG_X << SVG_QUOTE  << r.x() + _dx << SVG_QUOTE
+             << SVG_Y << SVG_QUOTE  << r.y() + _dy << SVG_QUOTE
+             << SVG_WIDTH           << r.width()   << SVG_QUOTE
+             << SVG_HEIGHT          << r.height()  << SVG_QUOTE
+             << SVG_PRESERVE_ASPECT << SVG_NONE    << SVG_QUOTE;
+
+    stream() << " xlink:href=\"data:" << mimeFormat << ";base64,"
+             << imageData.toBase64() << SVG_QUOTE << SVG_ELEMENT_END << endl;
+}
+
+void SvgPaintEngine::updateState(const QPaintEngineState &s)
 {
     // Always start fresh
     stateString.clear();
 
     // stateString = Attribute Settings
 
-    // SVG class attribute, based on Ms::Element::Type
+    // SVG class attribute, based on Ms::ElementType
     stateStream << SVG_CLASS << getClass(_element) << SVG_QUOTE;
 
     // Brush and Pen attributes
-    stateStream << qbrushToSvg(state.brush());
-    stateStream <<   qpenToSvg(state.pen());
+    stateStream << qbrushToSvg(s.brush());
+    stateStream <<   qpenToSvg(s.pen());
 
 // TBD:  "opacity" attribute: Is it ever used?
 //       Or is opacity determined by fill-opacity & stroke-opacity instead?
 // PLUS: qFuzzyIsNull() is not officially supported in Qt.
 //       Should probably use QFuzzyCompare() instead.
-    if (!qFuzzyIsNull(state.opacity() - 1))
-        stateStream << SVG_OPACITY << state.opacity() << SVG_QUOTE;
+    if (!qFuzzyIsNull(s.opacity() - 1))
+        stateStream << SVG_OPACITY << s.opacity() << SVG_QUOTE;
 
     // Translations, SVG transform="translate()", are handled separately from
     // other transformations such as rotation. Qt translates everything, but
     // other transformations do occur, and must be handled here.
-    QTransform t = state.transform();
+    QTransform t = s.transform();
 
     // Tablature Note Text:
     // m11 and m22 have floating point flotsam, for example: 1.000000629
@@ -1187,10 +1222,10 @@ void SvgPaintEngine::drawPath(const QPainterPath &p)
             stream() << SVG_CURVE << x << SVG_COMMA << y;
             ++i;
             while (i < p.elementCount()) {
-                const QPainterPath::Element &e = p.elementAt(i);
-                if (e.type == QPainterPath::CurveToDataElement) {
-                    stream() << SVG_SPACE << e.x + _dx
-                             << SVG_COMMA << e.y + _dy;
+                const QPainterPath::Element &ee = p.elementAt(i);
+                if (ee.type == QPainterPath::CurveToDataElement) {
+                    stream() << SVG_SPACE << ee.x + _dx
+                             << SVG_COMMA << ee.y + _dy;
                     ++i;
                 }
                 else {
