@@ -2,7 +2,7 @@
 //  MuseScore
 //  Music Composition & Notation
 //
-//  Copyright (C) 2013-17 Werner Schweer & others
+//  Copyright (C) 2013-15 Werner Schweer & others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2
@@ -83,17 +83,17 @@ void PositionCursor::paint(QPainter* p)
                   p->setBrush(_color);
                   p->drawConvexPolygon(points, 3);
                   break;
-            default:                            // fill the rectangle and add TAB string marks, if required                  
+            default:                            // fill the rectangle and add TAB string marks, if required
                   p->fillRect(_rect, color());
                   if (_sv->score()->noteEntryMode()) {
                         int         track       = _sv->score()->inputTrack();
-                        Staff*      staff       = _sv->score()->staff(track2staff(track));
-                        if (staff) {
-                              StaffType*  staffType   = staff->staffType();
+                        if (track >= 0) {
+                              Staff*      staff       = _sv->score()->staff(track2staff(track));
+                              const StaffType*  staffType   = staff->staffType(Fraction(0,1));
                               if (staffType && staffType->group() == StaffGroup::TAB)
                                     staffType->drawInputStringMarks(p, _sv->score()->inputState().string(),
-                                                track2voice(track), _rect);
-                              }
+                                       track2voice(track), _rect);
+                                    }
                         }
                   break;
             }
@@ -127,28 +127,29 @@ QRectF PositionCursor::bbox() const
 //   move
 //---------------------------------------------------------
 
-void PositionCursor::move(int tick)
+void PositionCursor::move(const Fraction& t)
       {
+      Fraction tick(t);
       QRectF r(bbox());
       //
       // set mark height for whole system
       //
-      if (_type == CursorType::LOOP_OUT)
-        tick --;
+      if (_type == CursorType::LOOP_OUT && tick > Fraction(0,1))
+            tick -= Fraction::fromTicks(1);           // tick--
       Score* score = _sv->score();
       Measure* measure = score->tick2measureMM(tick);
       if (measure == 0)
             return;
-      qreal x;
-      int offset = 0;
+      qreal x = 0.0;
+      const Fraction offset = {0,1};    //??
 
       Segment* s;
-      for (s = measure->first(Segment::Type::ChordRest); s;) {
-            int t1 = s->tick();
+      for (s = measure->first(SegmentType::ChordRest); s;) {
+            Fraction t1 = s->tick();
             int x1 = s->canvasPos().x();
             qreal x2;
-            int t2;
-            Segment* ns = s->next(Segment::Type::ChordRest);
+            Fraction t2;
+            Segment* ns = s->next(SegmentType::ChordRest);
             if (ns) {
                   t2 = ns->tick();
                   x2 = ns->canvasPos().x();
@@ -160,9 +161,9 @@ void PositionCursor::move(int tick)
             t1 += offset;
             t2 += offset;
             if (tick >= t1 && tick < t2) {
-                  int   dt = t2 - t1;
+                  Fraction dt = t2 - t1;
                   qreal dx = x2 - x1;
-                  x = x1 + dx * (tick-t1) / dt;
+                  x = x1 + dx * (tick-t1).ticks() / dt.ticks();
                   break;
                   }
             s = ns;
@@ -171,7 +172,7 @@ void PositionCursor::move(int tick)
             return;
 
       System* system = measure->system();
-      if (system == 0)
+      if (system == 0 || system->page() == 0)
             return;
       double y        = system->staffYpage(0) + system->page()->pos().y();
       double _spatium = score->spatium();
