@@ -1,7 +1,6 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: textpalette.cpp 4612 2011-07-27 13:14:35Z wschweer $
 //
 //  Copyright (C) 2002-2010 Werner Schweer and others
 //
@@ -19,6 +18,7 @@
 //=============================================================================
 
 #include "palette.h"
+#include "menus.h"
 #include "textpalette.h"
 #include "icons.h"
 #include "libmscore/text.h"
@@ -569,30 +569,7 @@ TextPalette::TextPalette(QWidget* parent)
       QSplitter* ws = new QSplitter;
       lws = new QListWidget;
 
-      ScoreFont* scoreFont = ScoreFont::fontFactory("Bravura");
-      QFile fi(scoreFont->fontPath() + "ranges.json");
-      if (!fi.open(QIODevice::ReadOnly))
-            qDebug("ScoreFont: open ranges file <%s> failed", qPrintable(fi.fileName()));
-      QJsonParseError error;
-      QJsonObject o = QJsonDocument::fromJson(fi.readAll(), &error).object();
-      if (error.error != QJsonParseError::NoError)
-            qDebug("Json parse error in <%s>(offset: %d): %s", qPrintable(fi.fileName()),
-               error.offset, qPrintable(error.errorString()));
-	int i = 0;
-      QStringList smuflRangeNames;
-      for (auto s : o.keys()) {
-            QJsonObject range = o.value(s).toObject();
-            QString desc = range.value("description").toString();
-            QJsonArray glyphs = range.value("glyphs").toArray();
-            if (glyphs.size() > 0) {
-                  for (QJsonValue g : glyphs)
-                        smuflMap[i].append(g.toString());
-                  smuflRangeNames.append(desc);
-                  i++;
-                  }
-            }
-
-      lws->addItems(smuflRangeNames);
+      lws->addItems(smuflRanges()->keys());
       lws->setCurrentRow(0);
 
       ws->addWidget(lws);
@@ -606,7 +583,7 @@ TextPalette::TextPalette(QWidget* parent)
       QSplitter* wu = new QSplitter;
       lwu = new QListWidget;
       lwu->setSortingEnabled(true);
-      for (i = 0; i < unicodeRangeNames.length(); i++) {
+      for (int i = 0; i < unicodeRangeNames.length(); i++) {
             QListWidgetItem* newItem = new QListWidgetItem(qApp->translate("accidental", unicodeRangeNames.at(i).toUtf8().constData()));
             newItem->setData(Qt::UserRole, i);
             lwu->addItem(newItem);
@@ -835,7 +812,8 @@ void TextPalette::populateCommon()
 void TextPalette::populateSmufl()
       {
       int row = lws->currentRow();
-      QStringList smuflNames = smuflMap[row];
+      QString key = smuflRanges()->keys().at(row);
+      QStringList smuflNames = (*smuflRanges())[key];
 
       pSmufl->clear();
       for (QString name : smuflNames) {
@@ -866,7 +844,7 @@ void TextPalette::populateUnicode()
 //   setText
 //---------------------------------------------------------
 
-void TextPalette::setText(Text* te)
+void TextPalette::setText(TextBase* te)
       {
       _textElement = te;
       }
