@@ -13,6 +13,7 @@
 #include "musescore.h"
 #include "libmscore/score.h"
 #include "libmscore/undo.h"
+#include "network/loginmanager.h"
 #include "uploadscoredialog.h"
 
 namespace Ms {
@@ -25,7 +26,7 @@ void MuseScore::showUploadScoreDialog()
       {
       if (!currentScore())
             return;
-      if (!currentScore()->sanityCheck()) {
+      if (!currentScore()->sanityCheck(QString())) {
             QMessageBox msgBox;
             msgBox.setWindowTitle(QObject::tr("Upload Error"));
             msgBox.setText(tr("This score cannot be saved online. Please fix the corrupted measures and try again."));
@@ -57,7 +58,7 @@ UploadScoreDialog::UploadScoreDialog(LoginManager* loginManager)
 
       license->addItem(tr("All Rights reserved"), "all-rights-reserved");
       license->addItem(tr("Creative Commons Attribution"), "cc-by");
-	license->addItem(tr("Creative Commons Attribution No Derivative Works"), "cc-by-nd");
+      license->addItem(tr("Creative Commons Attribution No Derivative Works"), "cc-by-nd");
       license->addItem(tr("Creative Commons Attribution Share Alike"), "cc-by-sa");
       license->addItem(tr("Creative Commons Attribution Noncommercial"), "cc-by-nc");
       license->addItem(tr("Creative Commons Attribution Noncommercial Non Derivate Works"), "cc-by-nc-nd");
@@ -94,7 +95,6 @@ UploadScoreDialog::UploadScoreDialog(LoginManager* loginManager)
       connect(updateExistingCb, SIGNAL(toggled(bool)), lblChanges, SLOT(setVisible(bool)));
       connect(updateExistingCb, SIGNAL(toggled(bool)), changes, SLOT(setVisible(bool)));
 
-
       connect(buttonBox,   SIGNAL(clicked(QAbstractButton*)), SLOT(buttonBoxClicked(QAbstractButton*)));
       chkSignoutOnExit->setVisible(false);  // currently unused, so hide it
       _loginManager = loginManager;
@@ -132,7 +132,7 @@ void UploadScoreDialog::upload(int nid)
            QMessageBox::critical(this, tr("Missing title"), tr("Please provide a title"));
            return;
            }
-     Score* score = mscore->currentScore()->rootScore();
+     Score* score = mscore->currentScore()->masterScore();
      QString path = QDir::tempPath() + QString("/temp_%1.mscz").arg(qrand() % 100000);
      if(mscore->saveAs(score, true, path, "mscz")) {
            QString licenseString = license->currentData().toString();
@@ -149,7 +149,7 @@ void UploadScoreDialog::uploadSuccess(const QString& url, const QString& nid, co
       {
       setVisible(false);
       _url = url;
-      Score* score = mscore->currentScore()->rootScore();
+      Score* score = mscore->currentScore()->masterScore();
       QMap<QString, QString>  metatags = score->metaTags();
       if (metatags.value("source") != url) {
             metatags.insert("source", url);
@@ -208,7 +208,7 @@ void UploadScoreDialog::showOrHideUploadAudio()
 void UploadScoreDialog::display()
       {
       lblUsername->setText(_loginManager->userName());
-      QString source = mscore->currentScore()->rootScore()->metaTag("source");
+      QString source = mscore->currentScore()->masterScore()->metaTag("source");
       if (!source.isEmpty()) {
             QStringList sl = source.split("/");
             if (sl.length() > 0) {
@@ -217,7 +217,7 @@ void UploadScoreDialog::display()
                   int nid = nidString.toInt(&ok);
                   if (ok) {
                         _nid = nid;
-                        _loginManager->getScore(nid);
+                        _loginManager->getScoreInfo(nid);
                         return;
                         }
                   }
@@ -248,7 +248,7 @@ void UploadScoreDialog::onGetScoreSuccess(const QString &t, const QString &desc,
       changes->clear();
       updateExistingCb->setChecked(true);
       updateExistingCb->setVisible(true);
-      linkToScore->setText(tr("[%1link%2]")
+      linkToScore->setText(tr("[%1Link%2]")
                            .arg("<a href=\"" + url + "\">")
                            .arg("</a>"));
       showOrHideUploadAudio();
