@@ -1,7 +1,6 @@
 //=============================================================================
 //  MuseScore
 //  Music Composition & Notation
-//  $Id:$
 //
 //  Copyright (C) 2012 Werner Schweer
 //
@@ -28,6 +27,7 @@
 #include "libmscore/stafftype.h"
 #include "libmscore/sym.h"
 #include "libmscore/chordline.h"
+#include "libmscore/sym.h"
 #include "mtest/testutils.h"
 
 #define DIR QString("libmscore/parts/")
@@ -42,25 +42,26 @@ class TestParts : public QObject, public MTest
       {
       Q_OBJECT
 
-      void createParts(Score* score);
+      void createParts(MasterScore* score);
       void testPartCreation(const QString& test);
 
-      Score* doAddBreath();
-      Score* doRemoveBreath();
-      Score* doAddFingering();
-      Score* doRemoveFingering();
-      Score* doAddSymbol();
-      Score* doRemoveSymbol();
-      Score* doAddChordline();
-      Score* doRemoveChordline();
-//      Score* doAddImage();
-//      Score* doRemoveImage();
+      MasterScore* doAddBreath();
+      MasterScore* doRemoveBreath();
+      MasterScore* doAddFingering();
+      MasterScore* doRemoveFingering();
+      MasterScore* doAddSymbol();
+      MasterScore* doRemoveSymbol();
+      MasterScore* doAddChordline();
+      MasterScore* doRemoveChordline();
+//      MasterScore* doAddImage();
+//      MasterScore* doRemoveImage();
 
    private slots:
       void initTestCase();
 
       void createPart1();
       void createPart2();
+      void voicesExcerpt();
 
       void createPartBreath();
       void addBreath();
@@ -104,8 +105,8 @@ class TestParts : public QObject, public MTest
 
       void appendMeasure();
       void insertMeasure();
-      void styleScore();
-      void styleScoreReload();
+//      void styleScore();
+//      void styleScoreReload();
 //      void stylePartDefault();
 //      void styleScoreDefault();
 //      void staffStyles();
@@ -131,7 +132,7 @@ void TestParts::initTestCase()
 //   createParts
 //---------------------------------------------------------
 
-void TestParts::createParts(Score* score)
+void TestParts::createParts(MasterScore* score)
       {
       //
       // create first part
@@ -140,16 +141,16 @@ void TestParts::createParts(Score* score)
       parts.append(score->parts().at(0));
       Score* nscore = new Score(score);
 
-      Excerpt ex(score);
-      ex.setPartScore(nscore);
-      ex.setTitle(parts.front()->longName());
-      ex.setParts(parts);
-      ::createExcerpt(&ex);
+      Excerpt* ex = new Excerpt(score);
+      ex->setPartScore(nscore);
+      ex->setParts(parts);
+      ex->setTitle(parts.front()->partName());
+      Excerpt::createExcerpt(ex);
+      score->excerpts().append(ex);
+//      ex->setTitle(parts.front()->longName());
       QVERIFY(nscore);
 
-      nscore->setName(parts.front()->partName());
-      score->undo(new AddExcerpt(nscore));
-      nscore->style()->set(StyleIdx::createMultiMeasureRests, true);
+//      nscore->setName(parts.front()->partName());
 
       //
       // create second part
@@ -157,15 +158,80 @@ void TestParts::createParts(Score* score)
       parts.clear();
       parts.append(score->parts().at(1));
       nscore = new Score(score);
-      ex.setTitle(parts.front()->longName());
-      ex.setPartScore(nscore);
-      ex.setParts(parts);
-      ::createExcerpt(&ex);
+
+      ex = new Excerpt(score);
+      ex->setPartScore(nscore);
+      ex->setParts(parts);
+      ex->setTitle(parts.front()->partName());
+      Excerpt::createExcerpt(ex);
+      score->excerpts().append(ex);
+//      ex->setTitle(parts.front()->longName());
       QVERIFY(nscore);
 
-      nscore->setName(parts.front()->partName());
-      score->undo(new AddExcerpt(nscore));
-      nscore->style()->set(StyleIdx::createMultiMeasureRests, true);
+//      nscore->setName(parts.front()->partName());
+
+      score->setExcerptsChanged(true);
+      }
+
+//---------------------------------------------------------
+//   voicesExcerpt
+//---------------------------------------------------------
+
+void TestParts::voicesExcerpt()
+      {
+      MasterScore* score = readScore(DIR + "voices.mscx");
+
+      //
+      // create first part
+      //
+      QList<Part*> parts;
+      QMultiMap<int, int> trackList;
+      parts.append(score->parts().at(0));
+      Score* nscore = new Score(score);
+
+      trackList.insert(1, 0);
+      trackList.insert(2, 1);
+      trackList.insert(4, 4);
+
+      Excerpt* ex = new Excerpt(score);
+      ex->setPartScore(nscore);
+      nscore->setExcerpt(ex);
+      score->excerpts().append(ex);
+      ex->setTitle(parts.front()->longName());
+      ex->setParts(parts);
+      ex->setTracks(trackList);
+      Excerpt::createExcerpt(ex);
+      QVERIFY(nscore);
+
+//      nscore->setName(parts.front()->partName());
+
+      //
+      // create second part
+      //
+      parts.clear();
+      parts.append(score->parts().at(1));
+      nscore = new Score(score);
+
+      trackList.clear();
+      trackList.insert(11, 0);
+
+      ex = new Excerpt(score);
+      ex->setPartScore(nscore);
+      nscore->setExcerpt(ex);
+      score->excerpts().append(ex);
+      ex->setTitle(parts.front()->longName());
+      ex->setParts(parts);
+      ex->setTracks(trackList);
+      Excerpt::createExcerpt(ex);
+      QVERIFY(nscore);
+
+//      nscore->setName(parts.front()->partName());
+
+      score->setExcerptsChanged(true);
+
+      QVERIFY(saveCompareScore(score, "voices.mscx", DIR + "voices-ref.mscx"));
+
+      delete score;
       }
 
 //---------------------------------------------------------
@@ -174,8 +240,7 @@ void TestParts::createParts(Score* score)
 
 void TestParts::testPartCreation(const QString& test)
       {
-      Score* score = readScore(DIR + test + ".mscx");
-      score->doLayout();
+      MasterScore* score = readScore(DIR + test + ".mscx");
       QVERIFY(score);
       QVERIFY(saveCompareScore(score, test + "-1.mscx", DIR + test + ".mscx"));
       createParts(score);
@@ -189,20 +254,18 @@ void TestParts::testPartCreation(const QString& test)
 
 void TestParts::appendMeasure()
       {
-      Score* score = readScore(DIR + "part-all.mscx");
-      score->doLayout();
+      MasterScore* score = readScore(DIR + "part-all.mscx");
 
       QVERIFY(score);
       createParts(score);
 
       score->startCmd();
-      score->insertMeasure(Element::Type::MEASURE, 0);
+      score->insertMeasure(ElementType::MEASURE, 0);
       score->endCmd();
 
       QVERIFY(saveCompareScore(score, "part-all-appendmeasures.mscx", DIR + "part-all-appendmeasures.mscx"));
 
-      score->undo()->undo();
-      score->endUndoRedo();
+      score->undoRedo(true, 0);
 
       QVERIFY(saveCompareScore(score, "part-all-uappendmeasures.mscx", DIR + "part-all-uappendmeasures.mscx"));
       delete score;
@@ -214,36 +277,34 @@ void TestParts::appendMeasure()
 
 void TestParts::insertMeasure()
       {
-      Score* score = readScore(DIR + "part-all.mscx");
-      score->doLayout();
+      MasterScore* score = readScore(DIR + "part-all.mscx");
       QVERIFY(score);
       createParts(score);
 
       score->startCmd();
       Measure* m = score->firstMeasure();
-      score->insertMeasure(Element::Type::MEASURE, m);
+      score->insertMeasure(ElementType::MEASURE, m);
       score->endCmd();
 
       // QVERIFY(saveCompareScore(score, "part-all-insertmeasures.mscx", DIR + "part-all-insertmeasures.mscx"));
 
-      score->undo()->undo();
-      score->endUndoRedo();
+      score->undoRedo(true, 0);
 
       QVERIFY(saveCompareScore(score, "part-all-uinsertmeasures.mscx", DIR + "part-all-uinsertmeasures.mscx"));
       delete score;
       }
 
+#if 0
 //---------------------------------------------------------
 //   styleScore
 //---------------------------------------------------------
 
 void TestParts::styleScore()
       {
-      Score* score = readScore(DIR + "partStyle.mscx");
-      score->doLayout();
+      MasterScore* score = readScore(DIR + "partStyle.mscx");
       QVERIFY(score);
       createParts(score);
-      score->style()->set(StyleIdx::clefLeftMargin, 4.0);
+      score->style().set(StyleIdx::clefLeftMargin, 4.0);
       QVERIFY(saveCompareScore(score, "partStyle-score-test.mscx", DIR + "partStyle-score-ref.mscx"));
       delete score;
       }
@@ -254,10 +315,11 @@ void TestParts::styleScore()
 
 void TestParts::styleScoreReload()
       {
-      Score* partScore = readScore(DIR + "partStyle-score-reload.mscx");
+      MasterScore* partScore = readScore(DIR + "partStyle-score-reload.mscx");
       QVERIFY(saveCompareScore(partScore, "partStyle-score-reload-test.mscx", DIR + "partStyle-score-reload-ref.mscx"));
       delete partScore;
       }
+#endif
 
 //---------------------------------------------------------
 //   stylePartDefault
@@ -266,8 +328,7 @@ void TestParts::styleScoreReload()
 #if 0
 void TestParts::stylePartDefault()
       {
-      Score* score = readScore(DIR + "partStyle.mscx");
-      score->doLayout();
+      MasterScore* score = readScore(DIR + "partStyle.mscx");
       QVERIFY(score);
       // TODO: set defaultStyleForParts
       MScore::_defaultStyleForParts = new MStyle();
@@ -286,8 +347,7 @@ void TestParts::stylePartDefault()
 
 void TestParts::styleScoreDefault()
       {
-      Score* score = readScore(DIR + "partStyle.mscx");
-      score->doLayout();
+      MasterScore* score = readScore(DIR + "partStyle.mscx");
       QVERIFY(score);
       // TODO: set defaultStyle
       createParts(score);
@@ -339,23 +399,18 @@ void TestParts::createPartImage()
 //    doAddBreath
 //---------------------------------------------------------
 
-Score* TestParts::doAddBreath()
+MasterScore* TestParts::doAddBreath()
       {
-      Score* score = readScore(DIR + "part-empty-parts.mscx");
-      score->doLayout();
-
-      foreach(Excerpt* e, score->excerpts())
-            e->partScore()->doLayout();
+      MasterScore* score = readScore(DIR + "part-empty-parts.mscx");
 
       Measure* m   = score->firstMeasure();
-      Segment* s   = m->tick2segment(MScore::division);
-      Ms::Chord* chord = static_cast<Ms::Chord*>(s->element(0));
+      Segment* s   = m->tick2segment(Fraction(1,4));
+      Ms::Chord* chord = toChord(s->element(0));
       Note* note   = chord->upNote();
-      DropData dd;
-      dd.view = 0;
+      EditData dd(0);
       Breath* b = new Breath(score);
-      b->setBreathType(0);
-      dd.element = b;
+      b->setSymId(SymId::breathMarkComma);
+      dd.dropElement = b;
 
       score->startCmd();
       note->drop(dd);
@@ -370,7 +425,7 @@ Score* TestParts::doAddBreath()
 
 void TestParts::addBreath()
       {
-      Score* score = doAddBreath();
+      MasterScore* score = doAddBreath();
       QVERIFY(saveCompareScore(score, "part-breath-add.mscx", DIR + "part-breath-add.mscx"));
       delete score;
       }
@@ -381,10 +436,9 @@ void TestParts::addBreath()
 
 void TestParts::undoAddBreath()
       {
-      Score* score = doAddBreath();
+      MasterScore* score = doAddBreath();
 
-      score->undo()->undo();
-      score->endUndoRedo();
+      score->undoRedo(true, 0);
 
       QVERIFY(saveCompareScore(score, "part-breath-uadd.mscx", DIR + "part-breath-uadd.mscx"));
       delete score;
@@ -396,14 +450,10 @@ void TestParts::undoAddBreath()
 
 void TestParts::undoRedoAddBreath()
       {
-      Score* score = doAddBreath();
+      MasterScore* score = doAddBreath();
 
-      score->undo()->undo();
-      score->endUndoRedo();
-      score->doLayout();
-
-      score->undo()->redo();
-      score->endUndoRedo();
+      score->undoRedo(true, 0);
+      score->undoRedo(false, 0);
 
       QVERIFY(saveCompareScore(score, "part-breath-uradd.mscx", DIR + "part-breath-uradd.mscx"));
       delete score;
@@ -413,21 +463,18 @@ void TestParts::undoRedoAddBreath()
 //   doRemoveBreath
 //---------------------------------------------------------
 
-Score* TestParts::doRemoveBreath()
+MasterScore* TestParts::doRemoveBreath()
       {
-      Score* score = readScore(DIR + "part-breath-add.mscx");
-      score->doLayout();
-      foreach(Excerpt* e, score->excerpts())
-            e->partScore()->doLayout();
+      MasterScore* score = readScore(DIR + "part-breath-add.mscx");
 
       Measure* m   = score->firstMeasure();
-      Segment* s   = m->first()->next(Segment::Type::Breath);
+      Segment* s   = m->first()->next(SegmentType::Breath);
       Breath*  b   = static_cast<Breath*>(s->element(0));
 
       score->select(b);
       score->startCmd();
       score->cmdDeleteSelection();
-      score->setLayoutAll(true);
+      score->setLayoutAll();
       score->endCmd();
       return score;
       }
@@ -438,7 +485,7 @@ Score* TestParts::doRemoveBreath()
 
 void TestParts::removeBreath()
       {
-      Score* score = doRemoveBreath();
+      MasterScore* score = doRemoveBreath();
       QVERIFY(saveCompareScore(score, "part-breath-del.mscx", DIR + "part-breath-del.mscx"));
       delete score;
       }
@@ -449,9 +496,8 @@ void TestParts::removeBreath()
 
 void TestParts::undoRemoveBreath()
       {
-      Score* score = doRemoveBreath();
-      score->undo()->undo();
-      score->endUndoRedo();
+      MasterScore* score = doRemoveBreath();
+      score->undoRedo(true, 0);
       QVERIFY(saveCompareScore(score, "part-breath-udel.mscx", DIR + "part-breath-udel.mscx"));
       delete score;
       }
@@ -462,13 +508,9 @@ void TestParts::undoRemoveBreath()
 
 void TestParts::undoRedoRemoveBreath()
       {
-      Score* score = doRemoveBreath();
-      score->undo()->undo();
-      score->endUndoRedo();
-      score->doLayout();
-
-      score->undo()->redo();
-      score->endUndoRedo();
+      MasterScore* score = doRemoveBreath();
+      score->undoRedo(true, 0);
+      score->undoRedo(false, 0);
 
       QVERIFY(saveCompareScore(score, "part-breath-urdel.mscx", DIR + "part-breath-urdel.mscx"));
       delete score;
@@ -478,22 +520,18 @@ void TestParts::undoRedoRemoveBreath()
 //   doAddFingering
 //---------------------------------------------------------
 
-Score* TestParts::doAddFingering()
+MasterScore* TestParts::doAddFingering()
       {
-      Score* score = readScore(DIR + "part-empty-parts.mscx");
-      score->doLayout();
-      foreach(Excerpt* e, score->excerpts())
-            e->partScore()->doLayout();
+      MasterScore* score = readScore(DIR + "part-empty-parts.mscx");
 
       Measure* m   = score->firstMeasure();
-      Segment* s   = m->tick2segment(MScore::division);
+      Segment* s   = m->tick2segment(Fraction(1,4));
       Ms::Chord* chord = static_cast<Ms::Chord*>(s->element(0));
       Note* note   = chord->upNote();
-      DropData dd;
-      dd.view = 0;
+      EditData dd(0);
       Fingering* b = new Fingering(score);
       b->setXmlText("3");
-      dd.element = b;
+      dd.dropElement = b;
 
       score->startCmd();
       note->drop(dd);
@@ -507,7 +545,7 @@ Score* TestParts::doAddFingering()
 
 void TestParts::addFingering()
       {
-      Score* score = doAddFingering();
+      MasterScore* score = doAddFingering();
       QVERIFY(saveCompareScore(score, "part-fingering-add.mscx", DIR + "part-fingering-add.mscx"));
       delete score;
       }
@@ -518,9 +556,8 @@ void TestParts::addFingering()
 
 void TestParts::undoAddFingering()
       {
-      Score* score = doAddFingering();
-      score->undo()->undo();
-      score->endUndoRedo();
+      MasterScore* score = doAddFingering();
+      score->undoRedo(true, 0);
       QVERIFY(saveCompareScore(score, "part-fingering-uadd.mscx", DIR + "part-fingering-uadd.mscx"));
       delete score;
       }
@@ -531,11 +568,9 @@ void TestParts::undoAddFingering()
 
 void TestParts::undoRedoAddFingering()
       {
-      Score* score = doAddFingering();
-      score->undo()->undo();
-      score->endUndoRedo();
-      score->undo()->redo();
-      score->endUndoRedo();
+      MasterScore* score = doAddFingering();
+      score->undoRedo(true, 0);
+      score->undoRedo(false, 0);
       QVERIFY(saveCompareScore(score, "part-fingering-uradd.mscx", DIR + "part-fingering-uradd.mscx"));
       delete score;
       }
@@ -544,20 +579,17 @@ void TestParts::undoRedoAddFingering()
 //   doRemoveFingering
 //---------------------------------------------------------
 
-Score* TestParts::doRemoveFingering()
+MasterScore* TestParts::doRemoveFingering()
       {
-      Score* score = readScore(DIR + "part-fingering-parts.mscx");
-      score->doLayout();
-      foreach(Excerpt* e, score->excerpts())
-            e->partScore()->doLayout();
+      MasterScore* score = readScore(DIR + "part-fingering-parts.mscx");
 
       Measure* m   = score->firstMeasure();
-      Segment* s   = m->first()->next(Segment::Type::ChordRest);
+      Segment* s   = m->first()->next(SegmentType::ChordRest);
       Ms::Chord* chord = static_cast<Ms::Chord*>(s->element(0));
       Note* note   = chord->upNote();
       Element* fingering = 0;
       foreach(Element* e, note->el()) {
-            if (e->type() == Element::Type::FINGERING) {
+            if (e->type() == ElementType::FINGERING) {
                   fingering = e;
                   break;
                   }
@@ -566,7 +598,7 @@ Score* TestParts::doRemoveFingering()
 
       score->startCmd();
       score->cmdDeleteSelection();
-      score->setLayoutAll(true);
+      score->setLayoutAll();
       score->endCmd();
       return score;
       }
@@ -577,7 +609,7 @@ Score* TestParts::doRemoveFingering()
 
 void TestParts::removeFingering()
       {
-      Score* score = doRemoveFingering();
+      MasterScore* score = doRemoveFingering();
       QVERIFY(saveCompareScore(score, "part-fingering-del.mscx", DIR + "part-fingering-del.mscx"));
       delete score;
       }
@@ -588,9 +620,8 @@ void TestParts::removeFingering()
 
 void TestParts::undoRemoveFingering()
       {
-      Score* score = doRemoveFingering();
-      score->undo()->undo();
-      score->endUndoRedo();
+      MasterScore* score = doRemoveFingering();
+      score->undoRedo(true, 0);
       QVERIFY(saveCompareScore(score, "part-fingering-udel.mscx", DIR + "part-fingering-udel.mscx"));
       delete score;
       }
@@ -601,11 +632,9 @@ void TestParts::undoRemoveFingering()
 
 void TestParts::undoRedoRemoveFingering()
       {
-      Score* score = doRemoveFingering();
-      score->undo()->undo();
-      score->endUndoRedo();
-      score->undo()->redo();
-      score->endUndoRedo();
+      MasterScore* score = doRemoveFingering();
+      score->undoRedo(true, 0);
+      score->undoRedo(false, 0);
       QVERIFY(saveCompareScore(score, "part-fingering-urdel.mscx", DIR + "part-fingering-urdel.mscx"));
       delete score;
       }
@@ -614,22 +643,18 @@ void TestParts::undoRedoRemoveFingering()
 //   doAddSymbol
 //---------------------------------------------------------
 
-Score* TestParts::doAddSymbol()
+MasterScore* TestParts::doAddSymbol()
       {
-      Score* score = readScore(DIR + "part-empty-parts.mscx");
-      score->doLayout();
-      foreach(Excerpt* e, score->excerpts())
-            e->partScore()->doLayout();
+      MasterScore* score = readScore(DIR + "part-empty-parts.mscx");
 
       Measure* m   = score->firstMeasure();
-      Segment* s   = m->tick2segment(MScore::division);
+      Segment* s   = m->tick2segment(Fraction(1,4));
       Ms::Chord* chord = static_cast<Ms::Chord*>(s->element(0));
       Note* note   = chord->upNote();
-      DropData dd;
-      dd.view = 0;
+      EditData dd(0);
       Symbol* b  = new Symbol(score);
       b->setSym(SymId::gClef);
-      dd.element = b;
+      dd.dropElement = b;
 
       score->startCmd();
       note->drop(dd);
@@ -643,7 +668,7 @@ Score* TestParts::doAddSymbol()
 
 void TestParts::addSymbol()
       {
-      Score* score = doAddSymbol();
+      MasterScore* score = doAddSymbol();
       QVERIFY(saveCompareScore(score, "part-symbol-add.mscx", DIR + "part-symbol-add.mscx"));
       delete score;
       }
@@ -654,9 +679,8 @@ void TestParts::addSymbol()
 
 void TestParts::undoAddSymbol()
       {
-      Score* score = doAddSymbol();
-      score->undo()->undo();
-      score->endUndoRedo();
+      MasterScore* score = doAddSymbol();
+      score->undoRedo(true, 0);
       QVERIFY(saveCompareScore(score, "part-symbol-uadd.mscx", DIR + "part-symbol-uadd.mscx"));
       delete score;
       }
@@ -667,11 +691,9 @@ void TestParts::undoAddSymbol()
 
 void TestParts::undoRedoAddSymbol()
       {
-      Score* score = doAddSymbol();
-      score->undo()->undo();
-      score->endUndoRedo();
-      score->undo()->redo();
-      score->endUndoRedo();
+      MasterScore* score = doAddSymbol();
+      score->undoRedo(true, 0);
+      score->undoRedo(false, 0);
       QVERIFY(saveCompareScore(score, "part-symbol-uradd.mscx", DIR + "part-symbol-uradd.mscx"));
       delete score;
       }
@@ -680,20 +702,17 @@ void TestParts::undoRedoAddSymbol()
 //   doRemoveSymbol
 //---------------------------------------------------------
 
-Score* TestParts::doRemoveSymbol()
+MasterScore* TestParts::doRemoveSymbol()
       {
-      Score* score = readScore(DIR + "part-symbol-parts.mscx");
-      score->doLayout();
-      foreach(Excerpt* e, score->excerpts())
-            e->partScore()->doLayout();
+      MasterScore* score = readScore(DIR + "part-symbol-parts.mscx");
 
       Measure* m   = score->firstMeasure();
-      Segment* s   = m->first()->next(Segment::Type::ChordRest);
+      Segment* s   = m->first()->next(SegmentType::ChordRest);
       Ms::Chord* chord = static_cast<Ms::Chord*>(s->element(0));
       Note* note   = chord->upNote();
       Element* se = 0;
       foreach(Element* e, note->el()) {
-            if (e->type() == Element::Type::SYMBOL) {
+            if (e->type() == ElementType::SYMBOL) {
                   se = e;
                   break;
                   }
@@ -702,7 +721,7 @@ Score* TestParts::doRemoveSymbol()
 
       score->startCmd();
       score->cmdDeleteSelection();
-      score->setLayoutAll(true);
+      score->setLayoutAll();
       score->endCmd();
       return score;
       }
@@ -713,7 +732,7 @@ Score* TestParts::doRemoveSymbol()
 
 void TestParts::removeSymbol()
       {
-      Score* score = doRemoveSymbol();
+      MasterScore* score = doRemoveSymbol();
       QVERIFY(saveCompareScore(score, "part-symbol-del.mscx", DIR + "part-symbol-del.mscx"));
       delete score;
       }
@@ -724,9 +743,8 @@ void TestParts::removeSymbol()
 
 void TestParts::undoRemoveSymbol()
       {
-      Score* score = doRemoveSymbol();
-      score->undo()->undo();
-      score->endUndoRedo();
+      MasterScore* score = doRemoveSymbol();
+      score->undoRedo(true, 0);
       QVERIFY(saveCompareScore(score, "part-symbol-udel.mscx", DIR + "part-symbol-udel.mscx"));
       delete score;
       }
@@ -737,11 +755,9 @@ void TestParts::undoRemoveSymbol()
 
 void TestParts::undoRedoRemoveSymbol()
       {
-      Score* score = doRemoveSymbol();
-      score->undo()->undo();
-      score->endUndoRedo();
-      score->undo()->redo();
-      score->endUndoRedo();
+      MasterScore* score = doRemoveSymbol();
+      score->undoRedo(true, 0);
+      score->undoRedo(false, 0);
       QVERIFY(saveCompareScore(score, "part-symbol-urdel.mscx", DIR + "part-symbol-urdel.mscx"));
       delete score;
       }
@@ -750,22 +766,18 @@ void TestParts::undoRedoRemoveSymbol()
 //   doAddChordline
 //---------------------------------------------------------
 
-Score* TestParts::doAddChordline()
+MasterScore* TestParts::doAddChordline()
       {
-      Score* score = readScore(DIR + "part-empty-parts.mscx");
-      score->doLayout();
-      foreach(Excerpt* e, score->excerpts())
-            e->partScore()->doLayout();
+      MasterScore* score = readScore(DIR + "part-empty-parts.mscx");
 
       Measure* m   = score->firstMeasure();
-      Segment* s   = m->tick2segment(MScore::division);
+      Segment* s   = m->tick2segment(Fraction(1,4));
       Ms::Chord* chord = static_cast<Ms::Chord*>(s->element(0));
       Note* note   = chord->upNote();
-      DropData dd;
-      dd.view = 0;
+      EditData dd(0);
       ChordLine* b  = new ChordLine(score);
       b->setChordLineType(ChordLineType::FALL);
-      dd.element = b;
+      dd.dropElement = b;
 
       score->startCmd();
       note->drop(dd);
@@ -779,7 +791,7 @@ Score* TestParts::doAddChordline()
 
 void TestParts::addChordline()
       {
-      Score* score = doAddChordline();
+      MasterScore* score = doAddChordline();
       QVERIFY(saveCompareScore(score, "part-chordline-add.mscx", DIR + "part-chordline-add.mscx"));
       delete score;
       }
@@ -790,9 +802,8 @@ void TestParts::addChordline()
 
 void TestParts::undoAddChordline()
       {
-      Score* score = doAddChordline();
-      score->undo()->undo();
-      score->endUndoRedo();
+      MasterScore* score = doAddChordline();
+      score->undoRedo(true, 0);
       QVERIFY(saveCompareScore(score, "part-chordline-uadd.mscx", DIR + "part-chordline-uadd.mscx"));
       delete score;
       }
@@ -803,34 +814,28 @@ void TestParts::undoAddChordline()
 
 void TestParts::undoRedoAddChordline()
       {
-      Score* score = doAddChordline();
-      score->undo()->undo();
-      score->endUndoRedo();
-      score->undo()->redo();
-      score->endUndoRedo();
+      MasterScore* score = doAddChordline();
+      score->undoRedo(true, 0);
+      score->undoRedo(false, 0);
       QVERIFY(saveCompareScore(score, "part-chordline-uradd.mscx", DIR + "part-chordline-uradd.mscx"));
       delete score;
       }
 
-
 //---------------------------------------------------------
 //   doRemoveChordline
 //---------------------------------------------------------
 
-Score* TestParts::doRemoveChordline()
+MasterScore* TestParts::doRemoveChordline()
       {
-      Score* score = readScore(DIR + "part-chordline-parts.mscx");
-      score->doLayout();
-      foreach(Excerpt* e, score->excerpts())
-            e->partScore()->doLayout();
+      MasterScore* score = readScore(DIR + "part-chordline-parts.mscx");
 
       Measure* m   = score->firstMeasure();
-      Segment* s   = m->first()->next(Segment::Type::ChordRest);
+      Segment* s   = m->first()->next(SegmentType::ChordRest);
       Ms::Chord* chord = static_cast<Ms::Chord*>(s->element(0));
 
       Element* se = 0;
       foreach(Element* e, chord->el()) {
-            if (e->type() == Element::Type::CHORDLINE) {
+            if (e->type() == ElementType::CHORDLINE) {
                   se = e;
                   break;
                   }
@@ -839,7 +844,7 @@ Score* TestParts::doRemoveChordline()
 
       score->startCmd();
       score->cmdDeleteSelection();
-      score->setLayoutAll(true);
+      score->setLayoutAll();
       score->endCmd();
       return score;
       }
@@ -850,7 +855,7 @@ Score* TestParts::doRemoveChordline()
 
 void TestParts::removeChordline()
       {
-      Score* score = doRemoveChordline();
+      MasterScore* score = doRemoveChordline();
       QVERIFY(saveCompareScore(score, "part-chordline-del.mscx", DIR + "part-chordline-del.mscx"));
       delete score;
       }
@@ -861,10 +866,8 @@ void TestParts::removeChordline()
 
 void TestParts::undoRemoveChordline()
       {
-      Score* score = doRemoveChordline();
-      score->undo()->undo();
-      score->endUndoRedo();
-//      score->doLayout();
+      MasterScore* score = doRemoveChordline();
+      score->undoRedo(true, 0);
       QVERIFY(saveCompareScore(score, "part-chordline-udel.mscx", DIR + "part-chordline-udel.mscx"));
       delete score;
       }
@@ -875,11 +878,9 @@ void TestParts::undoRemoveChordline()
 
 void TestParts::undoRedoRemoveChordline()
       {
-      Score* score = doRemoveChordline();
-      score->undo()->undo();
-      score->endUndoRedo();
-      score->undo()->redo();
-      score->endUndoRedo();
+      MasterScore* score = doRemoveChordline();
+      score->undoRedo(true, 0);
+      score->undoRedo(false, 0);
       QVERIFY(saveCompareScore(score, "part-chordline-urdel.mscx", DIR + "part-chordline-urdel.mscx"));
       delete score;
       }
@@ -888,22 +889,18 @@ void TestParts::undoRedoRemoveChordline()
 //   doAddImage
 //---------------------------------------------------------
 
-Score* TestParts::doAddImage()
+MasterScore* TestParts::doAddImage()
       {
-      Score* score = readScore(DIR + "part1-2o.mscx");
-      score->doLayout();
-      foreach(Excerpt* e, score->excerpts())
-            e->partScore()->doLayout();
+      MasterScore* score = readScore(DIR + "part1-2o.mscx");
 
       Measure* m   = score->firstMeasure();
       Segment* s   = m->tick2segment(MScore::division);
       Ms::Chord* chord = static_cast<Ms::Chord*>(s->element(0));
       Note* note   = chord->upNote();
-      DropData dd;
-      dd.view = 0;
+      EditData dd(0);
       RasterImage* b = new RasterImage(score);
       b->load(DIR + "schnee.png");
-      dd.element = b;
+      dd.dropElement = b;
 
       score->startCmd();
       note->drop(dd);
@@ -917,7 +914,7 @@ Score* TestParts::doAddImage()
 
 void TestParts::addImage()
       {
-      Score* score = doAddImage();
+      MasterScore* score = doAddImage();
       QVERIFY(saveCompareScore(score, "part25.mscx", DIR + "part25o.mscx"));
       delete score;
       }
@@ -928,9 +925,8 @@ void TestParts::addImage()
 
 void TestParts::undoAddImage()
       {
-      Score* score = doAddImage();
-      score->undo()->undo();
-      score->endUndoRedo();
+      MasterScore* score = doAddImage();
+      score->undoRedo(true, 0);
       QVERIFY(saveCompareScore(score, "part26.mscx", DIR + "part26o.mscx"));
       delete score;
       }
@@ -941,11 +937,9 @@ void TestParts::undoAddImage()
 
 void TestParts::undoRedoAddImage()
       {
-      Score* score = doAddImage();
-      score->undo()->undo();
-      score->endUndoRedo();
-      score->undo()->redo();
-      score->endUndoRedo();
+      MasterScore* score = doAddImage();
+      score->undoRedo(true, 0);
+      score->undoRedo(false, 0);
       QVERIFY(saveCompareScore(score, "part27.mscx", DIR + "part27o.mscx"));
       delete score;
       }
@@ -954,12 +948,9 @@ void TestParts::undoRedoAddImage()
 //   doRemoveImage
 //---------------------------------------------------------
 
-Score* TestParts::doRemoveImage()
+MasterScore* TestParts::doRemoveImage()
       {
-      Score* score = readScore(DIR + "part12o.mscx");
-      score->doLayout();
-      foreach(Excerpt* e, score->excerpts())
-            e->partScore()->doLayout();
+      MasterScore* score = readScore(DIR + "part12o.mscx");
 
       Measure* m   = score->firstMeasure();
       Segment* s   = m->first()->next(SegChordRest);
@@ -976,7 +967,7 @@ Score* TestParts::doRemoveImage()
 
       score->startCmd();
       score->cmdDeleteSelection();
-      score->setLayoutAll(true);
+      score->setLayoutAll();
       score->endCmd();
       return score;
       }
@@ -987,7 +978,7 @@ Score* TestParts::doRemoveImage()
 
 void TestParts::removeImage()
       {
-      Score* score = doRemoveImage();
+      MasterScore* score = doRemoveImage();
       QVERIFY(saveCompareScore(score, "part28.mscx", DIR + "part28o.mscx"));
       delete score;
       }
@@ -998,9 +989,8 @@ void TestParts::removeImage()
 
 void TestParts::undoRemoveImage()
       {
-      Score* score = doRemoveImage();
-      score->undo()->undo();
-      score->endUndoRedo();
+      MasterScore* score = doRemoveImage();
+      score->undoRedo(true, 0);
       QVERIFY(saveCompareScore(score, "part29.mscx", DIR + "part29o.mscx"));
       delete score;
       }
@@ -1011,11 +1001,9 @@ void TestParts::undoRemoveImage()
 
 void TestParts::undoRedoRemoveImage()
       {
-      Score* score = doRemoveImage();
-      score->undo()->undo();
-      score->endUndoRedo();
-      score->undo()->redo();
-      score->endUndoRedo();
+      MasterScore* score = doRemoveImage();
+      score->undoRedo(true, 0);
+      score->undoRedo(false, 0);
       QVERIFY(saveCompareScore(score, "part30.mscx", DIR + "part30o.mscx"));
       delete score;
       }
@@ -1028,8 +1016,7 @@ void TestParts::undoRedoRemoveImage()
 #if 0
 void TestParts::staffStyles()
       {
-      Score* score = readScore(DIR + "part1.mscx");
-      score->doLayout();
+      MasterScore* score = readScore(DIR + "part1.mscx");
       QVERIFY(score);
 //      int numOfStaffTypes = score->staffTypes().count();
       createParts(score);
