@@ -1,9 +1,8 @@
 //=============================================================================
 //  MuseScore
 //  Music Composition & Notation
-//  $Id:$
 //
-//  Copyright (C) 2012 Werner Schweer
+//  Copyright (C) 2012-2017 Werner Schweer
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2
@@ -29,6 +28,8 @@ class TestText : public QObject, public MTest
       {
       Q_OBJECT
 
+      EditData editData;
+
    private slots:
       void initTestCase();
       void testText();
@@ -60,6 +61,7 @@ class TestText : public QObject, public MTest
 void TestText::initTestCase()
       {
       initMTest();
+
       }
 
 //---------------------------------------------------------
@@ -69,17 +71,17 @@ void TestText::initTestCase()
 void TestText::testDelete()
       {
       Text* text = new Text(score);
-      text->setTextStyle(score->textStyle(TextStyleType::DYNAMICS));
+      text->initSubStyle(SubStyle::DYNAMICS);
 
       text->setPlainText("aaa bbb ccc\nddd eee fff\nggg hhh iii");
       text->layout();
       QCOMPARE(text->xmlText(), QString("aaa bbb ccc\nddd eee fff\nggg hhh iii"));
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToStart();
       QVERIFY(text->movePosition(QTextCursor::Down, QTextCursor::KeepAnchor, 2));
       text->deleteSelectedText();
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("ggg hhh iii"));
       }
 
@@ -91,50 +93,50 @@ void TestText::testDelete()
 void TestText::testText()
       {
       Text* text = new Text(score);
-      text->setTextStyle(score->textStyle(TextStyleType::DYNAMICS));
+      text->initSubStyle(SubStyle::DYNAMICS);
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->layout();
 
       text->moveCursorToEnd();
       text->insertText("a");
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("a"));
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToEnd();
       text->insertText("bc");
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("abc"));
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToEnd();
       text->insertText("d");
       text->insertText("e");
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("abcde"));
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToStart();
       text->insertText("1");
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("1abcde"));
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToStart();
       text->insertText("0");
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("01abcde"));
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToStart();
       text->movePosition(QTextCursor::Right);
       text->movePosition(QTextCursor::Right);
       text->insertText("2");
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("012abcde"));
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToEnd();
       text->movePosition(QTextCursor::Left);
       text->movePosition(QTextCursor::Left);
@@ -142,20 +144,20 @@ void TestText::testText()
       text->movePosition(QTextCursor::Left);
       text->movePosition(QTextCursor::Left);
       text->insertText("3");
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("0123abcde"));
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToEnd();
       text->insertSym(SymId::segno);
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("0123abcde<sym>segno</sym>"));
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToEnd();
       text->movePosition(QTextCursor::Left);
       text->insertText("#");
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("0123abcde#<sym>segno</sym>"));
       }
 
@@ -166,38 +168,38 @@ void TestText::testText()
 void TestText::testSpecialSymbols()
       {
       Text* text = new Text(score);
-      text->setTextStyle(score->textStyle(TextStyleType::DYNAMICS));
+      text->initSubStyle(SubStyle::DYNAMICS);
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->layout();
 
       text->moveCursorToEnd();
       text->insertText("<");
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("&lt;"));
 
       text->selectAll();
       text->deleteSelectedText();
       text->insertText("&");
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("&amp;"));
 
       text->selectAll();
       text->deleteSelectedText();
       text->insertText(">");
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("&gt;"));
 
       text->selectAll();
       text->deleteSelectedText();
       text->insertText("\"");
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("&quot;"));
 
       text->selectAll();
       text->deleteSelectedText();
       text->insertText("&gt;");
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("&amp;gt;"));
 
       text->selectAll();
@@ -205,10 +207,10 @@ void TestText::testSpecialSymbols()
       text->insertText("&&");
       text->moveCursorToEnd();
       text->deletePreviousChar();
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("&amp;"));
       text->deletePreviousChar();
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString(""));
       }
 
@@ -219,45 +221,45 @@ void TestText::testSpecialSymbols()
 void TestText::testPaste()
       {
       Text* text = new Text(score);
-      text->setTextStyle(score->textStyle(TextStyleType::DYNAMICS));
+      text->initSubStyle(SubStyle::DYNAMICS);
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->layout();
       text->moveCursorToEnd();
 
       QApplication::clipboard()->setText("copy & paste");
-      text->paste();
-      text->endEdit();
+      text->paste(0);
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("copy &amp; paste"));
 
       text->selectAll();
       text->deleteSelectedText();
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->layout();
       text->moveCursorToEnd();
       QApplication::clipboard()->setText("copy &aa paste");
-      text->paste();
-      text->endEdit();
+      text->paste(0);
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("copy &amp;aa paste"));
 
       text->selectAll();
       text->deleteSelectedText();
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->layout();
       text->moveCursorToEnd();
       QApplication::clipboard()->setText("&");
-      text->paste();
-      text->endEdit();
+      text->paste(0);
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("&amp;"));
 
       text->selectAll();
       text->deleteSelectedText();
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->layout();
       text->moveCursorToEnd();
       QApplication::clipboard()->setText("&sometext");
-      text->paste();
-      text->endEdit();
+      text->paste(0);
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("&amp;sometext"));
       }
 //---------------------------------------------------------
@@ -267,74 +269,74 @@ void TestText::testPaste()
 void TestText::testTextProperties()
       {
       Text* text = new Text(score);
-      text->setTextStyle(score->textStyle(TextStyleType::STAFF));
+      text->initSubStyle(SubStyle::STAFF);
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->layout();
 
       text->moveCursorToEnd();
       text->insertText("ILoveMuseScore");
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("ILoveMuseScore"));
 
       //select Love and make it bold
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToStart();
       text->movePosition(QTextCursor::Right);
       text->movePosition(QTextCursor::Right, QTextCursor::KeepAnchor, 4);
 
       text->setFormat(FormatId::Bold , true);
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("I<b>Love</b>MuseScore"));
 
       //select Love and unbold it
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToStart();
       text->movePosition(QTextCursor::Right);
       text->movePosition(QTextCursor::Right, QTextCursor::KeepAnchor, 4);
 
       text->setFormat(FormatId::Bold , false);
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("ILoveMuseScore"));
 
       //select Love and make it bold again
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToStart();
       text->movePosition(QTextCursor::Right);
       text->movePosition(QTextCursor::Right, QTextCursor::KeepAnchor, 4);
 
       text->setFormat(FormatId::Bold , true);
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("I<b>Love</b>MuseScore"));
 
       //select veMu and make it bold
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToStart();
       text->movePosition(QTextCursor::Right, QTextCursor::MoveAnchor, 3);
       text->movePosition(QTextCursor::Right, QTextCursor::KeepAnchor, 4);
 
       text->setFormat(FormatId::Bold , true);
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("I<b>LoveMu</b>seScore"));
 
       //select Mu and make it nonbold
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToStart();
       text->movePosition(QTextCursor::Right, QTextCursor::MoveAnchor, 5);
       text->movePosition(QTextCursor::Right, QTextCursor::KeepAnchor, 2);
 
       text->setFormat(FormatId::Bold , false);
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("I<b>Love</b>MuseScore"));
 
       //make veMuse italic
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToStart();
       QVERIFY(text->movePosition(QTextCursor::Right, QTextCursor::MoveAnchor, 3));
       QVERIFY(text->movePosition(QTextCursor::Right, QTextCursor::KeepAnchor, 6));
 
       text->setFormat(FormatId::Italic , true);
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("I<b>Lo<i>ve</i></b><i>Muse</i>Score"));
 
       }
@@ -402,11 +404,11 @@ void TestText::testReadWrite() {
       auto testrw = [](Score* score, Text* t) {
             QBuffer buffer;
             buffer.open(QIODevice::WriteOnly);
-            Xml xml(&buffer);
+            XmlWriter xml(score, &buffer);
             t->write(xml);
             buffer.close();
 
-            XmlReader e(buffer.buffer());
+            XmlReader e(score, buffer.buffer());
             Text* text2 = new Text(score);
             e.readNextStartElement();
             text2->read(e);
@@ -449,15 +451,15 @@ void TestText::testReadWrite() {
 void TestText::testBasicUnicodeDeletePreviousChar()
       {
       Text* text = new Text(score);
-      text->setTextStyle(score->textStyle(TextStyleType::DYNAMICS));
+      text->initSubStyle(SubStyle::DYNAMICS);
 
       text->setPlainText(QString("⟁⟂⟃⟄"));
 
       text->layout();
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToEnd();
       text->deletePreviousChar();
-      text->endEdit();
+      text->endEdit(editData);
 
       QCOMPARE(text->xmlText(), QString("⟁⟂⟃"));
       }
@@ -470,15 +472,15 @@ void TestText::testBasicUnicodeDeletePreviousChar()
 void TestText::testSupplementaryUnicodeDeletePreviousChar()
       {
       Text* text = new Text(score);
-      text->setTextStyle(score->textStyle(TextStyleType::DYNAMICS));
+      text->initSubStyle(SubStyle::DYNAMICS);
 
       text->setPlainText(QString("𝄆𝄆𝄆𝄏𝄏𝄏"));
 
       text->layout();
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToEnd();
       text->deletePreviousChar();
-      text->endEdit();
+      text->endEdit(editData);
 
       QCOMPARE(text->xmlText(), QString("𝄆𝄆𝄆𝄏𝄏"));
       }
@@ -491,11 +493,11 @@ void TestText::testSupplementaryUnicodeDeletePreviousChar()
 void TestText::testMixedTypesDeletePreviousChar()
       {
       Text* text = new Text(score);
-      text->setTextStyle(score->textStyle(TextStyleType::DYNAMICS));
+      text->initSubStyle(SubStyle::DYNAMICS);
 
       text->setXmlText("<sym>cClefSquare</sym>𝄆<sym>repeatLeft</sym><sym>textBlackNoteLongStem</sym><sym>textBlackNoteLongStem</sym><sym>noteheadWhole</sym> ⟂<sym>repeatRight</sym> 𝄇");
       text->layout();
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToEnd();
       text->deletePreviousChar();
       text->deletePreviousChar();
@@ -503,7 +505,7 @@ void TestText::testMixedTypesDeletePreviousChar()
       text->deletePreviousChar();
       text->deletePreviousChar();
       text->deletePreviousChar();
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("<sym>cClefSquare</sym>𝄆<sym>repeatLeft</sym><sym>textBlackNoteLongStem</sym><sym>textBlackNoteLongStem</sym>"));
       }
 
@@ -515,13 +517,13 @@ void TestText::testMixedTypesDeletePreviousChar()
 void TestText::testSupplementaryUnicodeInsert1()
       {
       Text* text = new Text(score);
-      text->setTextStyle(score->textStyle(TextStyleType::DYNAMICS));
+      text->initSubStyle(SubStyle::DYNAMICS);
       text->setPlainText(QString("𝄏"));
       text->layout();
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToStart();
       text->insertText(QString("𝄆"));
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("𝄆𝄏"));
       }
 
@@ -533,14 +535,14 @@ void TestText::testSupplementaryUnicodeInsert1()
 void TestText::testSupplementaryUnicodeInsert2()
       {
       Text* text = new Text(score);
-      text->setTextStyle(score->textStyle(TextStyleType::DYNAMICS));
+      text->initSubStyle(SubStyle::DYNAMICS);
       text->layout();
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToStart();
       text->insertText(QString("𝄏"));
       text->moveCursorToStart();
       text->insertText(QString("𝄆"));
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("𝄆𝄏"));
       }
 
@@ -552,28 +554,28 @@ void TestText::testSupplementaryUnicodeInsert2()
 void TestText::testSupplementaryUnicodePaste()
       {
       Text* text = new Text(score);
-      text->setTextStyle(score->textStyle(TextStyleType::DYNAMICS));
+      text->initSubStyle(SubStyle::DYNAMICS);
       text->setPlainText(QString(""));
       text->layout();
 
       QApplication::clipboard()->setText(QString("𝄏"));
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToStart();
-      text->paste();
-      text->endEdit();
+      text->paste(0);
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("𝄏"));
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToStart();
-      text->paste();
-      text->endEdit();
+      text->paste(0);
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("𝄏𝄏"));
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToEnd();
-      text->paste();
-      text->endEdit();
+      text->paste(0);
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("𝄏𝄏𝄏"));
       }
 
@@ -584,11 +586,11 @@ void TestText::testSupplementaryUnicodePaste()
 void TestText::testRightToLeftWithSupplementaryUnicode()
       {
       Text* text = new Text(score);
-      text->setTextStyle(score->textStyle(TextStyleType::DYNAMICS));
+      text->initSubStyle(SubStyle::DYNAMICS);
       text->setPlainText(QString(""));
       text->layout();
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToStart();
       text->insertText(QString("𝄆"));
       text->insertText(QString("م"));
@@ -598,19 +600,19 @@ void TestText::testRightToLeftWithSupplementaryUnicode()
       text->insertText(QString("ق"));
       text->insertText(QString("ى"));
       text->insertText(QString("𝄇"));
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("𝄆موسيقى𝄇"));
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->cursor()->setColumn(1);
       text->deletePreviousChar();
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("موسيقى𝄇"));
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->cursor()->setColumn(5);
       text->deleteChar();
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("موسيق𝄇"));
       }
 
@@ -621,24 +623,24 @@ void TestText::testRightToLeftWithSupplementaryUnicode()
 void TestText::testPasteSymbolAndSupplemental()
       {
       Text* text = new Text(score);
-      text->setTextStyle(score->textStyle(TextStyleType::DYNAMICS));
+      text->initSubStyle(SubStyle::DYNAMICS);
       text->setPlainText(QString(""));
       text->layout();
 
       QApplication::clipboard()->setText(QString("<sym>gClef</sym>𝄎"));
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToStart();
-      text->paste();
-      text->endEdit();
+      text->paste(0);
+      text->endEdit(editData);
       QVERIFY(text->fragmentList()[0].format.type() == CharFormatType::SYMBOL);
       QVERIFY(text->fragmentList()[1].format.type() == CharFormatType::TEXT);
       QCOMPARE(text->xmlText(), QString("<sym>gClef</sym>𝄎"));
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToStart();
       text->insertText(QString("𝄎"));
-      text->endEdit();
+      text->endEdit(editData);
       QVERIFY(text->fragmentList()[0].format.type() == CharFormatType::TEXT);
       QVERIFY(text->fragmentList()[1].format.type() == CharFormatType::SYMBOL);
       QVERIFY(text->fragmentList()[2].format.type() == CharFormatType::TEXT);
@@ -652,36 +654,36 @@ void TestText::testPasteSymbolAndSupplemental()
 void TestText::testMixedSelectionDelete()
       {
       Text* text = new Text(score);
-      text->setTextStyle(score->textStyle(TextStyleType::DYNAMICS));
+      text->initSubStyle(SubStyle::DYNAMICS);
       text->layout();
       QApplication::clipboard()->setText(QString("[A]𝄎<sym>gClef</sym> 𝄎𝄇"));
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToStart();
-      text->paste();
-      text->endEdit();
+      text->paste(0);
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("[A]𝄎<sym>gClef</sym> 𝄎𝄇"));
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToStart();
       text->cursor()->setSelectColumn(4);
       text->cursor()->setColumn(7);
       text->deleteSelectedText();
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("[A]𝄎𝄇"));
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->cursor()->setColumn(4);
       text->deletePreviousChar();
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("[A]𝄇"));
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToEnd();
       text->insertSym(SymId::segno);
-      text->endEdit();
+      text->endEdit(editData);
       text->insertText(QString("e"));
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("[A]𝄇<sym>segno</sym>e"));
       }
 
@@ -692,41 +694,41 @@ void TestText::testMixedSelectionDelete()
 void TestText::testChineseBasicSupplemental()
       {
       Text* text = new Text(score);
-      text->setTextStyle(score->textStyle(TextStyleType::DYNAMICS));
+      text->initSubStyle(SubStyle::DYNAMICS);
       text->setPlainText(QString(""));
       text->layout();
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->moveCursorToStart();
       text->insertText(QString("你"));  // this is supplemental unicode
       text->insertText(QString("好"));  // this is basic unicode
       text->insertText(QString("。"));
       QApplication::clipboard()->setText(QString("我爱Musescore"));
-      text->paste();
-      text->endEdit();
+      text->paste(0);
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("你好。我爱Musescore"));
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       QApplication::clipboard()->setText(QString("你屠槪真軔")); // some random supplemental unicode
       text->moveCursorToStart();
-      text->paste();
+      text->paste(0);
       text->moveCursorToEnd();
-      text->paste();
-      text->endEdit();
+      text->paste(0);
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("你屠槪真軔你好。我爱Musescore你屠槪真軔"));
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->cursor()->setSelectColumn(4);
       text->cursor()->setColumn(20);
       text->deleteSelectedText();
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("你屠槪真屠槪真軔")); // this is only supplemental
 
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
       text->cursor()->setColumn(4);
       text->deleteChar();
       text->deletePreviousChar();
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("你屠槪槪真軔")); // deleted the two chars in the middle
       }
 
@@ -739,48 +741,48 @@ void TestText::testChineseBasicSupplemental()
 void TestText::testDropUnicodeAfterSMUFLwhenCursorSetToSymbol()
       {
       Text* text = new Text(score);
-      text->setTextStyle(score->textStyle(TextStyleType::DYNAMICS));
+      text->initSubStyle(SubStyle::DYNAMICS);
       text->setPlainText(QString(""));
       text->layout();
-      text->startEdit(0, QPoint());
+      text->startEdit(editData);
 
       Symbol* symbolSMUFL = new Symbol(score); // create a new element, as Measure::drop() will eventually delete it
       symbolSMUFL->setSym(SymId::noteheadWhole);
 
-      DropData dropSMUFL;
-      dropSMUFL.element = symbolSMUFL;
+      EditData dropSMUFL;
+      dropSMUFL.dropElement = symbolSMUFL;
       text->drop(dropSMUFL);
 
       // the bug happened when cursor is in symbol mode
       CharFormat* cf = text->cursor()->format();
       cf->setType(CharFormatType::SYMBOL);
 
-      DropData dropFSymbol;
+      EditData dropFSymbol;
       FSymbol* fsymbol = new FSymbol(score);
       fsymbol->setCode(0x0001D10E); // unicode hex code for '𝄎'
-      dropFSymbol.element = fsymbol;
+      dropFSymbol.dropElement = fsymbol;
       text->drop(dropFSymbol);
 
-      text->endEdit();
+      text->endEdit(editData);
       QCOMPARE(text->xmlText(), QString("<sym>noteheadWhole</sym>𝄎"));
       }
 
 //---------------------------------------------------------
 ///   testDropBasicUnicodeWhenNotInEditMode
-///     Simple test dropping basic unicode, but excercising the path when edit mode not already engaged
+///     Simple test dropping basic unicode, but exercising the path when edit mode not already engaged
 //---------------------------------------------------------
 
 void TestText::testDropBasicUnicodeWhenNotInEditMode()
       {
       Text* text = new Text(score);
-      text->setTextStyle(score->textStyle(TextStyleType::DYNAMICS));
+      text->initSubStyle(SubStyle::DYNAMICS);
       text->setPlainText(QString(""));
       text->layout();
 
-      DropData dropFSymbol;
+      EditData dropFSymbol;
       FSymbol* fsymbol = new FSymbol(score);
       fsymbol->setCode(0x4D); // Basic Unicode code for 'M'
-      dropFSymbol.element = fsymbol;
+      dropFSymbol.dropElement = fsymbol;
       text->drop(dropFSymbol);
 
       QCOMPARE(text->xmlText(), QString("M"));
@@ -788,20 +790,20 @@ void TestText::testDropBasicUnicodeWhenNotInEditMode()
 
 //---------------------------------------------------------
 ///   testDropSupplementaryUnicodeWhenNotInEditMode
-///     Simple test dropping supplementary unicode, but excercising the path when edit mode not already engaged
+///     Simple test dropping supplementary unicode, but exercising the path when edit mode not already engaged
 //---------------------------------------------------------
 
 void TestText::testDropSupplementaryUnicodeWhenNotInEditMode()
       {
       Text* text = new Text(score);
-      text->setTextStyle(score->textStyle(TextStyleType::DYNAMICS));
+      text->initSubStyle(SubStyle::DYNAMICS);
       text->setPlainText(QString(""));
       text->layout();
 
-      DropData dropFSymbol;
+      EditData dropFSymbol;
       FSymbol* fsymbol = new FSymbol(score);
       fsymbol->setCode(0x0001D10E); // Supplementary Unicode code for '𝄎'
-      dropFSymbol.element = fsymbol;
+      dropFSymbol.dropElement = fsymbol;
       text->drop(dropFSymbol);
 
       QCOMPARE(text->xmlText(), QString("𝄎"));
