@@ -38,42 +38,34 @@ inline void initMyResources() {
       Q_INIT_RESOURCE(musescorefonts_Free);
 }
 
-extern Ms::Score::FileError importOve(Ms::Score*, const QString& name);
+extern Ms::Score::FileError importOve(Ms::MasterScore*, const QString& name);
+
+Q_LOGGING_CATEGORY(undoRedo, "undoRedo", QtCriticalMsg)
+// Q_LOGGING_CATEGORY(undoRedo, "undoRedo", QtDebugMsg)
 
 namespace Ms {
 
 #ifdef OMR
-extern Score::FileError importPdf(Score*, const QString&);
+extern Score::FileError importPdf(MasterScore*, const QString&);
 #endif
 
-extern Score::FileError importBB(Score*, const QString&);
-extern Score::FileError importCapella(Score*, const QString&);
-extern Score::FileError importCapXml(Score*, const QString&);
-extern Score::FileError importCompressedMusicXml(Score*, const QString&);
-extern Score::FileError importMusicXml(Score*, const QString&);
-extern Score::FileError importGTP(Score*, const QString&);
+extern Score::FileError importBB(MasterScore*, const QString&);
+extern Score::FileError importCapella(MasterScore*, const QString&);
+extern Score::FileError importCapXml(MasterScore*, const QString&);
+extern Score::FileError importCompressedMusicXml(MasterScore*, const QString&);
+extern Score::FileError importMusicXml(MasterScore*, const QString&);
+extern Score::FileError importGTP(MasterScore*, const QString&);
 extern bool saveXml(Score*, const QString&);
 bool debugMode = false;
 QString revision;
 bool enableTestMode;
 
-Score* score;
+MasterScore* score;
 MasterSynthesizer* synti;
 QString dataPath;
 QIcon* icons[0];
 QString mscoreGlobalShare;
 
-// MuseScoreCore* MuseScoreCore::mscoreCore;
-
-//---------------------------------------------------------
-//   Preferences
-//---------------------------------------------------------
-
-Preferences preferences;
-Preferences::Preferences()
-      {
-      }
-
 //---------------------------------------------------------
 //   writeReadElement
 //    writes and element and reads it back
@@ -86,7 +78,7 @@ Element* MTest::writeReadElement(Element* element)
       //
       QBuffer buffer;
       buffer.open(QIODevice::WriteOnly);
-      Xml xml(&buffer);
+      XmlWriter xml(element->score(), &buffer);
       xml.header();
       element->write(xml);
       buffer.close();
@@ -94,13 +86,10 @@ Element* MTest::writeReadElement(Element* element)
       //
       // read element
       //
-// printf("===read <%s>===\n", element->name());
-// printf("%s\n", buffer.buffer().data());
 
       XmlReader e(buffer.buffer());
       e.readNextStartElement();
       QString tag(e.name().toString());
-// printf("read tag %s\n", qPrintable(tag));
       element = Element::name2Element(e.name(), score);
       element->read(e);
       return element;
@@ -111,6 +100,7 @@ Element* MTest::writeReadElement(Element* element)
 //---------------------------------------------------------
 
 MTest::MTest()
+   : ed(0)
       {
       MScore::testMode = true;
       }
@@ -119,7 +109,7 @@ MTest::MTest()
 //   readScore
 //---------------------------------------------------------
 
-Score* MTest::readScore(const QString& name)
+MasterScore* MTest::readScore(const QString& name)
       {
       QString path = root + "/" + name;
       return readCreatedScore(path);
@@ -129,19 +119,23 @@ Score* MTest::readScore(const QString& name)
 //   readCreatedScore
 //---------------------------------------------------------
 
-Score* MTest::readCreatedScore(const QString& name)
+MasterScore* MTest::readCreatedScore(const QString& name)
       {
-      Score* score = new Score(mscore->baseStyle());
+      MasterScore* score = new MasterScore(mscore->baseStyle());
       QFileInfo fi(name);
       score->setName(fi.completeBaseName());
-//      MScore::testMode = true;
       QString csl  = fi.suffix().toLower();
 
+      ScoreLoad sl;
       Score::FileError rv;
-      if (csl == "cap")
+      if (csl == "cap") {
             rv = importCapella(score, name);
-      else if (csl == "capx")
+            score->setMetaTag("originalFormat", csl);
+            }
+      else if (csl == "capx") {
             rv = importCapXml(score, name);
+            score->setMetaTag("originalFormat", csl);
+            }
       else if (csl == "ove")
             rv = importOve(score, name);
       else if (csl == "sgu")
@@ -161,11 +155,14 @@ Score* MTest::readCreatedScore(const QString& name)
       else
             rv = Score::FileError::FILE_UNKNOWN_TYPE;
 
-
       if (rv != Score::FileError::FILE_NO_ERROR) {
             QWARN(qPrintable(QString("readScore: cannot load <%1> type <%2>\n").arg(name).arg(csl)));
             delete score;
-            return 0;
+            score = 0;
+            }
+      else {
+            for (Score* s : score->scoreList())
+                  s->doLayout();
             }
       return score;
       }
@@ -178,7 +175,7 @@ bool MTest::saveScore(Score* score, const QString& name) const
       {
       QFileInfo fi(name);
 //      MScore::testMode = true;
-      return score->saveFile(fi);
+      return score->Score::saveFile(fi);
       }
 
 //---------------------------------------------------------
@@ -213,6 +210,7 @@ qDebug() << "Running " << cmd << " with arg1: " << compareWith << " and arg2: "
 //   saveCompareScore
 //---------------------------------------------------------
 
+// bool MTest::saveCompareScore(MasterScore* score, const QString& saveName, const QString& compareWith) const
 bool MTest::saveCompareScore(Score* score, const QString& saveName, const QString& compareWith) const
       {
       if (!saveScore(score, saveName))
@@ -224,7 +222,7 @@ bool MTest::saveCompareScore(Score* score, const QString& saveName, const QStrin
 //   saveCompareMusicXMLScore
 //---------------------------------------------------------
 
-bool MTest::saveCompareMusicXmlScore(Score* score, const QString& saveName, const QString& compareWith)
+bool MTest::saveCompareMusicXmlScore(MasterScore* score, const QString& saveName, const QString& compareWith)
       {
       saveMusicXml(score, saveName);
       return compareFiles(saveName, compareWith);
@@ -234,16 +232,17 @@ bool MTest::saveCompareMusicXmlScore(Score* score, const QString& saveName, cons
 //   savePdf
 //---------------------------------------------------------
 
-bool MTest::savePdf(Score* cs, const QString& saveName)
+bool MTest::savePdf(MasterScore* cs, const QString& saveName)
       {
       QPrinter printerDev(QPrinter::HighResolution);
-      const PageFormat* pf = cs->pageFormat();
-      printerDev.setPaperSize(pf->size(), QPrinter::Inch);
+      double w = cs->styleD(Sid::pageWidth);
+      double h = cs->styleD(Sid::pageHeight);
+      printerDev.setPaperSize(QSizeF(w,h), QPrinter::Inch);
 
       printerDev.setCreator("MuseScore Version: " VERSION);
       printerDev.setFullPage(true);
       printerDev.setColorMode(QPrinter::Color);
-      printerDev.setDocName(cs->name());
+//      printerDev.setDocName(cs->name());
       printerDev.setOutputFormat(QPrinter::PdfFormat);
 
       printerDev.setOutputFileName(saveName);
@@ -283,7 +282,7 @@ bool MTest::savePdf(Score* cs, const QString& saveName)
 //   saveMusicXml
 //---------------------------------------------------------
 
-bool MTest::saveMusicXml(Score* score, const QString& saveName)
+bool MTest::saveMusicXml(MasterScore* score, const QString& saveName)
       {
       return saveXml(score, saveName);
       }
@@ -318,6 +317,8 @@ bool MTest::saveCompareMimeData(QByteArray mimeData, const QString& saveName, co
 
 void MTest::initMTest()
       {
+      qputenv("QML_DISABLE_DISK_CACHE", "true");
+      qSetMessagePattern("%{function}: %{message}");
       initMyResources();
 //      DPI  = 120;
 //      PDPI = 120;
@@ -327,11 +328,13 @@ void MTest::initMTest()
       mscore = new MScore;
       new MuseScoreCore;
       mscore->init();
-      preferences.shortestNote = MScore::division / 4; // midi quantization: 1/16
+      ed.init();
+
+      preferences.init(true);
 
       root = TESTROOT "/mtest";
       loadInstrumentTemplates(":/instruments.xml");
-      score = readScore("/test.mscx");
+      score = readScore("test.mscx");
       }
 }
 
