@@ -2,7 +2,6 @@
 //=============================================================================
 //  MuseScore
 //  Music Composition & Notation
-//  $Id:$
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2
@@ -50,7 +49,7 @@ void TestSfzLoop::initTestCase()
       initMTest();
       synth = new Zerberus();
       synth->init(samplerate);
-      Ms::preferences.mySoundfontsPath += ";" + root;
+      preferences.setPreference(PREF_APP_PATHS_MYSOUNDFONTS, root);
       synth->loadInstrument("loopTest.sfz");
       }
 
@@ -60,7 +59,7 @@ void TestSfzLoop::initTestCase()
 
 void TestSfzLoop::testEnvelopesParsing()
       {
-      QCOMPARE(synth->instrument(0)->zones().size(), (unsigned long) 4);
+      QCOMPARE(synth->instrument(0)->zones().size(), (size_t) 4);
 
       std::list<Zone *>::iterator curZone = synth->instrument(0)->zones().begin();
       QCOMPARE((*curZone)->keyLo, (char) 20);
