@@ -2,7 +2,6 @@
 //=============================================================================
 //  MuseScore
 //  Music Composition & Notation
-//  $Id:$
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2
@@ -50,12 +49,12 @@ void TestOpcodes::initTestCase()
 void TestOpcodes::testOpcodes()
       {
       Zerberus* synth = new Zerberus();
-      Ms::preferences.mySoundfontsPath += ";" + root;
+      preferences.setPreference(PREF_APP_PATHS_MYSOUNDFONTS, root);
       synth->loadInstrument("opcodeTest.sfz");
 
       std::list<Zone *>::iterator curZone = synth->instrument(0)->zones().begin();
 
-      QCOMPARE(synth->instrument(0)->zones().size(), (unsigned long) 12);
+      QCOMPARE(synth->instrument(0)->zones().size(), (size_t) 12);
       QCOMPARE((*curZone)->keyLo, (char) 60);
       QCOMPARE((*curZone)->keyHi, (char) 70);
       QCOMPARE((*curZone)->keyBase, (char) 40);
