@@ -18,7 +18,7 @@
 namespace Ms {
 
 class Note;
-class Xml;
+class XmlWriter;
 
 enum class BeatType : char;
 
@@ -99,6 +99,8 @@ enum {
       CTRL_LRPN               = 0x64,
 
       CTRL_MODULATION         = 0x01,
+      CTRL_BREATH             = 0x02,
+      CTRL_FOOT               = 0x04,
       CTRL_PORTAMENTO_TIME    = 0x05,
       CTRL_VOLUME             = 0x07,
       CTRL_PANPOT             = 0x0a,
@@ -179,7 +181,7 @@ class MidiCoreEvent {
       void setData(int t, int a, int b) { _type = t; _a = a; _b = b; }
 
       bool isChannelEvent() const;
-      void write(Xml&) const;
+      void write(XmlWriter&) const;
       bool operator==(const MidiCoreEvent& e) const {
             return e._type == _type && e._channel == _channel && e._a == _a && e._b == _b;
             }
@@ -278,7 +280,7 @@ class Event : public PlayEvent {
       ~Event();
       bool operator==(const Event&) const;
 
-      void write(Xml&) const;
+      void write(XmlWriter&) const;
       void dump() const;
 
 
