@@ -136,6 +136,21 @@ QList<MidiPatch*> MasterSynthesizer::getPatchInfo() const
       return pl;
       }
 
+//---------------------------------------------------------
+//   getPatchInfo
+//---------------------------------------------------------
+
+MidiPatch* MasterSynthesizer::getPatchInfo(QString synti, int bank, int program)
+      {
+      for (Synthesizer* s : _synthesizer) {
+            for (MidiPatch* p: s->getPatchInfo()) {
+                  if (p->synti == synti && p->bank == bank && p->prog == program)
+                        return p;
+                  }
+            }
+      return nullptr;
+      }
+
 //---------------------------------------------------------
 //   allSoundsOff
 //---------------------------------------------------------
@@ -199,8 +214,12 @@ void MasterSynthesizer::setEffect(int ab, int idx)
             return;
             }
       lock2 = true;
-      while (lock1)
-            sleep(1);
+      while(lock1)
+#if (!defined (_MSCVER) && !defined (_MSC_VER))
+         sleep(1);
+#else
+         Sleep(1000);      // MS-equivalent function, time in ms instead of seconds.
+#endif
       _effect[ab] = _effectList[ab][idx];
       lock2 = false;
       }
@@ -320,6 +339,12 @@ bool MasterSynthesizer::setState(const SynthesizerState& ss)
                               case 3:
                                     setMasterTuning(v.data.toDouble());
                                     break;
+                              case 4:
+                                    setDynamicsMethod(v.data.toInt());
+                                    break;
+                              case 5:
+                                    setCcToUseIndex(v.data.toInt());
+                                    break;
                               default:
                                     qDebug("MasterSynthesizer::setState: unknown master id <%d>", v.id);
                               }
@@ -357,6 +382,8 @@ SynthesizerState MasterSynthesizer::state() const
       g.push_back(IdValue(1, QString("%1").arg(_effect[1] ? _effect[1]->name() : "NoEffect")));
       g.push_back(IdValue(2, QString("%1").arg(gain())));
       g.push_back(IdValue(3, QString("%1").arg(masterTuning())));
+      g.push_back(IdValue(4, QString("%1").arg(dynamicsMethod())));
+      g.push_back(IdValue(5, QString("%1").arg(ccToUseIndex())));
       ss.push_back(g);
       for (Synthesizer* s : _synthesizer)
             ss.push_back(s->state());
@@ -379,9 +406,11 @@ bool MasterSynthesizer::storeState()
             qDebug("cannot write synthesizer settings <%s>", qPrintable(s));
             return false;
             }
-      Xml xml(&f);
+      XmlWriter xml(0, &f);
       xml.header();
-      state().write(xml);
+      // force the write, since the msynth state is created when state() is called and so will
+      // automatically have _isDefault = true, when in fact we need to write the state here, default or not
+      state().write(xml, true);
       return true;
       }
 
@@ -407,5 +436,4 @@ void MasterSynthesizer::setMasterTuning(double val)
       for (Synthesizer* s : _synthesizer)
             s->setMasterTuning(_masterTuning);
       }
-}
-
+} // namespace Ms
