@@ -93,6 +93,7 @@ struct SfzRegion {
       int pan;    // [-100, 100]
       long long offset; // [0, 4Gb) or [0, 4294967295]
       float group_volume; // [-144 to 6] (dB)
+      float global_volume; // [-144 to 6] (dB)
 
       //filters
       bool isCutoffDefined;
@@ -166,6 +167,8 @@ void SfzRegion::init(const QString& _path)
       pan = 0;
       offset = 0;
       group_volume = 0.0f;
+      global_volume = 0.0f;
+
       isCutoffDefined = false;
       cutoff = 0;
       fil_keytrack = 0;
@@ -229,6 +232,7 @@ void SfzRegion::setZone(Zone* z) const
       z->delay        = delay * 1000; //convert seconds from sfz to ms for computations
       z->pan          = pan;
       z->group_volume = pow(10.0, group_volume / 20.0); //dB -> volume multiplier
+      z->global_volume = pow(10.0, global_volume / 20.0); //dB -> volume multiplier
 
       z->isCutoffDefined = isCutoffDefined;
       z->cutoff = cutoff;
@@ -316,7 +320,7 @@ void ZInstrument::addRegion(SfzRegion& r)
 static void readLongLong(const QString& data, long long& val)
       {
       bool ok;
-      float d = data.toLongLong(&ok);
+      long long d = data.toLongLong(&ok);
       if (ok)
             val = d;
       }
@@ -550,6 +554,8 @@ void SfzRegion::readOp(const QString& b, const QString& data, SfzControl &c)
             readLongLong(opcode_data, offset);
       else if (opcode == "group_volume")
             readFloat(opcode_data, group_volume);
+      else if (opcode == "global_volume")
+            readFloat(opcode_data, global_volume);
       else if (opcode == "cutoff") {
             isCutoffDefined = true;
             readFloat(opcode_data, cutoff);
@@ -601,15 +607,15 @@ bool ZInstrument::loadSfz(const QString& s)
       for (int i = 0;i < 128; i++)
             c.set_cc[i] = -1;
 
-      int idx = 0;
+      int idx0 = 0;
       bool inBlockComment = false;
       // preprocessor
-      while(idx < fileContents.size()) {
+      while(idx0 < fileContents.size()) {
             QRegularExpression findWithSpaces("\"(.+)\"");
             QRegularExpression comment("//.*$");
             QRegularExpression trailingSpacesOrTab("^[\\s\\t]*");
             QRegularExpressionMatch foundWithSpaces;
-            QString curLine = fileContents[idx];
+            QString curLine = fileContents[idx0];
             QString curLineCopy = curLine;
             bool nextIsImportant = false;
             int idxBlockComment = 0;
@@ -646,7 +652,7 @@ bool ZInstrument::loadSfz(const QString& s)
 
             curLine = curLine.remove(comment);
             curLine.remove(trailingSpacesOrTab);
-            fileContents[idx] = curLine;
+            fileContents[idx0] = curLine;
 
             if (curLine.startsWith("#define")) {
                   QStringList define = curLine.split(" ");
@@ -655,7 +661,7 @@ bool ZInstrument::loadSfz(const QString& s)
                         c.defines.insert(std::pair<QString, QString>(define[1], define[2]));
                   else if(foundWithSpaces.hasMatch())
                         c.defines.insert(std::pair<QString, QString>(define[1], foundWithSpaces.captured(1)));
-                  fileContents.removeAt(idx);
+                  fileContents.removeAt(idx0);
                   }
             else if (curLine.startsWith("#include")) {
                   foundWithSpaces = findWithSpaces.match(curLine);
@@ -672,17 +678,17 @@ bool ZInstrument::loadSfz(const QString& s)
 
                         int offset = 1;
                         for (QString newFileLine : newFileContents) {
-                              fileContents.insert(idx+offset, newFileLine);
+                              fileContents.insert(idx0+offset, newFileLine);
                               offset++;
                               }
 
-                        fileContents.removeAt(idx);
+                        fileContents.removeAt(idx0);
                         }
                   }
             else if (curLine.isEmpty())
-                  fileContents.removeAt(idx);
+                  fileContents.removeAt(idx0);
             else
-                  idx++;
+                  idx0++;
             }
 
       int total = fileContents.size();
@@ -697,9 +703,9 @@ bool ZInstrument::loadSfz(const QString& s)
       bool globMode = false;
       zerberus->setLoadProgress(0);
 
-      for (int idx = 0; idx < fileContents.size(); idx++) {
-            QString curLine = fileContents[idx];
-            zerberus->setLoadProgress(((qreal) idx * 100) /  (qreal) total);
+      for (int idx1 = 0; idx1 < fileContents.size(); idx1++) {
+            QString curLine = fileContents[idx1];
+            zerberus->setLoadProgress(((qreal) idx1 * 100) /  (qreal) total);
 
             if (zerberus->loadWasCanceled())
                   return false;
@@ -720,7 +726,7 @@ bool ZInstrument::loadSfz(const QString& s)
                         globMode = false;
                         }
                   else {
-                        r = glob; // initalize group with global values
+                        r = glob; // initialize group with global values
                         }
                   groupMode = true;
                   curLine = curLine.mid(7);
@@ -758,8 +764,8 @@ bool ZInstrument::loadSfz(const QString& s)
                         }
                   else
                         ei = curLine.size();
-                  QString s = curLine.mid(si, ei-si);
-                  r.readOp(match.captured(1), s, c);
+                  QString str = curLine.mid(si, ei-si);
+                  r.readOp(match.captured(1), str, c);
                   }
             }
 
